#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "callback.h"
#include "dialogs.h"
#include "support.h"
#include "main.h"
#include "custom.h"
#include "misc.h"

// =========================================================

void dlg_destroy (GtkWidget *widget, gpointer data);
int validate_fields (GtkWidget *dlg);
void restore_dialog_data (int usedefault);
void save_dialog_data ();
void on_dlg_ok (GtkWidget *widget, gpointer user_data);

#define GetRValue(rgb)  ((guint8)(rgb))
#define GetGValue(rgb)  ((guint8)(((guint16)(rgb)) >> 8))
#define GetBValue(rgb)  ((guint8)((rgb)>>16))  
#define HTML_PREVIW_PAGE    "preview.html"

static int IsTwoPower (unsigned int uiNum)
{
  for(;;)
  {
    uiNum = uiNum << 1;
    if(uiNum == 0x80000000 )
      return 1;
    if(uiNum > 0x80000000 || uiNum == 0)
      return 0;
  }
}

// =========================================================
// Menu callbacks

void on_open_project_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  OpenProject(NULL);
}

void on_save_project_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  if (SaveProject() == 0)
    message_box(_("Problems during the saving of project"), 0);
}

void on_save_all_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  char* str;

  file_browse ("HTML", &str);

  if (str != NULL)
  {
    Publish(current_applet, str);
    g_free (str);
  }
}

void on_exit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  gtk_widget_destroy (main_dlg);
}

static void browser_find (GtkMenuItem *menuitem, gpointer user_data)
{
  char* str;

  file_browse ("Find", &str);

  if (str != NULL)
  {
    gtk_entry_set_text (GTK_ENTRY (user_data), str);
    g_free (str);
  }
}

void on_change_browser_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  GtkWidget *browser;
  GtkWidget *vbox, *hbox;
  GtkWidget *entry, *button;
  char buf[1024];
  int ret;

  GetSavedVar(INI_APP_SECTION, INI_BROWSER, buf);

  browser = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (browser), _("Change preview browser"));
  gtk_window_set_default_size (GTK_WINDOW (browser), 350, 80);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (browser), vbox);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0);

  entry = gtk_entry_new ();
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);

  gtk_entry_set_text (GTK_ENTRY (entry), buf);

  button = gtk_button_new_with_label (_("Find..."));
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  gtk_widget_set_usize (button, 75, -2);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc)browser_find, entry);

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0);

  button = gtk_button_new_with_label (_("OK"));
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  gtk_widget_set_usize (button, 75, 22);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc)on_dlg_ok, &ret);

  button = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  gtk_widget_set_usize (button, 75, -2);

  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (browser));

  gtk_widget_show (browser);
  gtk_grab_add (browser);
  gtk_main ();

  if (ret == 1)
  {
    SaveVar(INI_APP_SECTION, INI_BROWSER, gtk_entry_get_text (GTK_ENTRY (entry)));
  }

  gtk_grab_remove (browser);
  gtk_widget_destroy (browser);
}

void on_insert_regcode_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  /*
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
  */
}

void on_applet_info_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  char* str;

  if (current_dialog == DLG_STEP0)
    str = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "applet_entry")));
  else
    str = applet_info[current_applet].name;

  ShowHelp (str);
}

void on_instructions_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ShowHelp(TOPIC_INSTRUCTION);
}

void on_tech_faq_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ShowHelp(TOPIC_TECHFAQ);
}

void on_scrolltext_parameters_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ShowHelp(TOPIC_SCRTEXT_HELP);
}

void on_home_page_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  StartBrowser(_("http://www.anfyteam.com"));
}

void on_registration_info_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ShowHelp(TOPIC_REG_HELP);
}

void on_online_registration_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  StartBrowser(_("http://www.anfyteam.com/purchasean.html"));
}

void on_copyright_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  ShowHelp(TOPIC_COPYRIGHT_HELP);
}

void on_about_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  GtkWidget *dlg, *hbox, *vbox, *w;

  dlg = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (dlg), "About Anfy");
  gtk_container_border_width (GTK_CONTAINER (dlg), 10);

  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
		      GTK_SIGNAL_FUNC (dlg_destroy), NULL);

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (dlg), hbox);
  gtk_widget_show (hbox);

  vbox = gtk_vbox_new (TRUE, 5);
  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
  gtk_widget_show (vbox);

  w = gtk_label_new ("Anfy "VERSION);
  gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 3);
  gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
  gtk_widget_show (w);

  w = gtk_label_new ("Copyright (c) 2000 Fabio Ciucci");
  gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 3);
  gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
  gtk_widget_show (w);

  w = gtk_label_new ("Linux editor by Leonardo Zide");
  gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 3);
  gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
  gtk_widget_show (w);

  w = gtk_label_new ("email bugs to leonardo@centroin.com.br");
  gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 3);
  gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
  gtk_widget_show (w);

  vbox = gtk_vbox_new (TRUE, 5);
  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
  gtk_widget_show (vbox);

  w = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (w), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (dlg));
  GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (w);
  gtk_widget_show (w);

  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_main ();
}

// =========================================================
// Navigation buttons callbacks

void on_applet_default (GtkWidget *widget, gpointer user_data)
{
  restore_dialog_data (1);
}

void on_applet_preview (GtkWidget *widget, gpointer user_data)
{
  char strCommandLine[1024];
  int applet;

  applet = current_applet;

  GetAppletPath(applet, strCommandLine);
  strcat (strCommandLine, "/");

  if (current_dialog == DLG_STEP0)
    strcat (strCommandLine, GetDemoFile(applet));
  else
    strcat (strCommandLine, HTML_PREVIW_PAGE);
		
  StartBrowser(strCommandLine);
}

void on_previous_clicked (GtkButton *button, gpointer user_data)
{
  int prev = DLG_STEP0;

  save_dialog_data();

  switch (current_applet)
  {
  case APPLET_ANBLUR    : prev = DLG_ANBLUR;     break;
  case APPLET_ANFYBUMP  : prev = DLG_ANFYBUMP;   break;
  case APPLET_ANFADE    : prev = DLG_ANFADE;     break;
  case APPLET_ANFLAG    : prev = DLG_ANFLAG;     break;
  case APPLET_ANLENS    : prev = DLG_ANLENS;     break;
  case APPLET_BLOBS     : prev = DLG_BLOBS;      break;
  case APPLET_CUBEMENU  : prev = DLG_CUBEMENU;   break;
  case APPLET_DEFORM    : prev = DLG_DEFORM;     break;
  case APPLET_FIRE      : prev = DLG_FIRE;       break;
  case APPLET_FLAGLOAD  : prev = DLG_FLAGLOAD;   break;
  case APPLET_FLOZOIDS  : prev = DLG_FLOZOIDS;   break;
  case APPLET_FLUID     : prev = DLG_FLUID;      break;
  case APPLET_HUEROT    : prev = DLG_HUEROT;     break;
  case APPLET_ANFRACT   : prev = DLG_ANFRACT;    break;
  case APPLET_ANLAKE    : prev = DLG_ANLAKE;     break;
  case APPLET_ANFYCAM   : prev = DLG_ANFYCAM;    break;
  case APPLET_LIFE2D    : prev = DLG_LIFE2D;     break;
  case APPLET_MANDEL    : prev = DLG_MANDEL;     break;
  case APPLET_PLASMA    : prev = DLG_PLASMA;     break;
  case APPLET_ROTATE    : prev = DLG_ROTATE;     break;
  case APPLET_TMAPCUBE  : prev = DLG_TMAPCUBE;   break;
  case APPLET_ANTUNNEL  : prev = DLG_TUNNEL;     break;
  case APPLET_TUNNEL3D  : prev = DLG_TUNNEL3D;   break;
  case APPLET_ANVOXEL   : prev = DLG_VOXEL;      break;
  case APPLET_WARP      : prev = DLG_WARP;       break;
  case APPLET_ANWATER   : prev = DLG_ANWATER;    break;
  case APPLET_WORMHOLE  : prev = DLG_WORMHOLE;   break;
  case APPLET_GALAXY    : prev = DLG_GALAXY;     break;
  case APPLET_BOOKFLIP  : prev = DLG_BOOKFLIP;   break;
  case APPLET_MOSAIC    : prev = DLG_MOSAIC;     break;
  case APPLET_TSCROLLER : prev = DLG_TSCROLLER;  break;
  case APPLET_WOBBLER   : prev = DLG_WOBBLER;    break;
  case APPLET_ANFY3D    : prev = DLG_ANFY3D_0;   break;
  case APPLET_WHEEL     : prev = DLG_WHEEL;	 break;
  case APPLET_TREEMENU  : prev = DLG_TREEMENU;   break;
  case APPLET_MORPHMENU :
    if (dlg_plugin < 0)
      prev = DLG_MORPHMENU;
    else if (dlg_plugin == 0)
      prev = DLG_MM_GALAXY;
    else if (dlg_plugin == 1)
      prev = DLG_MM_BLOBS;
    break;
  case APPLET_ANFYPAINT : prev = DLG_ANFYPAINT;  break;
  case APPLET_SOLIDSCROLLER : prev = DLG_ANFYFONT; break;
  case APPLET_ANFYSNOW  : prev = DLG_ANFYSNOW;   break;
  case APPLET_FIREWORKS : prev = DLG_FIREWORKS;  break;
  default:
    prev = DLG_STEP0;
  }

  switch (current_dialog)
  {
  case DLG_STEP1:
    current_dialog = DLG_REGISTRATION;
    break;
  case DLG_REGISTRATION:
    if (current_applet != APPLET_ANFYPAINT)
      current_dialog = DLG_EXPERTMODE;
    break;
  case DLG_EXPERTMODE:
    if ((dlg_showtextstyle) &&
	(current_applet != APPLET_ANBLUR) &&
	(current_applet != APPLET_ANFADE) &&
	(current_applet != APPLET_ANLENS) &&
	(current_applet != APPLET_MANDEL) &&
	(current_applet != APPLET_MOSAIC) &&
	(current_applet != APPLET_BOOKFLIP) &&
	(current_applet != APPLET_WHEEL) &&
	(current_applet != APPLET_MORPHMENU) && 
	(current_applet != APPLET_TREEMENU) &&
	(current_applet != APPLET_ANFYCAM) &&
	(current_applet != APPLET_ANFYPAINT) &&
	(current_applet != APPLET_SOLIDSCROLLER))
      current_dialog = DLG_TEXTSTYLE;  
    else
    {
      if (dlg_showfloat)
	current_dialog = DLG_CONF_MENU_2;
      else
	current_dialog = prev;
    }
    break;
  case DLG_TEXTSTYLE:
    if (dlg_showfloat)
      current_dialog = DLG_CONF_MENU_2;
    else
      current_dialog = prev;
    break;
  case DLG_ANFY3D_0:
    current_dialog = DLG_ANFY3D;
    break;
  case DLG_ANFYFONT:
    current_dialog = DLG_SOLIDSCROLLER;
    break;
  case DLG_MM_GALAXY:
  case DLG_MM_BLOBS:
    current_dialog = DLG_MORPHMENU;
    break;
  case DLG_WHEEL:
  case DLG_MORPHMENU:
  case DLG_TREEMENU:
    current_dialog = DLG_CONF_MENU;
    break;
  case DLG_CONF_MENU_2:
    current_dialog = prev;
    break;
  case DLG_ANBLUR:
  case DLG_ANFYBUMP:
  case DLG_ANFADE:
  case DLG_ANFLAG:
  case DLG_BLOBS:
  case DLG_CUBEMENU:
  case DLG_DEFORM:
  case DLG_FIRE:
  case DLG_FLAGLOAD:
  case DLG_FLOZOIDS:
  case DLG_FLUID:
  case DLG_HUEROT:
  case DLG_ANFRACT:
  case DLG_ANLAKE:
  case DLG_ANLENS:
  case DLG_LIFE2D:
  case DLG_MANDEL:
  case DLG_PLASMA:
  case DLG_ROTATE:
  case DLG_TMAPCUBE:
  case DLG_TUNNEL:
  case DLG_TUNNEL3D:
  case DLG_VOXEL:
  case DLG_WARP:
  case DLG_ANWATER:
  case DLG_WORMHOLE:
  case DLG_GALAXY:
  case DLG_BOOKFLIP:
  case DLG_MOSAIC:
  case DLG_WOBBLER:
  case DLG_ANFY3D:
  case DLG_CONF_MENU:
  case DLG_TSCROLLER:
  case DLG_ANFYCAM:
  case DLG_ANFYPAINT:
  case DLG_SOLIDSCROLLER:
  case DLG_ANFYSNOW:
  case DLG_FIREWORKS:
    current_dialog = DLG_STEP0;
    break;
  }

  change_dialog ();
}

void on_next_clicked (GtkButton *button, gpointer user_data)
{
  if (!validate_fields (GTK_WIDGET (button)->parent))
    return;

  save_dialog_data();

  switch (current_dialog)
  {
    case DLG_STEP0:
    {
      switch (current_applet)
      {
      case APPLET_ANBLUR: current_dialog = DLG_ANBLUR; break;
      case APPLET_ANFADE: current_dialog = DLG_ANFADE; break;
      case APPLET_ANFLAG: current_dialog = DLG_ANFLAG; break;
      case APPLET_ANFYBUMP: current_dialog = DLG_ANFYBUMP; break;
      case APPLET_ANFRACT: current_dialog = DLG_ANFRACT; break;
      case APPLET_ANLAKE: current_dialog = DLG_ANLAKE; break;
      case APPLET_ANFYCAM: current_dialog = DLG_ANFYCAM; break;
      case APPLET_ANLENS: current_dialog = DLG_ANLENS; break;
      case APPLET_ANTUNNEL: current_dialog = DLG_TUNNEL; break;
      case APPLET_ANVOXEL: current_dialog = DLG_VOXEL; break;
      case APPLET_ANWATER: current_dialog = DLG_ANWATER; break;
      case APPLET_BLOBS: current_dialog = DLG_BLOBS; break;
      case APPLET_CUBEMENU: current_dialog = DLG_CUBEMENU; break;
      case APPLET_DEFORM: current_dialog = DLG_DEFORM; break;
      case APPLET_FIRE: current_dialog = DLG_FIRE; break;
      case APPLET_FLAGLOAD: current_dialog = DLG_FLAGLOAD; break;
      case APPLET_FLOZOIDS: current_dialog = DLG_FLOZOIDS; break;
      case APPLET_FLUID : current_dialog = DLG_FLUID; break;
      case APPLET_HUEROT: current_dialog = DLG_HUEROT; break;
      case APPLET_LIFE2D: current_dialog = DLG_LIFE2D; break;
      case APPLET_MANDEL: current_dialog = DLG_MANDEL; break;
      case APPLET_PLASMA: current_dialog = DLG_PLASMA; break;
      case APPLET_ROTATE: current_dialog = DLG_ROTATE; break;
      case APPLET_TMAPCUBE: current_dialog = DLG_TMAPCUBE; break;
      case APPLET_TUNNEL3D: current_dialog = DLG_TUNNEL3D; break;
      case APPLET_WARP: current_dialog = DLG_WARP; break;
      case APPLET_WORMHOLE: current_dialog = DLG_WORMHOLE; break;
      case APPLET_GALAXY: current_dialog = DLG_GALAXY; break;
      case APPLET_BOOKFLIP: current_dialog = DLG_BOOKFLIP; break;
      case APPLET_MOSAIC: current_dialog = DLG_MOSAIC; break;
      case APPLET_TSCROLLER: current_dialog = DLG_TSCROLLER; break;
      case APPLET_WOBBLER: current_dialog = DLG_WOBBLER; break;
      case APPLET_ANFY3D: current_dialog = DLG_ANFY3D; break;
      case APPLET_WHEEL: current_dialog = DLG_CONF_MENU; break;
      case APPLET_TREEMENU: current_dialog = DLG_CONF_MENU; break;
      case APPLET_MORPHMENU: current_dialog = DLG_CONF_MENU; break;
      case APPLET_ANFYPAINT: current_dialog = DLG_ANFYPAINT; break;
      case APPLET_SOLIDSCROLLER: current_dialog = DLG_SOLIDSCROLLER; break;
      case APPLET_ANFYSNOW: current_dialog = DLG_ANFYSNOW; break;
      case APPLET_FIREWORKS: current_dialog = DLG_FIREWORKS; break;
      }
    } break;

  case DLG_ANFY3D:
    current_dialog = DLG_ANFY3D_0;
    break;
  case DLG_SOLIDSCROLLER:
    current_dialog = DLG_ANFYFONT;
    break;
  case DLG_CONF_MENU:
    if (current_applet == APPLET_WHEEL)
      current_dialog = DLG_WHEEL;
    else if (current_applet == APPLET_TREEMENU)
      current_dialog = DLG_TREEMENU;
    else if (current_applet == APPLET_MORPHMENU)
      current_dialog = DLG_MORPHMENU;
    break;
  case DLG_TEXTSTYLE:
    current_dialog = DLG_EXPERTMODE;
    break;
  case DLG_CONF_MENU_2:
    if (current_applet == APPLET_CUBEMENU)
    {
      if (dlg_showtextstyle)
	current_dialog = DLG_TEXTSTYLE;  
      else
	current_dialog = DLG_EXPERTMODE;
    }
    else
      current_dialog = DLG_EXPERTMODE;
    break;
  case DLG_EXPERTMODE:
    current_dialog = DLG_REGISTRATION;
    break;
  case DLG_REGISTRATION:
    current_dialog = DLG_STEP1;
    break;
  case DLG_ANBLUR:
  case DLG_ANFADE:
  case DLG_ANLENS:
  case DLG_MANDEL:
  case DLG_BOOKFLIP:
  case DLG_MOSAIC:
  case DLG_ANFYCAM:
  case DLG_ANFYFONT:
    current_dialog = DLG_EXPERTMODE;
    break;
  case DLG_ANFYPAINT:
    current_dialog = DLG_REGISTRATION;
    break;
  case DLG_WHEEL:
  case DLG_TREEMENU:
  case DLG_MM_GALAXY:
  case DLG_MM_BLOBS:
    if (dlg_showfloat)
      current_dialog = DLG_CONF_MENU_2;
    else
      current_dialog = DLG_EXPERTMODE;
    break;
  case DLG_MORPHMENU:
    if (dlg_plugin == 0)
      current_dialog = DLG_MM_GALAXY;
    else if (dlg_plugin == 1)
      current_dialog = DLG_MM_BLOBS;
    else
    {
      if (dlg_showfloat)
	current_dialog = DLG_CONF_MENU_2;
      else
	current_dialog = DLG_EXPERTMODE;
    }
    break;
  case DLG_CUBEMENU:
    if (dlg_showfloat)
      current_dialog = DLG_CONF_MENU_2;
    else
      {
	if (dlg_showtextstyle)
	  current_dialog = DLG_TEXTSTYLE;  
	else
	  current_dialog = DLG_EXPERTMODE;
      }
    break;
  case DLG_ANFYBUMP:
  case DLG_ANFLAG:
  case DLG_BLOBS:
  case DLG_DEFORM:
  case DLG_FIRE:
  case DLG_FLAGLOAD:
  case DLG_FLOZOIDS:
  case DLG_FLUID:
  case DLG_HUEROT:
  case DLG_ANFRACT:
  case DLG_ANLAKE:
  case DLG_LIFE2D:
  case DLG_PLASMA:
  case DLG_ROTATE:
  case DLG_TMAPCUBE:
  case DLG_TUNNEL:
  case DLG_TUNNEL3D:
  case DLG_VOXEL:
  case DLG_WARP:
  case DLG_ANWATER:
  case DLG_WORMHOLE:
  case DLG_GALAXY:
  case DLG_TSCROLLER:
  case DLG_WOBBLER:
  case DLG_ANFY3D_0:
  case DLG_ANFYSNOW:
  case DLG_FIREWORKS:
    if (dlg_showtextstyle)
      current_dialog = DLG_TEXTSTYLE;  
    else
      current_dialog = DLG_EXPERTMODE;
    break;

  case DLG_STEP1:
    info_dlg (_("To add this effect to your html page, you must copy the .class files to the same dir as the .html file, along with any GIF and JPG images.\nThis can be done with the \"Copy all files to...\" button: you must choose a name for the html file, and a path where save it, along with all the .class files and needed images. We suggest to save in a new folder.\nAdding the <applet>...</applet> tag to your html page, without the needed .class files, will not work.\n"), _("For more details, read instructions."), _("Publish"));
    return;
    break;
  }

  change_dialog ();
}

void on_exit_clicked (GtkButton *button, gpointer user_data)
{
  if (message_box (_("Do you really want to exit?"), 1) == 1)
    gtk_widget_destroy (main_dlg);
}

// =========================================================
// Callbacks used by more than 1 dialog

// 'Resolution' combobox changed
void on_resolution_combo_changed (GtkWidget *widget, gpointer user_data)
{
  char* text, buf[16];
  int w, h, r;

  if ((current_applet != APPLET_BLOBS) &&
      (current_applet != APPLET_CUBEMENU) &&
      (current_applet != APPLET_TMAPCUBE) &&
      (current_applet != APPLET_ANFLAG) &&
      (current_applet != APPLET_FLAGLOAD) &&
      (current_applet != APPLET_WOBBLER))
  {
    text = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "width")));
    w = strtol (text, NULL, 10);
    text = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "height")));
    h = strtol (text, NULL, 10);
    text = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "resolution_entry")));
    r = strtol (text, NULL, 10);

    if (current_applet == APPLET_BOOKFLIP)
      h = h - dlg_oldextrah;

    if (dlg_oldres != 0)
    {
      sprintf (buf, "%d", (int)(w/dlg_oldres*r));
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), buf);
      sprintf (buf, "%d", (int)(h/dlg_oldres*r));
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
    }

    dlg_oldres = r;

    if (current_applet == APPLET_BOOKFLIP)
    {
      int e;

      text = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "extra_height")));
      e = strtol (text, NULL, 10);
      text = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "height")));
      h = strtol (text, NULL, 10);

      sprintf (buf, "%d", h+e);
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
    }

    /*
    if (current_dialog == DLG_ANLAKE)
      OnHalfreflect();
    */
  }
}

void on_image_list_add (GtkWidget *widget, gpointer user_data)
{
  unsigned int iWidth, iHeight, iW,iH;
  char strWorkDir[1024], strItem[1024], szImgFormat[10], buf[16];
  int iLength, iCount, index;
  char* filename, *ptr, *row[2] = { NULL, NULL };
  char szSourcePath[1024], szFileName[1024];
  IMAGE_INFO *info;
  GtkCList *list;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(current_applet, strWorkDir);
  iLength = strlen (strWorkDir);

  list = GTK_CLIST (lookup_widget (main_dlg, "image_list"));
  iCount = list->rows;
  if (iCount > 0) 
  {
    gtk_clist_get_text (list, 0, 0, &ptr);

    strcpy (strItem, strWorkDir);
    strcat (strItem, "/");
    strcat (strItem, ptr);

    if( !GetFormatAndSize( strItem, szImgFormat, &iW, &iH) )
    {
      g_free (filename);
      return;
    }
  }

  if (GetFormatAndSize(filename, szImgFormat, &iWidth, &iHeight))
  {
    if ((current_dialog == DLG_CUBEMENU) || (current_dialog == DLG_TMAPCUBE))
    {
      if (!(iWidth == 128 && iHeight == 128)
	  && !(iWidth == 256 && iHeight == 256))
	{
	  message_box (_("Error ! Images width and height must be 128*128 or 2"), 0);
	  g_free (filename);
	  return;
	}
    }

    if (iCount <=0)
    {
      iW = iWidth;
      iH = iHeight;
    }
    else if ((iWidth != iW) || (iHeight != iH))
    {
      message_box(_("Error ! All images width and height must be the same"), 0);
      g_free (filename);
      return;
    }

    PathFromFile(filename, szSourcePath);
    FileFromPath(filename, szFileName);
    if( !MyCopyFile(szSourcePath, strWorkDir, szFileName ) )
    {
      message_box(_("Error ! Problem when copying file in applet directory"), 0);
      g_free (filename);
      return;
    }

    row[0] = szFileName;
    index = gtk_clist_append (list, row);

    info = g_malloc (sizeof (IMAGE_INFO));
    gtk_clist_set_row_data (list, index, info);
    image_info = g_list_append (image_info, info);
    info->image = g_strdup (szFileName);
    info->link = g_strdup ("");
    info->status = g_strdup ("");
    info->flip = g_strdup ("");

    list->focus_row = index;
    gtk_clist_select_row (list, index, 0);

    if (list->rows == 6 
	&& current_dialog != DLG_ANFADE 
	&& current_dialog != DLG_BOOKFLIP
	&& current_dialog != DLG_MOSAIC)
      gtk_widget_set_sensitive (lookup_widget (main_dlg, "add"), FALSE);
  }

  if ((current_dialog != DLG_CUBEMENU) && (current_dialog != DLG_TMAPCUBE))
  {
    if (current_dialog == DLG_BOOKFLIP)
      iHeight += dlg_oldextrah;

    sprintf (buf, "%d", iWidth);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), buf);
    sprintf (buf, "%d", iHeight);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
    dlg_oldres=1;
  }
  
  /*
  if (current_dialog == DLG_MOSAIC)
    OnKillfocusTilesheight();

    OnSelchangeCboRes();
  */
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "remove"), TRUE);
  g_free (filename);
}

void on_image_list_remove (GtkWidget *widget, gpointer user_data)
{
  GtkCList* clist;
  IMAGE_INFO* info;
  int focus;

  clist = GTK_CLIST (lookup_widget (main_dlg, "image_list"));
  focus = clist->focus_row;

  if (focus >= 0)
  {
    info = gtk_clist_get_row_data (clist, focus);
    image_info = g_list_remove (image_info, info);
    free_image_info (info);
    gtk_clist_remove (clist, focus);

    if (focus >= clist->rows)
      focus = clist->rows - 1;
    clist->focus_row = focus;
    gtk_clist_select_row (clist, focus, 0);

    if (clist->rows == 0)
      on_image_list_select (clist, -1, 0, NULL);
  }

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "add"), TRUE);
}

void on_image_list_up (GtkWidget *widget, gpointer user_data)
{
  GtkCList* clist;
  int focus;

  clist = GTK_CLIST (lookup_widget (main_dlg, "image_list"));
  focus = clist->focus_row;

  if (clist->focus_row > 0)
  {
    gtk_clist_swap_rows (clist, focus, focus-1);
    gtk_clist_select_row (clist, focus-1, 0);
    clist->focus_row = focus-1;
  }
}

void on_image_list_down (GtkWidget *widget, gpointer user_data)
{
  GtkCList* clist;
  int focus;

  clist = GTK_CLIST (lookup_widget (main_dlg, "image_list"));
  focus = clist->focus_row;

  if (focus < (clist->rows-1))
  {
    gtk_clist_swap_rows (clist, focus, focus+1);
    gtk_clist_select_row (clist, focus+1, 0);
    clist->focus_row = focus+1;
  }
}

void on_image_list_select (GtkCList *clist, gint row, gint column, GdkEvent *event)
{
  gboolean enable = TRUE;
  GtkWidget* widget;
  IMAGE_INFO* info;

  if (row >= 0)
  {
    if (current_dialog != DLG_TMAPCUBE)
    {
      info = gtk_clist_get_row_data (clist, row);

      widget = lookup_widget (main_dlg, "image_link");
      gtk_entry_set_text (GTK_ENTRY (widget), info->link);
      gtk_object_set_data (GTK_OBJECT (widget), "save_text", &info->link);
      widget = lookup_widget (main_dlg, "image_status");
      gtk_entry_set_text (GTK_ENTRY (widget), info->status);
      gtk_object_set_data (GTK_OBJECT (widget), "save_text", &info->status);

      if (current_dialog == DLG_BOOKFLIP)
      {
	widget = lookup_widget (main_dlg, "flip_mode_entry");
	gtk_object_set_data (GTK_OBJECT (widget), "save_text", &info->flip);
	gtk_entry_set_text (GTK_ENTRY (widget), info->flip);
      }
    }
  }
  else
  {
    if (current_dialog != DLG_TMAPCUBE)
    {
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image_link")), "");
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image_status")), "");
    }

    enable = FALSE;
  }

  if (current_dialog != DLG_TMAPCUBE)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "image_link"), enable);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "image_status"), enable);
  }

  if (current_dialog == DLG_BOOKFLIP)
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "flip_mode_combo"), enable);

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "up"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "down"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "remove"), enable);
}

gint on_edit_focus_out (GtkWidget *widget, GdkEventFocus *event)
{
  char** save;

  save = gtk_object_get_data (GTK_OBJECT (widget), "save_text");

  if (save != NULL)
  {
    g_free (*save);
    *save = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
  }

  return FALSE;
}

void on_combo_changed (GtkWidget *widget, gpointer user_data)
{
  char** save;

  save = gtk_object_get_data (GTK_OBJECT (widget), "save_text");

  if (save != NULL)
  {
    g_free (*save);
    *save = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
  }
}

void on_textscroll (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    dlg_showtextstyle = 0;
  else
    dlg_showtextstyle = 1;

  if (current_dialog == DLG_FIREWORKS)
  {
    if (dlg_showtextstyle)
    {
      gtk_widget_set_sensitive (lookup_widget (main_dlg, "over_text"), TRUE);
    }
    else
    {
      gtk_widget_set_sensitive (lookup_widget (main_dlg, "over_text"), FALSE);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "over_text")), FALSE);
    }
  }
}

void on_back_color (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  if ((current_applet == APPLET_WHEEL) ||
      (current_applet == APPLET_MORPHMENU) || 
      (current_applet == APPLET_TREEMENU) ||
      (current_applet == APPLET_ANFYPAINT) ||
      (current_applet == APPLET_SOLIDSCROLLER) ||
      (current_applet == APPLET_ANFYSNOW) ||
      (current_applet == APPLET_FIREWORKS))
  {
    dbl[0] = (double)GetRValue(dlg_ul_backcolor)/255;
    dbl[1] = (double)GetGValue(dlg_ul_backcolor)/255;
    dbl[2] = (double)GetBValue(dlg_ul_backcolor)/255;
  }
  else
  {
    dbl[0] = (double)dlg_backcolor_red/255;
    dbl[1] = (double)dlg_backcolor_green/255;
    dbl[2] = (double)dlg_backcolor_blue/255;
  }

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    if ((current_applet == APPLET_WHEEL) ||
	(current_applet == APPLET_MORPHMENU) || 
	(current_applet == APPLET_TREEMENU) ||
	(current_applet == APPLET_ANFYPAINT) ||
	(current_applet == APPLET_SOLIDSCROLLER) ||
	(current_applet == APPLET_ANFYSNOW) ||
	(current_applet == APPLET_FIREWORKS))
    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_backcolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "color_pixmap"), r, g, b);
    }
    else
    {
      dlg_backcolor_red = (int)(dbl[0]*255);
      dlg_backcolor_green = (int)(dbl[1]*255);
      dlg_backcolor_blue = (int)(dbl[2]*255);

      set_color_pixmap (lookup_widget (main_dlg, "color_pixmap"),
			dlg_backcolor_red, dlg_backcolor_green, dlg_backcolor_blue);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void on_fore_color (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)dlg_forcolor_red/255;
  dbl[1] = (double)dlg_forcolor_green/255;
  dbl[2] = (double)dlg_forcolor_blue/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    dlg_forcolor_red = (int)(dbl[0]*255);
    dlg_forcolor_green = (int)(dbl[1]*255);
    dlg_forcolor_blue = (int)(dbl[2]*255);

    set_color_pixmap (lookup_widget (main_dlg, "fore_pixmap"),
		      dlg_forcolor_red, dlg_forcolor_green, dlg_forcolor_blue);
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void on_first_color (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)dlg_f1color_red/255;
  dbl[1] = (double)dlg_f1color_green/255;
  dbl[2] = (double)dlg_f1color_blue/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    dlg_f1color_red = (int)(dbl[0]*255);
    dlg_f1color_green = (int)(dbl[1]*255);
    dlg_f1color_blue = (int)(dbl[2]*255);

    set_color_pixmap (lookup_widget (main_dlg, "first_pixmap"),
		      dlg_f1color_red, dlg_f1color_green, dlg_f1color_blue);
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void on_last_color (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)dlg_f2color_red/255;
  dbl[1] = (double)dlg_f2color_green/255;
  dbl[2] = (double)dlg_f2color_blue/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    dlg_f2color_red = (int)(dbl[0]*255);
    dlg_f2color_green = (int)(dbl[1]*255);
    dlg_f2color_blue = (int)(dbl[2]*255);

    set_color_pixmap (lookup_widget (main_dlg, "last_pixmap"),
		      dlg_f2color_red, dlg_f2color_green, dlg_f2color_blue);
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnPenColor (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_pencolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_pencolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_pencolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_pencolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "pen_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void on_back_image_toggle (GtkWidget *widget, gpointer user_data)
{
  gboolean enable = TRUE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
  {
    enable = FALSE;
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "back_image")), "");
  }

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), enable);
}

void on_back_image_browse (GtkWidget *widget, gpointer user_data)
{
  char* filename, szImgFormat[10], buf[16], *ptr, strWorkDir[1024];
  unsigned int iWidth, iWidthBack;
  unsigned int iHeight, iHeightBack;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(current_applet, strWorkDir);

  if (GetFormatAndSize(filename, szImgFormat, &iWidthBack, &iHeightBack) == 1)
  {
    if (current_applet == APPLET_TSCROLLER 
	|| current_applet == APPLET_TREEMENU 
	|| current_applet == APPLET_ANFYSNOW 
	|| current_applet == APPLET_ANFRACT)
    {
      sprintf (buf, "%d", iHeightBack);
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
      sprintf (buf, "%d", iWidthBack);
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), buf);
    }
    else if (current_applet == APPLET_ANFYPAINT )
    {
      // non deve fare nulla
    }
    else
    {
      int iRes=1;

      if (current_applet == APPLET_ROTATE)
      {
	ptr = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "resolution_entry")));
	iRes = strtol (ptr, NULL, 10);
      }
      else
	if (current_dialog != DLG_WHEEL 
	    && current_dialog != DLG_MORPHMENU
	    && current_dialog != DLG_TREEMENU 
	    && current_dialog != DLG_FLOZOIDS 
	    && current_dialog != DLG_ANFYPAINT
	    && current_dialog != DLG_FIREWORKS)
      {
	ptr = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "resolution_entry")));
	iRes = strtol (ptr, NULL, 10);
      }

      ptr = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "width")));
      iWidth = strtol (ptr, NULL, 10);
      ptr = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "height")));
      iHeight = strtol (ptr, NULL, 10);

      iWidth /= iRes;
      iHeight /= iRes;
      if( iWidth >0  && iHeight >0 )
      {
	if( iWidth != iWidthBack || iHeight != iHeightBack)
	{
	  if (current_dialog == DLG_WHEEL 
	      || current_dialog == DLG_MORPHMENU 
	      || current_dialog == DLG_TREEMENU 
	      || current_dialog == DLG_FLOZOIDS 
	      || current_dialog == DLG_ANFYPAINT
	      || current_dialog == DLG_FIREWORKS)
	    message_box(_("Error ! The background image width and height must equal the applet size"), 0);
	  else
	    message_box(_("Error ! The background image width and height must equal the applet size divided by resolution"), 0);

	  g_free (filename);
	  return;
	}
      }
    }
  }
  else
  {
    g_free (filename);
    return;
  }

  {
    char szSourcePath[1024], file[1024];
    PathFromFile(filename, szSourcePath);
    FileFromPath(filename, file);
    if( !MyCopyFile(szSourcePath, strWorkDir, file))
    {
      message_box(_("Error ! Problem when copying file in applet directory"), 0);
      g_free (filename);
      return;
    }
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "back_image")), file);
  }

  g_free (filename);
}

void on_image_browse (GtkWidget *widget, gpointer user_data)
{
  char* filename, szImgFormat[10], buf[16], strWorkDir[1024];
  unsigned int iWidth;
  unsigned int iHeight;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(current_applet, strWorkDir);

  if (GetFormatAndSize(filename, szImgFormat, &iWidth, &iHeight) == 1)
  {
    if( current_dialog == DLG_ROTATE  || current_dialog == DLG_WARP )
    {
      if( !IsTwoPower(iWidth) ||  !IsTwoPower(iHeight) )
      {
	message_box(_("Error ! The image size must be powers of 2, es: 64*64,128*64,256*128,16*16..."), 0);
	g_free (filename);
	return;
      }
    }

    if( current_dialog == DLG_WOBBLER )
    {
      if( !IsTwoPower(iWidth) ||  !IsTwoPower(iHeight) )
	message_box(_("Warning: this applet runs faster using images of special sizes such as 32x32 or 256x256!"), 0);
    }

    if( current_dialog == DLG_TUNNEL )
      {
	sprintf (buf, "%d", iHeight);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image_height")), buf);
	sprintf (buf, "%d", iWidth);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image_width")), buf);
      }
    else
      if (current_dialog != DLG_ROTATE 
	  && current_dialog != DLG_TUNNEL3D
	  && current_dialog != DLG_WARP
	  && current_dialog != DLG_WOBBLER)
	{
	  sprintf (buf, "%d", iHeight);
	  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
	  sprintf (buf, "%d", iWidth);
	  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), buf);
	  dlg_oldres=1;
	  /*
	    OnSelchangeCboRes();
	  */
	}

    if( current_dialog == DLG_VOXEL ) 
    {
      if( iWidth != 256 || iHeight != 256)
      {
	message_box(_("Error ! The image size must be 256*256"), 0);
	g_free (filename);
	return;
      }
    }

    {
      char szSourcePath[1024], file[1024];
      PathFromFile(filename, szSourcePath);
      FileFromPath(filename, file);
      if( !MyCopyFile(szSourcePath, strWorkDir, file))
      {
	message_box(_("Error ! Problem when copying file in applet directory"), 0);
	g_free (filename);
	return;
      }

      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image")), file);
    }
  }

  g_free (filename);
}

void OnBrowseFileImg (GtkWidget *widget, gpointer user_data)
{
  char* filename, szImgFormat[10], strWorkDir[1024], buf[16];
  unsigned int iWidth, iWidthAlt;
  unsigned int iHeight, iHeightAlt;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(current_applet, strWorkDir);

  if (GetFormatAndSize(filename, szImgFormat, &iWidth, &iHeight) == 1)
  {
    if (current_dialog == DLG_ANFYBUMP)
    {
      char* strSecondImage;

      strSecondImage = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "bump_image")));

      if (strlen (strSecondImage) != 0)
      {
	char strTmp[1024];
	GetAppletPath(current_applet, strTmp);

	strcat (strTmp, "/");
	strcat (strTmp, strSecondImage);
	if (GetFormatAndSize (strTmp, szImgFormat, &iWidthAlt, &iHeightAlt) == TRUE)
	{
	  if( iWidth != iWidthAlt || iHeight != iHeightAlt)
	  {
	    message_box (_("Error ! The bump image and altenative bump image must be of the same size"), 0);
	    g_free (filename);
	    return;
	  }
	}
	else
	{
	  g_free (filename);
	  return;
	}
      }
    }

    sprintf (buf, "%d", iHeight);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
    sprintf (buf, "%d", iWidth);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), buf);
    dlg_oldres = 1;
    /*
      if (current_dialog != DLG_ANLENS)
      {
	OnSelchangeCboRes();
      }
    */

    {
      char szSourcePath[1024], file[1024];
      PathFromFile(filename, szSourcePath);
      FileFromPath(filename, file);
      if( !MyCopyFile(szSourcePath, strWorkDir, file))
      {
	message_box(_("Error ! Problem when copying file in applet directory"), 0);
	g_free (filename);
	return;
      }

      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image")), file);
    }
    /*
      if( m_uiID == IDD_ANLAKE )
      {
        OnHalfreflect();
      }
    */
  }

  g_free (filename);
}

void on_progress (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *w;

  w = lookup_widget (main_dlg, "pen_progr");
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    gtk_widget_set_sensitive (w, TRUE);
  else
    gtk_widget_set_sensitive (w, FALSE);
}

// =========================================================
// STEP0 dialog callbacks

void on_step0_preview_all (GtkWidget *widget, gpointer user_data)
{
  char str[1024];

  strcpy (str, g_strAppPath);
  strcat (str, "/");
  strcat (str, g_strHelpDir);
  strcat (str, "/samples.html");

  StartBrowser (str);
}

void on_step0_combo_applet_changed (GtkWidget *widget, gpointer user_data)
{
  int i;
  GtkWidget* label;
  gchar *name, *desc = "", buf[50];
  name = gtk_entry_get_text (GTK_ENTRY (widget));                               

  label = lookup_widget (widget->parent, "description");

  // Update the description
  for (i = 0; i < NUMAPPLETS; i++)
    if (strcmp (applet_info[i].name, name) == 0)
    {
      desc = applet_info[i].description;
      current_applet = i;
      break;
    }
  gtk_label_set_text (GTK_LABEL (label), desc);

  // Update the 'Help' menu
  sprintf (buf, _("Applet %s info"), name);
  label = lookup_widget (widget->parent, "applet_info");
  gtk_label_set_text (GTK_LABEL(GTK_BIN(label)->child), buf);
}

void on_step0_category_changed (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *combo;
  GList *glist;

  combo = lookup_widget (widget->parent, "applet_combo");
  glist = NULL;

  switch ((int)user_data)
  {
    // Banner / Slideshow
    case 1:
    {
      glist = g_list_append (glist, applet_info[APPLET_BOOKFLIP].name);
      glist = g_list_append (glist, applet_info[APPLET_ANFADE].name);
      glist = g_list_append (glist, applet_info[APPLET_MOSAIC].name);
    } break;

    // Fractals & ALife
    case 2:
    {
	glist = g_list_append (glist, applet_info[APPLET_FLOZOIDS].name);
	glist = g_list_append (glist, applet_info[APPLET_ANFRACT].name);
	glist = g_list_append (glist, applet_info[APPLET_LIFE2D].name);
	glist = g_list_append (glist, applet_info[APPLET_MANDEL].name);
    } break;

    // Image effects
    case 3:
    {
	glist = g_list_append (glist, applet_info[APPLET_ANFYBUMP].name);
	glist = g_list_append (glist, applet_info[APPLET_DEFORM].name);
	glist = g_list_append (glist, applet_info[APPLET_FIREWORKS].name);
	glist = g_list_append (glist, applet_info[APPLET_HUEROT].name);
	glist = g_list_append (glist, applet_info[APPLET_ANLAKE].name);
	glist = g_list_append (glist, applet_info[APPLET_ANLENS].name);
	glist = g_list_append (glist, applet_info[APPLET_ROTATE].name);
	glist = g_list_append (glist, applet_info[APPLET_ANFYSNOW].name);
	glist = g_list_append (glist, applet_info[APPLET_WARP].name);
	glist = g_list_append (glist, applet_info[APPLET_ANWATER].name);
	glist = g_list_append (glist, applet_info[APPLET_WOBBLER].name);
    } break;

    // 3D Applets
    case 4:
    {
	glist = g_list_append (glist, applet_info[APPLET_ANFY3D].name);
	glist = g_list_append (glist, applet_info[APPLET_FLUID].name);
	glist = g_list_append (glist, applet_info[APPLET_GALAXY].name);
	glist = g_list_append (glist, applet_info[APPLET_TMAPCUBE].name);
	glist = g_list_append (glist, applet_info[APPLET_ANTUNNEL].name);
	glist = g_list_append (glist, applet_info[APPLET_TUNNEL3D].name);
	glist = g_list_append (glist, applet_info[APPLET_ANVOXEL].name);
	glist = g_list_append (glist, applet_info[APPLET_WORMHOLE].name);
    } break;

    // Navigation menu
    case 5:
    {
	glist = g_list_append (glist, applet_info[APPLET_CUBEMENU].name);
	glist = g_list_append (glist, applet_info[APPLET_MORPHMENU].name);
	glist = g_list_append (glist, applet_info[APPLET_TREEMENU].name);
	glist = g_list_append (glist, applet_info[APPLET_WHEEL].name);
    } break;

    // Other
    case 6:
    {
	glist = g_list_append (glist, applet_info[APPLET_ANFYCAM].name);
	glist = g_list_append (glist, applet_info[APPLET_ANFYPAINT].name);
	glist = g_list_append (glist, applet_info[APPLET_BLOBS].name);
	glist = g_list_append (glist, applet_info[APPLET_ANBLUR].name);
	glist = g_list_append (glist, applet_info[APPLET_FIRE].name);
	glist = g_list_append (glist, applet_info[APPLET_ANFLAG].name);
	glist = g_list_append (glist, applet_info[APPLET_FLAGLOAD].name);
	glist = g_list_append (glist, applet_info[APPLET_PLASMA].name);
	glist = g_list_append (glist, applet_info[APPLET_SOLIDSCROLLER].name);
	glist = g_list_append (glist, applet_info[APPLET_TSCROLLER].name);
    } break;
  }

  gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist);
  g_list_free (glist);
}

// =========================================================
// EXPERT dialog callbacks

void on_expert_standard (GtkWidget *widget, gpointer user_data)
{
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "memory")), "1000");
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "task")), "3");
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "minsync")), "10");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "netscape")), FALSE);
}

void on_expert_compatibility (GtkWidget *widget, gpointer user_data)
{
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "memory")), "15");
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "task")), "1");
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "minsync")), "30");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "netscape")), TRUE);
}

void on_expert_speed (GtkWidget *widget, gpointer user_data)
{
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "memory")), "1000");
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "task")), "6");
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "minsync")), "2");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "netscape")), FALSE);
}

void on_expert_enable_over (GtkWidget *widget, gpointer user_data)
{
  gboolean enable;

  enable = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_image")));

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "browse"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "xoffset"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "yoffset"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "image"), enable);

  if (!enable)
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image")), "");
}

void on_expert_browse (GtkWidget *widget, gpointer user_data)
{
  char* filename, strAppletDir[1024];

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(current_applet, strAppletDir);

  {
    char szSourcePath[1024], file[1024];
    PathFromFile(filename, szSourcePath);
    FileFromPath(filename, file);
    if (!MyCopyFile(szSourcePath, strAppletDir, file))
    {
      message_box(_("Error ! Problem when copying file in applet directory"), 0);
      g_free (filename);
      return;
    }

    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image")), file);
  }
  g_free (filename);
}

void on_step1_clipboard (GtkWidget *widget, gpointer this_text)
{
  gtk_editable_select_region (GTK_EDITABLE (this_text), 0, gtk_text_get_length (GTK_TEXT (this_text))); 
  gtk_editable_copy_clipboard (GTK_EDITABLE (this_text));
}

// =========================================================
// REGISTRATION dialog callbacks

void GetListRegCode(char* strRegCode, GtkWidget* dlg)
{
  GtkCList *cl;
  char* ptr;
  int i;

  strcpy (strRegCode, "");
  cl = GTK_CLIST (lookup_widget (dlg, "regcode_list"));

  for (i = 0; i < cl->rows; i++)
  {
    gtk_clist_get_text (cl, i, 0, &ptr);

    strcat (strRegCode, ptr);
		
    if (i+1 < cl->rows)
      strcat (strRegCode, "+");
  }
}

void on_add_code (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *entry;
  char* row[2];

  entry = lookup_widget (widget->parent, "regcode_entry");
  row[0] = gtk_entry_get_text (GTK_ENTRY (entry));

  if (strlen (row[0]) == 0)
    return;
  row[1] = NULL;

  gtk_clist_append (GTK_CLIST (lookup_widget (widget->parent, "regcode_list")), row);
  gtk_entry_set_text (GTK_ENTRY (entry), "");
  gtk_widget_set_sensitive (lookup_widget (widget->parent, "regcode_remove"), TRUE);

  {
    char* strDomain, strItem[256];

    //    strDomain = gtk_entry_get_text (GTK_ENTRY (lookup_widget (widget->parent, "domain_entry")));
    gtk_label_get (GTK_LABEL (GTK_BIN (lookup_widget (widget->parent, "domains"))->child), &strDomain);
    GetListRegCode(strItem, widget->parent);

    printf ("aa-%s-%s\n", strDomain,strItem);
    SaveVar(INI_APP_SECTION, strDomain, strItem);
  }
}

void on_remove_code (GtkWidget *widget, gpointer user_data)
{
  GtkCList *cl;
  int index;

  cl = GTK_CLIST (lookup_widget (widget->parent, "regcode_list"));
  index = cl->focus_row;

  if (index >= 0)
  {
    gtk_clist_remove (cl, index);

    if (cl->rows > 0)
    {
      if (index > cl->rows)
	index--;

      cl->focus_row = index;
      gtk_clist_select_row (cl, index, 0);
    }
    else
      gtk_widget_set_sensitive (lookup_widget (widget->parent, "regcode_remove"), FALSE);

    {
      char* strDomain, strItem[256];

      strDomain = gtk_entry_get_text (GTK_ENTRY (lookup_widget (widget->parent, "domain_entry")));
      GetListRegCode(strItem, widget->parent);
      SaveVar(INI_APP_SECTION, strDomain, strItem);
    }
  }
}

void on_add_domain (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *menu, *item;
  GtkWidget *entry, *child;
  GList *children;
  char *domain, *str, buf[64];
  int index = 0;

  entry = lookup_widget (widget->parent, "domain_entry");
  domain = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
  children = GTK_MENU_SHELL (GTK_OPTION_MENU (lookup_widget (widget->parent, "domains"))->menu)->children;

  // Check if the string is already in the list
  while (children)
  {
    child = (GtkWidget*)children->data;
    children = children->next;
    index++;
 
    if (GTK_BIN (child)->child == NULL)
	child =  (lookup_widget (widget->parent, "domains"));

    gtk_label_get (GTK_LABEL (GTK_BIN (child)->child), &str);

    if (strcmp (str, domain) == 0)
    {
      message_box (_("The name already exist in the list, write new name"), 0);
      gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget)), entry);
      g_free (domain);
      return;
    }
  }

  // Add new entry
  menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (lookup_widget (widget->parent, "domains")));
  item = gtk_menu_item_new_with_label (domain);
  gtk_signal_connect (GTK_OBJECT (item), "activate",
                      GTK_SIGNAL_FUNC (OnSelchangeDomains), NULL);
  gtk_widget_show (item);
  gtk_menu_append (GTK_MENU (menu), item);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (lookup_widget (widget->parent, "domains")), menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (lookup_widget (widget->parent, "domains")), index);

  sprintf (buf, "%s%d", INI_NDOMAIN_, index);
  SaveVar (INI_APP_SECTION, buf, domain);

  gtk_widget_set_sensitive (lookup_widget (widget->parent, "regcode_add"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (widget->parent, "domain_remove"), TRUE);
  gtk_clist_clear (GTK_CLIST (lookup_widget (widget->parent, "regcode_list")));
  g_free (domain);
}

void on_remove_domain (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *opt;
  GtkMenu *menu;
  char strSection[256], *domain;
  int index;

  opt = lookup_widget (widget->parent, "domains");
  menu = GTK_MENU (gtk_option_menu_get_menu (GTK_OPTION_MENU (opt)));
  gtk_label_get (GTK_LABEL (GTK_BIN (opt)->child), &domain);

  index = g_list_index (GTK_MENU_SHELL (GTK_OPTION_MENU (opt)->menu)->children, gtk_menu_get_active (menu));
  sprintf (strSection, "%s%d", INI_NDOMAIN_, index);
  SaveVar (INI_APP_SECTION, strSection, "");
  SaveVar (INI_APP_SECTION, domain, "");

  gtk_widget_destroy (gtk_menu_get_active (menu));
  gtk_option_menu_set_menu (GTK_OPTION_MENU (opt), GTK_WIDGET (menu));
  gtk_option_menu_set_history (GTK_OPTION_MENU (opt), 0);

  if (GTK_MENU_SHELL (menu)->children == NULL)
  {
    gtk_clist_clear (GTK_CLIST (lookup_widget (widget->parent, "regcode_list")));
    gtk_widget_set_sensitive (lookup_widget (widget->parent, "domain_remove"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (widget->parent, "regcode_add"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (widget->parent, "regcode_remove"), FALSE);
  }
}

void OnSelchangeDomains (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *list;
  char strRegCode[256], *pszItem, *domain;

  gtk_label_get (GTK_LABEL (GTK_BIN (lookup_widget (widget->parent, "domains"))->child), &domain);
  SaveVar(INI_APP_SECTION, INI_DOMAIN, domain);
  list = lookup_widget (main_dlg, "regcode_list");
  gtk_widget_set_sensitive (lookup_widget (widget->parent, "domain_remove"), TRUE);
  gtk_clist_clear (GTK_CLIST (list));

  GetSavedVar(INI_APP_SECTION, domain, strRegCode);
  printf ("a-%s-%s\n", domain, strRegCode);

  pszItem = g_malloc (strlen (strRegCode)+1);
  if (pszItem != NULL)
  {
    char *token = NULL, *seps = "+";

    strcpy(pszItem, strRegCode);
    if (strcmp (pszItem, INI_NO) != 0)
    {
      token = strtok( pszItem, seps );
      while (token != NULL)   
      {
	char* row[2] = { token, NULL };
	gtk_clist_append (GTK_CLIST (list), row);
	token = strtok (NULL, seps);   
      }

      if (GTK_CLIST (list)->rows > 0)
	gtk_widget_set_sensitive (lookup_widget (widget->parent, "regcode_remove"), TRUE);
      else
	gtk_widget_set_sensitive (lookup_widget (widget->parent, "regcode_remove"), FALSE);
    }
    g_free (pszItem);
  }
}

void on_enable_link (GtkWidget *widget, gpointer user_data)
{
  if ((current_applet != APPLET_CUBEMENU) &&
      (current_applet != APPLET_ANFADE) &&
      (current_applet != APPLET_MOSAIC) &&
      (current_applet != APPLET_BOOKFLIP) &&
      (current_applet != APPLET_WHEEL) &&
      (current_applet != APPLET_MORPHMENU) &&
      (current_applet != APPLET_TREEMENU))
  {
    gboolean enable = FALSE;

    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
      enable = TRUE;

    gtk_widget_set_sensitive (lookup_widget (main_dlg, "link_entry"), enable);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "enable_target"), enable);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "target_entry"), enable);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "mode_combo"), enable);

    if (!enable)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_target")), FALSE);
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "link_entry")), "");
    }
  }

  on_new_frame (lookup_widget (main_dlg, "enable_target"), NULL);
}

void on_new_frame (GtkWidget *widget, gpointer user_data)
{
  gboolean enable = FALSE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    enable = TRUE;

    gtk_widget_set_sensitive (lookup_widget (main_dlg, "target_entry"), enable);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "mode_combo"), enable);

    if (enable)
      {
	/*
      OnSelchangeFrameMode();
	*/
      }
    else
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "mode_entry")), "_blank");

    /*
    if( GetDlgItem(IDC_AUTOAPPLY) != NULL && !m_bDeleting )
      if( ((CButton*)GetDlgItem(IDC_AUTOAPPLY))->GetCheck() == 1 )
	OnApplyChange();
    */
}

void on_change_frame_mode (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *entry;
  GtkLabel *label;
  char* str;

  label = GTK_LABEL (lookup_widget (main_dlg, "mode_label"));
  entry = lookup_widget (main_dlg, "target_entry");
  str = gtk_entry_get_text (GTK_ENTRY (widget));

  if (strcmp (str, _("other target...")) != 0)
  {
    if (strcmp (str, "_blank") == 0)
      gtk_label_set_text (label, _("Load in a new blank unnamed browser window"));
    else if (strcmp (str, "_self") == 0)
      gtk_label_set_text (label, _("Load into the same window the applet occupies"));
    else if (strcmp (str, "_parent") == 0)
      gtk_label_set_text (label, _("Load into the immediate FRAMESET parent"));
    else if (strcmp (str, "_top") == 0)
      gtk_label_set_text (label, _("Load into the top body of the window"));

    gtk_widget_set_sensitive (entry, FALSE);
    gtk_entry_set_text (GTK_ENTRY (entry), "");
  }
  else
  {
    gtk_widget_set_sensitive (entry, TRUE);
    gtk_label_set_text (label, _("Specify name:"));
  }
}

// =========================================================
// other callbacks

void on_autodesign (GtkWidget *widget, gpointer user_data)
{
  gboolean enable;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    enable = FALSE;
  else
    enable=TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "rsize"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "rfactor"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ssize"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "sfactor"), enable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "fish_entry"), enable);
}

void on_fire_changeres (GtkWidget *widget, gpointer user_data)
{
  char *tmp, *x, *y, buf[16];
  int w, h;

  tmp = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "fire_width")));
  x = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "x_entry")));
  w = strtol (tmp, NULL, 10) * strtol (x, NULL, 10);

  tmp = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "fire_height")));
  y = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "y_entry")));
  h = strtol (tmp, NULL, 10) * strtol (y, NULL, 10);

  sprintf (buf, "%d", w);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), buf);
  sprintf (buf, "%d", h);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
}

void on_random (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "script_combo"), FALSE);
  else
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "script_combo"), TRUE);
}

void OnCross (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "factor"), FALSE);
  else
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "factor"), TRUE);
}

void OnAnwaterLight(GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=FALSE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable=TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "min"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "max"), bEnable);
}

void OnFixdropvalue(GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=TRUE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    bEnable=FALSE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "fix_x"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "fix_y"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "fix_size"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "fix_factor"), bEnable);

  if( !bEnable )
  {
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "fix_x")), "0");
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "fix_y")), "0");
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "fix_size")), "0");
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "fix_factor")), "0");
  }
}

void OnChkDistorsion (GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=FALSE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable=TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "xy"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "diagonal_combo"), bEnable);
}

void OnHalfreflect (GtkWidget *widget, gpointer user_data) 
{
  char* strTmp, strImgFormat[10], strImageFile[1024], buf[16];
  unsigned int iWidth, iHeight;
  int iRes;

  strTmp = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "resolution_combo_entry")));
  iRes = strtol(strTmp, NULL, 10);
  strTmp = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "image")));

  GetAppletPath (current_applet, strImageFile);
  strcat (strImageFile, "/");
  strcat (strImageFile, strTmp);
  if (GetFormatAndSize (strImageFile, strImgFormat, &iWidth, &iHeight) == TRUE)
  {
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "half_effect"))) == TRUE)
      sprintf (buf, "%d", (int) (iHeight*iRes*1.5));
    else
      sprintf (buf, "%d", (int) (iHeight*iRes*2.0));

    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
  }
}

void OnInfoCustom3d (GtkWidget *widget, gpointer user_data) 
{
  info_dlg (_("This applet is NOT capable of loading custom objects. If you want an applet that display\nthe rotating logo of your company, or any 3d object and scene you want, you can download Anfy 3D full.\nIf you are interested, go to\n"), _("Anfy 3d Full Home Page"), _("Custom 3D objects ?"));
}

void OnFog (GtkWidget *widget, gpointer user_data) 
{
  gboolean bEnable=FALSE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable=TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "start"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "stop"), bEnable);
}

void OnSelchangeBackmode (GtkWidget *widget, gpointer user_data) 
{
  GtkWidget* w;
  char* strBackType;

  strBackType = gtk_entry_get_text (GTK_ENTRY (widget));

  w = lookup_widget (main_dlg, "blur");
  if (strcmp (strBackType, "black") == 0)
    gtk_widget_set_sensitive (w, TRUE);
  else
  {
    gtk_widget_set_sensitive (w, FALSE);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), FALSE);
  }

  if (strcmp (strBackType, "sphere") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "render_entry"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "bt_entry"), TRUE);
  }
  else
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "render_entry"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "bt_entry"), FALSE);
  }

  w = lookup_widget (main_dlg, "back_color");
  if (strcmp (strBackType, "colour") == 0)
    gtk_widget_set_sensitive (w, TRUE);
  else
    gtk_widget_set_sensitive (w, FALSE);		
}

void OnAnfy3d_check (GtkWidget *widget, gpointer user_data) 
{
  GtkWidget *b, *e;
  gboolean bEnable=FALSE;

  switch (GPOINTER_TO_INT (user_data))
  {
  case 1:
    b = lookup_widget (main_dlg, "t1_browse");
    e = lookup_widget (main_dlg, "t1_entry");
    break;
  case 2:
    b = lookup_widget (main_dlg, "t2_browse");
    e = lookup_widget (main_dlg, "t2_entry");
    break;
  case 3:
    b = lookup_widget (main_dlg, "p_browse");
    e = lookup_widget (main_dlg, "p_entry");
    break;
  case 4:
    b = lookup_widget (main_dlg, "g_browse");
    e = lookup_widget (main_dlg, "g_entry");
    break;
  default:
    return;
  }

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable=TRUE;
  else
    gtk_entry_set_text (GTK_ENTRY (e), "");

  gtk_widget_set_sensitive (b, bEnable);
  gtk_widget_set_sensitive (e, bEnable);
}

void OnAnfy3d_browse (GtkWidget *widget, gpointer user_data) 
{
  GtkWidget *e;
  char* str;

  switch (GPOINTER_TO_INT (user_data))
  {
  case 0:
    e = lookup_widget (main_dlg, "t0_entry");
    break;
  case 1:
    e = lookup_widget (main_dlg, "t1_entry");
    break;
  case 2:
    e = lookup_widget (main_dlg, "t2_entry");
    break;
  case 3:
    e = lookup_widget (main_dlg, "p_entry");
    break;
  case 4:
    e = lookup_widget (main_dlg, "g_entry");
    break;
  default:
    return;
  }
  
  file_browse ("Open", &str);

  if (str != NULL)
  {
    gtk_entry_set_text (GTK_ENTRY (e), str);
    g_free (str);
  }
}

void OnSelchangeMaintype (GtkWidget *widget, gpointer user_data) 
{
  gboolean bEnable=TRUE;
  char* str;

  str = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "main_type_entry")));

  if (strcmp (str, "sheet") != 0)
    bEnable=FALSE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "height"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "width"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "height_faces"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "width_faces"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "add"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ampl"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "p1"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "p2"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "func_combo"), bEnable);
}

void OnSelchangeChildtype (GtkWidget *widget, gpointer user_data) 
{
  gboolean bEnable=FALSE;
  char* str;

  str = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "child_type_entry")));

  if (strcmp (str, "Off") != 0)
    bEnable=TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "child_render_combo"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "child_tex_combo"), bEnable);
}

void OnBrowseAudio (GtkWidget *widget, gpointer user_data)
{
  char strWorkDir[1024];
  char* filename;
  char szSourcePath[1024], szFileName[1024];

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(APPLET_ANBLUR, strWorkDir);

  PathFromFile(filename, szSourcePath);
  FileFromPath(filename, szFileName);
  if (!MyCopyFile(szSourcePath, strWorkDir, szFileName ))
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    g_free (filename);
    return;
  }

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "audio")), szFileName);
  g_free (filename);
}

void OnBends (GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=FALSE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable=TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "bends_num"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "bends_min"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "bends_max"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "bends_offset"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "bends_offset_inc"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "bends_size_inc"), bEnable);
}

void OnEnableAlt (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "bump_image"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "bump_image_browse"), TRUE);
  }
  else
  {
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "bump_image")), "");
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "bump_image"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "bump_image_browse"), FALSE);
  }
}

void OnBrowseFileImg2 (GtkWidget *widget, gpointer user_data)
{
  char strWorkDir[1024], szImgFormat[10];
  char* filename;
  char szSourcePath[1024], szFileName[1024];
  unsigned int iWidth, iWidthAlt;
  unsigned int iHeight, iHeightAlt;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(APPLET_ANFYBUMP, strWorkDir);

  if (GetFormatAndSize (filename, szImgFormat, &iWidth, &iHeight) == TRUE)
  {
    char* strFirstImage;

    strFirstImage = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "image")));

    if (strlen (strFirstImage) != 0)
    {
      if (GetFormatAndSize (strFirstImage, szImgFormat, &iWidthAlt, &iHeightAlt) == TRUE )
      {
	if( iWidth != iWidthAlt || iHeight != iHeightAlt)
	{
	  message_box (_("Error ! The bump image and altenative bump image must be of the same size"), 0);
	  g_free (filename);
	  return;
	}
      }
      else
      {
	g_free (filename);
	return;
      }
    }
  }
  else
  {
    g_free (filename);
    return;
  }
 
  PathFromFile(filename, szSourcePath);
  FileFromPath(filename, szFileName);
  if (!MyCopyFile(szSourcePath, strWorkDir, szFileName ))
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    g_free (filename);
    return;
  }

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "bump_image")), szFileName);
  g_free (filename);
}

void OnCyclic (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    return;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "pong_audio"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ping_audio"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "pong_browse"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ping_browse"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "pong_pause"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ping_pause"), FALSE);
}

void OnDoublesize (GtkWidget *widget, gpointer user_data)
{
  unsigned int iWidth, iHeight;
  GtkEntry *w, *h;
  char tmp[16];

  w = GTK_ENTRY (lookup_widget (main_dlg, "width"));
  h = GTK_ENTRY (lookup_widget (main_dlg, "height"));

  iWidth = strtol (gtk_entry_get_text (w), NULL, 10);
  iHeight = strtol (gtk_entry_get_text (h), NULL, 10);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
  {
    sprintf (tmp, "%d", (int)iWidth/2);
    gtk_entry_set_text (w, tmp);
    sprintf (tmp, "%d", (int)iHeight/2);
    gtk_entry_set_text (h, tmp);
  }
  else
  {
    sprintf (tmp, "%d", (int)iWidth*2);
    gtk_entry_set_text (w, tmp);
    sprintf (tmp, "%d", (int)iHeight*2);
    gtk_entry_set_text (h, tmp);
  }
}

void OnPingpong (GtkWidget *widget, gpointer user_data) 
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    return;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "pong_audio"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ping_audio"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "pong_browse"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ping_browse"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "pong_pause"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "ping_pause"), TRUE);
}

void OnBrowseFileFrames (GtkWidget *widget, gpointer user_data) 
{
  /*
  char* filename, strWorkDir[1024];

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(APPLET_ANBLUR, strWorkDir);

  if (filename[strlen(filename)-5] < '0' || filename[strlen(filename)-5] > '9')
  {
    message_box (_("Error ! The file name must terminate with sequential number:\nframe1.jpg, frame2.jpg, frame3.jpg..."), 0);
    g_free (filename);
    return;
  }

  FileFromPath(filename, szFileName);
  PathFromFile(filename, szOldPath);
  if (!MyCopyFile(szOldPath,strWorkDir,szFileName))
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    g_free (filename);
    return;
  }

		POSITION posFile=fd.GetStartPosition();
		TCHAR szFileName[_MAX_PATH];
		TCHAR szOldPath[_MAX_PATH];
		CString strPathFileName;
		CString strTmp;

		int iLength;
		int iNumImgs=0;
		while(posFile)
		{
			strPathFileName=fd.GetNextPathName(posFile);
			iNumImgs++;
		}
		if(iNumImgs<=1)
		{
			AfxMessageBox(IDS_ERR_SEL_IMAGE, MB_ICONEXCLAMATION);
			return;
		}

		strTmp=szFileName;
		int i=strTmp.GetLength();	
		i-= 4;						//.gif .jpg
		if(iNumImgs>9)				//i numeri alla fine del nome del file
			i-= ( (iNumImgs % 10) + 1 );
		else
			i--;
		strTmp=strTmp.Mid(0,i);

		TCHAR szImgFormat[10];
		unsigned int iWidth;
		unsigned int iHeight;
		if( GetFormatAndSize( strPathFileName, szImgFormat, &iWidth, &iHeight) == TRUE )
		{
			SetDlgItemText(IDC_IMG_PATHNAME,  strTmp );
			SetDlgItemInt(IDC_NIMGS, iNumImgs);
			if( ((CButton*)GetDlgItem(IDC_DOUBLESIZE))->GetCheck() == 1 )
			{
				SetDlgItemInt(IDC_WIDTH,iWidth*2);
				SetDlgItemInt(IDC_HEIGHT,iHeight*2);
			}
			else
			{
				SetDlgItemInt(IDC_WIDTH,iWidth);
				SetDlgItemInt(IDC_HEIGHT,iHeight);
			}
			SetDlgItemText(IDC_FILETYPE, szImgFormat);
		}
  */
}

void OnBrowseFilePong (GtkWidget *widget, gpointer user_data)
{
  char strWorkDir[1024];
  char* filename;
  char szSourcePath[1024], szFileName[1024];

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(APPLET_ANBLUR, strWorkDir);

  PathFromFile(filename, szSourcePath);
  FileFromPath(filename, szFileName);
  if( !MyCopyFile(szSourcePath, strWorkDir, szFileName ) )
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    return;
  }

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "pong_audio")), szFileName);
  g_free (filename);
}

void OnBrowseFilePing (GtkWidget *widget, gpointer user_data)
{
  char strWorkDir[1024];
  char* filename;
  char szSourcePath[1024], szFileName[1024];

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(APPLET_ANBLUR, strWorkDir);

  PathFromFile(filename, szSourcePath);
  FileFromPath(filename, szFileName);
  if( !MyCopyFile(szSourcePath, strWorkDir, szFileName ) )
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    return;
  }

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "ping_audio")), szFileName);
  g_free (filename);
}

void OnPresets (GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=FALSE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable=TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "preset_wobbler"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "preset_waves"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "preset_star"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "preset_sphere"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "preset_breath"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "preset_metal"), bEnable);
}

void OnWobbler (GtkWidget *widget, gpointer user_data) 
{
  int i;
  const char* Presets[] = {
    "3.0", "0.1", "5.0", "20.0","400.0", "8.0", "40.0", "30.0", "50.0", "40.0", "200",	"200", "0", "10",
    "5.0", "0.0", "5.0", "5.0", "400.0","102.0","120.0", "140.0", "0.4", "10.0", "0", "0", "0", "200",
    "80.0","0.0", "80.0","80.0","40.0",  "0.0", "10.0","100000", "10.0", "10.0", "200", "200", "0", "100",
    "12.0","0.0", "12.0","12.0","220.0", "0.0", "1000.0","1000.0", "10.0", "10.0", "200", "200","0","30",
    "10.0","0.3", "2.0", "20.0","380.0", "0.0", "10.0","10.0", "10.0", "10.0", "0", "200", "0", "10",
    "6.0", "0.4", "2.0", "22.0","4000.0","16.0","5.8","20.0", "1.4", "100.0", "140", "140",	"0", "10"
};
  const unsigned int MaxPreset=14;

  i = GPOINTER_TO_INT (user_data);
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    return;

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "dist_initial")), Presets[0 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "zoom")), Presets[1 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "dist_min")), Presets[2 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "dist_max")), Presets[3 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "radius")), Presets[4 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "wave")), Presets[5 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "speed1")), Presets[6 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "speed2")), Presets[7 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "speed3")), Presets[8 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "speed4")), Presets[9 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "movex")), Presets[10 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "movey")), Presets[11 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "start")), Presets[12 + (MaxPreset*i)]);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "move")), Presets[13 + (MaxPreset*i)]);
}

void OnFindImage (GtkWidget *widget, gpointer user_data)
{
  char* filename, strAppletDir[1024];
  char szSourcePath[1024], szFileName[1024];

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath(current_applet, strAppletDir);

  PathFromFile(filename, szSourcePath);
  FileFromPath(filename, szFileName);
  if( !MyCopyFile(szSourcePath, strAppletDir, szFileName ) )
  {
    message_box (_("Error ! Problem when copying file in applet directory"), 0);
    return;
  }

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "image")), szFileName);
}

void OnTileImage (GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=TRUE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable=FALSE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_color"), bEnable);
}

void OnSelchangeRotmode (GtkWidget *widget, gpointer user_data)
{
  char* strItem;

  strItem = gtk_entry_get_text (GTK_ENTRY (widget));

  if (strcmp (strItem, "Zoom and rotate") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "rot_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_min"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_max"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_speed"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "x_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "distortion"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "first"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "second"), FALSE);
  }
  else if (strcmp (strItem, "Rotate only") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "rot_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_min"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_max"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_speed"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "x_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "distortion"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "first"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "second"), FALSE);
  }
  else if (strcmp (strItem, "Zoom only") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "rot_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_min"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_max"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_speed"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "x_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "distortion"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "first"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "second"), FALSE);
  }
  else if (strcmp (strItem, "Distort only") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "rot_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_min"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_max"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_speed"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "x_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "distortion"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "first"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "second"), FALSE);
  }
  else if (strcmp (strItem, "Zoom and distort") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "rot_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_min"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_max"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_speed"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "x_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "distortion"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "first"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "second"), FALSE);
  }
  else if (strcmp (strItem, "Automatic") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "rot_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_min"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_max"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "zoom_speed"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "x_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "distortion"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "first"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "second"), TRUE);
  }
}

void OnSelchangeText_Type (GtkWidget *widget, gpointer user_data)
{
  char* strValue;
 
  strValue = gtk_entry_get_text (GTK_ENTRY (widget));

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "wave_mode_combo"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "wave_speed"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "wave_type_combo"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "reverse"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "jump_amp"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "jump_speed"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "v_spac"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_offset"), FALSE);

  if (strcmp (strValue, "waving") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "wave_mode_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "wave_speed"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "wave_type_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "reverse"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_offset"), TRUE);
  }
  else if (strcmp (strValue, "jumping") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "jump_amp"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "jump_speed"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_offset"), TRUE);
  }
  else if (strcmp (strValue, "vertical") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "v_spac"), TRUE);
  }
  else if (strcmp (strValue, "horizontal") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_offset"), TRUE);
  }
}

void OnSelchangeMirrormode (GtkWidget *widget, gpointer user_data)
{
  char* strValue;
 
  strValue = gtk_entry_get_text (GTK_ENTRY (widget));

  if (strcmp (strValue, "disabled") == 0)
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "mirror_height"), FALSE);
  else
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "mirror_height"), TRUE);
}

void tree_select_row (GtkCTree *ctree, GtkCTreeNode *row, gint column)
{
  gchar* strText;
  ITEM_DATA* pItemData;

  gtk_ctree_get_node_info (ctree, row, &strText, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "text")), strText);

  pItemData=(ITEM_DATA*)gtk_ctree_node_get_row_data (ctree, row);
  if (pItemData != NULL)
  {
    if (GTK_CTREE_ROW (row)->children)
    {
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "url")), "");

      if (current_applet != APPLET_TREEMENU)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_target")), FALSE);
    }
    else
    {
      if (!(current_applet == APPLET_TREEMENU 
	    && row == GTK_CTREE_NODE (GTK_CLIST (ctree)->row_list)))
      {
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "url")), pItemData->strURL);
      }
    }

    if (current_applet != APPLET_TREEMENU)
    {
      if (pItemData->bNewFrame)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_target")), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_target")), FALSE);

      if ((strcmp (pItemData->strNewFrameName, "_blank") == 0) ||
	  (strcmp (pItemData->strNewFrameName, "_self") == 0) ||
	  (strcmp (pItemData->strNewFrameName, "_parent") == 0) ||
	  (strcmp (pItemData->strNewFrameName, "_top") == 0))
      {
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "mode_entry")), pItemData->strNewFrameName);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "target_entry")), "");
	gtk_widget_set_sensitive (lookup_widget (main_dlg, "target_entry"), FALSE);
      }
      else
      {
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "mode_entry")), _("other target..."));
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "target_entry")), pItemData->strNewFrameName);
	gtk_widget_set_sensitive (lookup_widget (main_dlg, "target_entry"), TRUE);
      }
    }

    if (pItemData->bBack)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "back")), TRUE);
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "url")), "");
    }
    else
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "back")), FALSE);
    }

    if (current_applet == APPLET_TREEMENU)
    {
      if (current_applet != APPLET_TREEMENU)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_target")), FALSE);
      /*
      CString strIconFile;
      if (pItemData->iIconIndex >= 0 && m_astrIconList.GetSize() > pItemData->iIconIndex)
	strIconFile=m_astrIconList[pItemData->iIconIndex];
      SetDlgItemText(IDC_ICON_FILE, strIconFile );
      */

      gtk_widget_set_sensitive (lookup_widget (main_dlg, "remove"),
				(row == gtk_ctree_node_nth (ctree, 0)) ? FALSE : TRUE);
      gtk_widget_set_sensitive (lookup_widget (main_dlg, "sub"),
				(row == gtk_ctree_node_nth (ctree, 0)) ? FALSE : TRUE);
    }

    gtk_widget_set_sensitive (lookup_widget (main_dlg, "sub"), !pItemData->bBack);
    if (current_applet == APPLET_MORPHMENU)
      gtk_widget_set_sensitive (lookup_widget (main_dlg, "text"), !pItemData->bBack);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "url"),
			      (!(pItemData->bBack || (GTK_CTREE_ROW (row)->children))));

    if (current_applet == APPLET_TREEMENU)
      {
	//      gtk_widget_set_sensitive (lookup_widget (main_dlg, "enable_target"), FALSE);
      }
    else
      gtk_widget_set_sensitive (lookup_widget (main_dlg, "enable_target"), 
				(!pItemData->bBack && !(GTK_CTREE_ROW (row)->children)));
    /*
      OnNewFrame();
    */
  }
}

void OnBack (GtkWidget *widget, gpointer user_data)
{
  GtkCTree* ptc;
  GtkCTreeNode *sel;
  gboolean bBack=FALSE;
  ITEM_DATA* pItemData;

  ptc = GTK_CTREE (lookup_widget (main_dlg, "tree"));
  sel = gtk_ctree_node_nth (ptc, GTK_CLIST (ptc)->focus_row);
  //	sel=ptc->GetSelectedItem();
  //	CString strText;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
  {
    if (GTK_CTREE_ROW (sel)->children
	|| sel == GTK_CTREE_NODE (GTK_CLIST (ptc)->row_list)
	|| GTK_CTREE_ROW (sel)->parent == NULL)
    {
      message_box(_("Item has children(s) or it's root item, impossible to set \"back property\""), 0);
      gtk_window_set_focus (GTK_WINDOW (gtk_widget_get_toplevel (widget)), widget);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
      return;
    }

    if (current_applet == APPLET_WHEEL)
      gtk_ctree_node_set_text (ptc, sel, 0, _("BACK"));
    if (current_applet == APPLET_MORPHMENU)
      gtk_ctree_node_set_text (ptc, sel, 0, _("<---"));

    bBack=TRUE;
  }

  pItemData=(ITEM_DATA*)gtk_ctree_node_get_row_data (ptc, sel);
  if (pItemData != NULL)
    pItemData->bBack=bBack;

  tree_select_row (ptc, sel, 0);
  //  AggirnaItemSelezionato();
}

void OnApplyChange (GtkWidget *widget, gpointer user_data)
{
  GtkCTree* ptc;
  GtkCTreeNode *sel;
  ITEM_DATA* pItemData;
  gchar *strText;

  ptc = GTK_CTREE (lookup_widget (main_dlg, "tree"));
  sel = gtk_ctree_node_nth (ptc, GTK_CLIST (ptc)->focus_row);

  if( sel == NULL )
    return;

  strText = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "text")));
  gtk_ctree_node_set_text (ptc, sel, 0, strText);

  pItemData=(ITEM_DATA*)gtk_ctree_node_get_row_data (ptc, sel);
  if( pItemData != NULL )
  {
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_target"))) == TRUE)
      pItemData->bNewFrame=TRUE;
    else
      pItemData->bNewFrame=FALSE;

    strcpy (pItemData->strURL, gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "url"))));

    if (current_applet == APPLET_TREEMENU)
    {
      /*
	CString strIconImage;
			GetDlgItemText(IDC_ICON_FILE, strIconImage);
			pItemData->iIconIndex=-1;
			if( !strIconImage.IsEmpty() )
			{
				bool bFind=false;
				int size=m_astrIconList.GetSize();
				int index=0;
				for(index=0; index<size; index++)
				{
					if( m_astrIconList[index] == strIconImage)
					{
						bFind=true;
						break;
					}
				}

				if( bFind )
				{
					pItemData->iIconIndex = index;
				}
				else //Se non c'e' lo aggiungo
				{
					m_astrIconList.SetAtGrow(index,strIconImage);
					pItemData->iIconIndex=index;
				}
			}
    */
    }

    if (pItemData->bNewFrame)
    {
      /*
	GetDlgItemText( IDC_FRAME_MODE, pItemData->strNewFrameName);
	if( pItemData->strNewFrameName == g_strItemOther)
	  GetDlgItemText( IDC_LINK_FRAME, pItemData->strNewFrameName);
      */
    }
  }
}

extern GdkPixmap *folder0_pixmap, *folder1_pixmap, *item_pixmap;
extern GdkBitmap *folder0_mask, *folder1_mask, *item_mask;

void OnAddSubitem (GtkWidget *widget, gpointer user_data)
{
  GtkCTree* ptc;
  GtkCTreeNode *sel, *newItem;
  ITEM_DATA* pItemData;
  gchar *row[2] = { _("New item"), NULL };

  ptc = GTK_CTREE (lookup_widget (main_dlg, "tree"));
  sel = gtk_ctree_node_nth (ptc, GTK_CLIST (ptc)->focus_row);

  if (current_applet == APPLET_WHEEL)
    row[0] = _("NEW ITEM");
  if (current_applet == APPLET_MORPHMENU )
    row[0] = _("New item");

  {
    guint8 spacing;
    gboolean b;
    GdkPixmap *pm;
    GdkBitmap *bm;
    gchar* text;

    gtk_ctree_get_node_info (ptc, sel, &text, &spacing, &pm, &bm, &pm, &bm, &b, &b);
    text = g_strdup (text);
    gtk_ctree_set_node_info (ptc, sel, text, spacing, folder0_pixmap, folder0_mask,
			     folder1_pixmap, folder1_mask, FALSE, TRUE);
    g_free (text);
  }

  newItem = gtk_ctree_insert_node (ptc, sel, NULL, row, 5, item_pixmap, item_mask,
				   item_pixmap, item_mask, TRUE, FALSE);

  pItemData = malloc (sizeof (ITEM_DATA));
  if (pItemData != NULL)
  {
    pItemData->bNewFrame=FALSE;
    strcpy (pItemData->strNewFrameName, "");
    strcpy (pItemData->strURL, "");
    pItemData->bBack=FALSE;
    pItemData->iIconIndex=-1;
    pItemData->bDisableDropAfter=FALSE;

    gtk_ctree_node_set_row_data (ptc, newItem, pItemData);
  }

  gtk_ctree_expand (ptc, sel);

  /* Seleziono il nuovo item e aggiungo anche una icon di default
     ATTENZIONE l'impostazione dell'icon vale per 
     l'elemento attualemnte selezionato (che deve essere quello appena aggiunto) */
  gtk_ctree_select (ptc, newItem);
  if (current_applet == APPLET_TREEMENU)
  {
    /*
      SetDlgItemText(IDC_ICON_FILE, INI_MENU_ICONFILE_DEFAULT);
      // forzo un aggiornamento dell'item per memorizzare l'icona
      OnApplyChange();
    */
  }
}

void OnAdditemNode (GtkWidget *widget, gpointer user_data)
{
  GtkCTree* ptc;
  GtkCTreeNode *sel, *parent, *newItem;
  ITEM_DATA* pItemData;
  gchar *row[2] = { _("New item"), NULL };

  ptc = GTK_CTREE (lookup_widget (main_dlg, "tree"));
  sel = gtk_ctree_node_nth (ptc, GTK_CLIST (ptc)->focus_row);
  parent = GTK_CTREE_ROW (sel)->parent;

  if (current_applet == APPLET_WHEEL)
    row[0] = _("NEW ITEM");
  if (current_applet == APPLET_MORPHMENU )
    row[0] = _("New item");

  newItem = gtk_ctree_insert_node (ptc, parent, NULL, row, 5, item_pixmap, item_mask,
				   item_pixmap, item_mask, TRUE, FALSE);

  pItemData = malloc (sizeof (ITEM_DATA));
  if (pItemData != NULL)
  {
    pItemData->bNewFrame=FALSE;
    strcpy (pItemData->strNewFrameName, "");
    strcpy (pItemData->strURL, "");
    pItemData->bBack=FALSE;
    pItemData->iIconIndex=-1;
    pItemData->bDisableDropAfter=FALSE;

    gtk_ctree_node_set_row_data (ptc, newItem, pItemData);
  }

  /* Seleziono il nuovo item e aggiungo anche una icon di default
     ATTENZIONE l'impostazione dell'icon vale per 
     l'elemento attualemnte selezionato (che deve essere quello appena aggiunto) */
  gtk_ctree_expand (ptc, sel);
  gtk_ctree_select (ptc, newItem);
  if (current_applet == APPLET_TREEMENU)
  {
    /*
      SetDlgItemText(IDC_ICON_FILE, INI_MENU_ICONFILE_DEFAULT);
      // forzo un aggiornamento dell'item per memorizzare l'icona
      OnApplyChange();
    */
  }

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "text"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "url"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "sub"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "remove"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "enable_target"), TRUE);
}

void OnRemoveItem (GtkWidget *widget, gpointer user_data)
{
  GtkCTree* ptc;
  GtkCTreeNode *sel;
  ITEM_DATA* pItemData;

  ptc = GTK_CTREE (lookup_widget (main_dlg, "tree"));
  sel = gtk_ctree_node_nth (ptc, GTK_CLIST (ptc)->focus_row);

  //	m_bDeleting=TRUE;

  if (current_applet == APPLET_TREEMENU)
  {
    /*
		if( ptc->GetCount() == 1 || sel == ptc->GetRootItem() )
		{
			AfxMessageBox(IDS_ERR_DEL_ROOT,MB_ICONEXCLAMATION);
			return;
		}
    */
  }

  pItemData=(ITEM_DATA*)gtk_ctree_node_get_row_data (ptc, sel);
  free (pItemData);

  gtk_ctree_remove_node (ptc, sel);

  if (GTK_CLIST (ptc)->rows <=0 )
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "text"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "url"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "sub"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "remove"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "enable_target"), FALSE);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "enable_target")), FALSE);
    /*
      OnNewFrame();
    */
  }
}

void OnAutoapply (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "apply"), FALSE);
  else
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "apply"), TRUE);
}

void OnConfKillfocus (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "auto_apply"))) == TRUE)
    OnApplyChange (lookup_widget (main_dlg, "apply"), NULL);
}

void OnSelchangeBackType (GtkWidget *widget, gpointer user_data) 
{
  char* strItem;
  strItem = gtk_entry_get_text (GTK_ENTRY (widget));

  if (strcmp (strItem, "color") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_color"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), FALSE);
  }
  else if(strcmp (strItem, "blur") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_color"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), FALSE);
  }
  else if(strcmp (strItem, "image") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_color"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), TRUE);
  }
}

void OnTexture (GtkWidget *widget, gpointer user_data) 
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    return;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "texture_entry"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "wheel_color"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "text_color"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "select_color"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "extrude"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "text"), TRUE);
}

void OnGouraud (GtkWidget *widget, gpointer user_data) 
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    return;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "texture_entry"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "wheel_color"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "text_color"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "select_color"), TRUE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "extrude"), FALSE);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "text"), FALSE);
}

void OnSelchangeCharsetSel (GtkWidget *widget, gpointer user_data) 
{
  int i;
  char* str;

  str = gtk_entry_get_text (GTK_ENTRY (widget));

  for (i = 0; map_char_set[i].description != NULL; i++)
    if (strcmp (map_char_set[i].description, str) == 0)
    {
      gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "charset")), map_char_set[i].code);
      gtk_widget_set_sensitive (lookup_widget (main_dlg, "charset"), FALSE);
      return;
    }

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "charset"), TRUE);
}

void OnFireColor (GtkWidget *widget, gpointer user_data) 
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)dlg_firecolor_red/255;
  dbl[1] = (double)dlg_firecolor_green/255;
  dbl[2] = (double)dlg_firecolor_blue/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    dlg_firecolor_red = (int)(dbl[0]*255);
    dlg_firecolor_green = (int)(dbl[1]*255);
    dlg_firecolor_blue = (int)(dbl[2]*255);

    set_color_pixmap (lookup_widget (main_dlg, "fire_pixmap"),
		      dlg_firecolor_red, dlg_firecolor_green, dlg_firecolor_blue);
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnSeedColor (GtkWidget *widget, gpointer user_data) 
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_seedcolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_seedcolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_seedcolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_seedcolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "seed_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnSelBakcColor (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_selbackcolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_selbackcolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_selbackcolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_selbackcolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "selback_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnScrollColor (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_scrollcolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_scrollcolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_scrollcolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_scrollcolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "scroll_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnFilterColor (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_filtercolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_filtercolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_filtercolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_filtercolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "filter_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnSelectcolor (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_selectcolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_selectcolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_selectcolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_selectcolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "select_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnWheelcolor (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_wheelcolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_wheelcolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_wheelcolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_wheelcolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "wheel_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnTextcolor (GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)GetRValue(dlg_ul_textcolor)/255;
  dbl[1] = (double)GetGValue(dlg_ul_textcolor)/255;
  dbl[2] = (double)GetBValue(dlg_ul_textcolor)/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    {
      int r, g, b;

      r = (int)(dbl[0]*255);
      g = (int)(dbl[1]*255);
      b = (int)(dbl[2]*255);

      dlg_ul_textcolor = ((guint32)(((guint8)(r)|((guint16)((guint8)(g))<<8))|(((guint32)(guint8)(b))<<16)));
      set_color_pixmap (lookup_widget (main_dlg, "text_pixmap"), r, g, b);
    }
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnFloatmode (GtkWidget *widget, gpointer user_data)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    dlg_showfloat = 1;
  else
    dlg_showfloat = 0;
}

void OnBrowseFont (GtkWidget *widget, gpointer user_data)
{
  char szSourcePath[1024], strFontFile[1024];
  char strAppletDir[1024], strFontGifFile[1024];
  char *filename, *ptr;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  if (!UpdateFontProperties(filename))
  {
    message_box(_("Error ! Incorrect AnfyFont file format"), 0);
    g_free (filename);
    return;
  }

  GetAppletPath(current_applet,strAppletDir);
  PathFromFile(filename, szSourcePath);
  FileFromPath(filename, strFontFile);
  if( !MyCopyFile(szSourcePath, strAppletDir, strFontFile ) )
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    g_free (filename);
    return;
  }

  strcpy (strFontGifFile, strFontFile);
  ptr = strchr (strFontGifFile, '.');
  if (ptr != NULL)
    *ptr = 0;
  strcat (strFontGifFile, ".gif");

  if (!MyCopyFile (szSourcePath, strAppletDir, strFontGifFile))
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    g_free (filename);
    return;
  }

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "font")), strFontFile);

  g_free (filename);
}

void OnAnfyfontDown (GtkWidget *widget, gpointer user_data)
{
  StartBrowser(_("http://www.anfyteam.com/aft/"));
}

void OnTextFromFile (GtkWidget *widget, gpointer user_data)
{
  char *filename, *pScrollText;
  GtkText* w;
  guint pos;
  FILE *f;
  int len;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  f = fopen (filename, "rb");
  if (f == NULL)
  {
    message_box (_("Error reading textscroll file"), 0);
    g_free (filename);
  }

  fseek (f, 0, SEEK_END);
  len = ftell (f);
  rewind (f);

  pScrollText = g_malloc (len+1);
  len = fread (pScrollText, 1, len, f);
  pScrollText[len] = '\0';

  w = GTK_TEXT (lookup_widget (main_dlg, "text"));
  pos = gtk_text_get_length (w);
  gtk_text_set_point (w, pos);
  gtk_text_backward_delete (w, pos);
  gtk_text_insert (w, NULL, NULL, NULL, pScrollText, len);
  g_free (pScrollText);
  g_free (filename);
}

void OnLightsource (GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=FALSE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
    bEnable = TRUE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "intensity"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "light_min"), bEnable);
  gtk_widget_set_sensitive (lookup_widget (main_dlg, "light_max"), bEnable);
}

void UpdatePreviewScrollText ()
{
  gtk_widget_draw (lookup_widget (main_dlg, "preview"), NULL);
}

void OnChkShadow(GtkWidget *widget, gpointer user_data)
{
  gboolean bEnable=TRUE;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == FALSE)
    bEnable = FALSE;

  gtk_widget_set_sensitive (lookup_widget (main_dlg, "shadow_color"), bEnable);

  UpdatePreviewScrollText();
}

void OnColor(GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)dlg_textcolor_red/255;
  dbl[1] = (double)dlg_textcolor_green/255;
  dbl[2] = (double)dlg_textcolor_blue/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    dlg_textcolor_red = (int)(dbl[0]*255);
    dlg_textcolor_green = (int)(dbl[1]*255);
    dlg_textcolor_blue = (int)(dbl[2]*255);

    UpdatePreviewScrollText();
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnShadowcol(GtkWidget *widget, gpointer user_data)
{
  GtkWidget* dlg;
  double dbl[3];
  int ret = 0;

  dbl[0] = (double)dlg_textscolor_red/255;
  dbl[1] = (double)dlg_textscolor_green/255;
  dbl[2] = (double)dlg_textscolor_blue/255;

  dlg = gtk_color_selection_dialog_new ("Choose Color");
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);
  gtk_widget_show(dlg);
  gtk_grab_add(dlg);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->ok_button),
		      "clicked", (GtkSignalFunc)on_dlg_ok, &ret);
  gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_main ();

  if (ret == 1)
  {
    gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg)->colorsel), dbl);

    dlg_textscolor_red = (int)(dbl[0]*255);
    dlg_textscolor_green = (int)(dbl[1]*255);
    dlg_textscolor_blue = (int)(dbl[2]*255);

    UpdatePreviewScrollText();
  }

  gtk_grab_remove(dlg);
  gtk_widget_destroy(dlg);
}

void OnSelchangeTexttype(GtkWidget *widget, gpointer user_data)
{
  char* strItem;

  strItem = gtk_entry_get_text (GTK_ENTRY (widget));

  if (strcmp (strItem, "horizontal") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "min_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "max_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "v_spac"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_offset"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "sine_amplitude"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "jump_amplitude"), TRUE);
  }
  else if (strcmp (strItem, "vertical") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "min_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "max_combo"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "v_spac"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_offset"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "sine_amplitude"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "jump_amplitude"), FALSE);
  }
  else if ((strcmp (strItem, "zooming") == 0) || (strcmp (strItem, "invzooming") == 0))
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "min_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "max_combo"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "v_spac"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "y_offset"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "sine_amplitude"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "jump_amplitude"), FALSE);
  }

  UpdatePreviewScrollText();
}

void font_changed (GtkWidget *widget, gpointer user_data)
{
  //  fill_sizes_combo (lookup_widget (main_dlg, "size_combo"), gtk_entry_get_text (GTK_ENTRY (widget)));

  UpdatePreviewScrollText();
}

gint draw_preview (GtkWidget *widget, GdkEventExpose *event)
{
  GdkGC* gc;
  GdkColor c;
  GdkFont* font;
  char* str, name[256], *family, *weight, *slant, *size;
  int h;

  // Draw only last expose.
  if (event->count > 0)
    return TRUE;

  gdk_draw_rectangle (widget->window, widget->style->bg_gc[0], TRUE,
		      0, 0, widget->allocation.width, widget->allocation.height);

  gc = gdk_gc_new (widget->window);
  str = gtk_editable_get_chars (GTK_EDITABLE (lookup_widget (main_dlg, "text")), 0, -1);

  family = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "font_entry")));

  if (strcmp (family, "Courier") == 0)
    family = "courier";
  else if (strcmp (family, "Helvetica") == 0)
    family = "helvetica";
  else
    family = "times";

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "bold"))))
    weight = "bold";
  else
    weight = "medium";

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "italic"))))
    slant = "*";
  else
    slant = "r";

  size = gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, "size_entry")));

  sprintf (name, "-*-%s-%s-%s-*-*-%s-*-*-*-*-*-*-*", family, weight, slant, size);

  font = gdk_font_load (name);

  h = gdk_string_height (font, str);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "shadow"))) == TRUE)
  {
    c.red = (gushort)(dlg_textscolor_red*0xFF);
    c.green = (gushort)(dlg_textscolor_green*0xFF);
    c.blue = (gushort)(dlg_textscolor_blue*0xFF);
    gdk_color_alloc(gtk_widget_get_colormap(widget), &c);
    gdk_gc_set_foreground(gc, &c);
    gdk_draw_string (widget->window, font, gc, 1, h, str);
  }

  c.red = (gushort)(dlg_textcolor_red*0xFF);
  c.green = (gushort)(dlg_textcolor_green*0xFF);
  c.blue = (gushort)(dlg_textcolor_blue*0xFF);
  gdk_color_alloc(gtk_widget_get_colormap(widget), &c);
  gdk_gc_set_foreground(gc, &c);

  gdk_draw_string (widget->window, font, gc, 0, h, str);

  gdk_font_unref (font);
  gdk_gc_destroy(gc);

  return TRUE;
}

void OnSelchangeBackfx (GtkWidget *widget, gpointer user_data)
{
  char* strValue;

  strValue = gtk_entry_get_text (GTK_ENTRY (widget));

  if (strcmp (strValue, "color") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_color"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), FALSE);
  }
  else if (strcmp (strValue, "image") == 0)
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_color"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), TRUE);
  }
  else
  {
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_color"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (main_dlg, "back_image_browse"), FALSE);
  }
}

void OnBrowseTexture (GtkWidget *widget, gpointer user_data)
{
  char *filename, strWorkDir[1024], szImgFormat[10];
  char szSourcePath[1024], szFileName[1024];
  unsigned int iWidth, iHeight;

  file_browse ("Open", &filename);

  if (filename == NULL)
    return;

  GetAppletPath (current_applet, strWorkDir);

  if (GetFormatAndSize (filename, szImgFormat, &iWidth, &iHeight) == TRUE)
  {
    if (iWidth != 256 || iHeight != 256)
    {
      message_box(_("Only 256x256 GIF/JPG images allowed!"), 0);
      return;
    }
  }
			
  PathFromFile(filename, szSourcePath);
  FileFromPath(filename, szFileName);
  if (!MyCopyFile(szSourcePath, strWorkDir, szFileName))
  {
    message_box(_("Error ! Problem when copying file in applet directory"), 0);
    g_free (filename);
    return;
  }

  gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "texture")), szFileName);

  g_free (filename);
}
