//
// Misc. functions
//

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "main.h"
#include "jpeg.h"
#include "support.h"
#include "misc.h"
#include "custom.h"

#define _ttoi(s) strtol(s, NULL, 10)
#define MAX_SCROLLTEXT_SIZE         32767
#define HTML_HELP_ROOT          "/helproot.html"

void restore_dialog_data (int usedefault);
int validate_fields (GtkWidget *dlg);
void save_dialog_data ();
void file_browse (char* caption, char** filename);
char* convert_old_var (char* old);
gboolean SetAppletProject(const int applet);

// =========================================================

char* strupr(char* string)
{
  char *cp;
  for (cp=string; *cp; ++cp)
  {
    if ('a' <= *cp && *cp <= 'z')
      *cp += 'A' - 'a';
  }
 
  return string;
}

int SaveProject()
{
  char *strOutputFile, *tmp, *ptr, buf[256];
  FILE *file, *rc;

  file_browse ("Save", &strOutputFile);

  if (strOutputFile == NULL)
    return 0;

  if (!validate_fields(main_dlg))
  {
    g_free(strOutputFile);
    return 0;
  }

  save_dialog_data();

  file = fopen (strOutputFile, "w");
  g_free (strOutputFile);
  if (file == NULL)
    return 0;

  tmp = get_rc_path ();
  rc = fopen (tmp, "rt");
  g_free (tmp);

  while (fgets (buf, 256, rc) != 0)
  {
    if (buf[0] != '[')
      continue;
    strupr (buf);
    ptr = strchr (buf, ']');
    *ptr = '\0';

    if (strcmp (&buf[1], applet_info[current_applet].name) != 0)
      continue;

    fprintf (file, "[%s]\nVersion = 145\n", applet_info[current_applet].name);

    while (fgets (buf, 256, rc) != 0)
    {
      ptr = strchr (buf, '=');
      if (ptr == NULL)
	break;

      *ptr = '\0';
      tmp = ptr+2;

      while (strchr (ptr+1, '\"'))
	ptr = strchr (ptr+1, '\"');
      *ptr = '\0';

      fprintf (file, "%s = %s\n", convert_old_var (buf), tmp);
    }

    break;
  }

  fclose (rc);
  fclose (file);

  return 1;
}
char* convert_new_var (char* np);

int DoOpenProject(FILE* file)
{
  char buf[256], *ptr, *var;
  int i;

  while (fgets (buf, 256, file) != 0)
  {
    if (buf[0] != '[')
      continue;
    strupr (buf);
    ptr = strchr (buf, ']');
    *ptr = '\0';

    for (i = 0; i < NUMAPPLETS; i++)
      if (strcmp (applet_info[i].name, &buf[1]) == 0)
	break;

    if (i == NUMAPPLETS)
      return -1;

    if (current_dialog != DLG_STEP0)
    {
      if (current_applet != i)
	return 0;
    }
    current_applet = i;

    while (fgets (buf, 256, file) != 0)
    {
      ptr = strchr (buf, '=');
      if (ptr == NULL)
	continue;

      *ptr = 0;
      ptr++;
      g_strstrip (ptr);

      g_strstrip (buf);
      var = convert_new_var (buf);

      {
	char a[256];
	strcpy (a, buf);
	strupr (a);
	if (strcmp (a, "VERSION") == 0)
	  continue;
      }

      SaveVar(applet_info[current_applet].name, var, ptr);
    }
  }
  return 1;
}

#define PRJ_FILE_HEADER_14   "A J P 1 4"

int OpenProject14 (FILE* file)
{
  typedef struct _RegistryEntry
  {
    char     name[128];
    guint32  type;
    char     value[512];
    guint32  valueSize;
    char     filler[64];
  } RegistryEntry;

  int iRet = 0;
  RegistryEntry RegEntry;
  guint32 nBytesRead = sizeof (RegistryEntry);
  memset(&RegEntry, 0, sizeof (RegistryEntry));

  fread (&RegEntry, nBytesRead, 1, file);

  if (current_dialog == DLG_STEP0)
  {
    int i;
    for (i = 0; i < NUMAPPLETS; i++)
      if (strcmp (applet_info[i].name, RegEntry.value) == 0)
      {
	current_applet = i;
	break;
      }
  }

  if (strcmp(RegEntry.value, applet_info[current_applet].name) == 0)
  {
    while (fread (&RegEntry, nBytesRead, 1, file) == 1)
    {
      SaveVar(applet_info[current_applet].name, RegEntry.name, RegEntry.value);
    }
    iRet = 1;
  }
  else
    iRet = 0; 

  message_box(_("Warning ! The project file is in an old format, a resaving with the 'Save project' function is recommended"), 0);

  return iRet;
}

int OpenProject(char* a_pstrPathName)
{
  char* strPathName;
  int iRet = 0;
  FILE* file;

  if (a_pstrPathName == NULL)
  {
    file_browse ("Open", &strPathName);

    if (strPathName == NULL)
      return -1;
  }
  else
    strPathName = g_strdup (a_pstrPathName);

  file = fopen (strPathName, "r");
  g_free (strPathName);

  if (file != NULL)
  {
    char head[10];

    fread (head, 1, 10, file);
    rewind (file);

    if (strcmp (head, PRJ_FILE_HEADER_14) == 0)
      iRet = OpenProject14 (file);
    else
      iRet = DoOpenProject (file);

    fclose (file);
  }

  switch (iRet)
    {
    case -1:
      message_box(_("Problems during the project opening, verify file format"), 0);
      break;
    case 0:
      message_box(_("Problems during the project opening, verify that the file project is referred to the applet selected at the moment"), 0);
      break;
    case 1:
      if (current_dialog == DLG_STEP0)
      {
	if (SetAppletProject(current_applet))
	  {
	    /*
	    m_pStepDlg->UpdateData(FALSE);
	    SendMessage( WM_COMMAND, MAKEWPARAM( IDC_NEXT, BN_CLICKED ), MAKELPARAM(GetDlgItem(IDC_NEXT)->m_hWnd,0) );
	    */
	  }
	else
	  iRet=0;
      }
      restore_dialog_data(0);
      break;
    }

  return iRet;
}

void FindInArray (const char** aszArray, char* strItemPos)
{
  int i;

  for (i = 0; aszArray[i] != NULL; i++)
    {
      if (strcmp (strItemPos, aszArray[i]) == 0)
	{
	  sprintf (strItemPos,"%d",i);
	  return;
	}
    }
}

char* PathFromFile(char* szFilePath, char* szPath)
{
  int i, len;

  strcpy(szPath, szFilePath);
  len = strlen(szPath);
  for (i=len; i>=0; i--)
    {
      if (szPath[i]=='/')
	{
	  szPath[i]='\0';
	  break;
	}
    }

  return szPath;
}

char* FileFromPath(char* szFilePath, char* szFile)
{
  char szTMP[1024];
  int i;

  strcpy(szTMP, szFilePath);
  g_strreverse(szTMP);

  for(i=0; ;i++)
    {
      if(szTMP[i] == '/')
	break;
      else
	szFile[i]=szTMP[i];
    }

  szFile[i]='\0';
  g_strreverse(szFile);

  return szFile;
}

int CopyFile (char *s, char* d)
{
  FILE *src, *dst;
  void* buf;
  int l;

  src = fopen (s, "rb");
  if (!src)
    return 0;
  dst = fopen (d, "wb");
  if (!dst)
    {
      fclose (src);
      return 0;
    }

  fseek (src, 0, SEEK_END);
  l = ftell (src);
  rewind (src);

  buf = g_malloc (l);

  fread (buf, l, 1, src);
  fwrite (buf, l, 1, dst);

  g_free (buf);
  fclose (src);
  fclose (dst);

  return 1;
}

int MyCopyFile(char* szDirOri, char* szDirDest, char* szFileName)
{
  char strOldFile[1024], strNewFile[1024];
	
  strcpy (strOldFile, szDirOri);
  strcat (strOldFile, "/");
  strcat (strOldFile, szFileName);

  strcpy (strNewFile, szDirDest);
  strcat (strNewFile, "/");
  strcat (strNewFile, szFileName);

  if (strcmp (strNewFile, strOldFile) != 0)
  {
      if( !CopyFile(strOldFile,strNewFile) )
	{
	  /*
	  ShowSystemError();
//#ifdef _DEBUG
	  CString strMessage;
	  strMessage =  "Pathfile origine :";
	  strMessage += strOldFile;
	  strMessage += "\n";
	  strMessage += "Pathfile destinazione";
	  strMessage += strNewFile;
	  strMessage += "\n";
	  AfxMessageBox(strMessage);
//#endif
	  */
	  return 0;
	}
    }
  /*
    SetFileAttributes(strNewFile, FILE_ATTRIBUTE_ARCHIVE | FILE_ATTRIBUTE_NORMAL );
  */
  return 1;
}

#define LM_to_uint(a,b)    (((b)<<8)|(a))

int GetFormatAndSize(char* strPathFileName, char* szImgFormat, unsigned int* uiWidth, unsigned int* uiHeight)
{
  FILE* f;
  int ret = 0;
  szImgFormat[0]='\0';

  f = fopen (strPathFileName, "rb");
  if (f != NULL)
  {
    unsigned char buf[255];

    if (fread (buf, 1, 6, f) == 6)
    {
      if (strncmp((char*)buf, "GIF", 3) == 0)
      {
	char version[4];
	strncpy(version, (char *)buf + 3, 3);
	version[3] = '\0';

	if( strcmp(version,"87a")==0 || strcmp(version,"89a")==0 ) 
	{
	  fread(buf,1,7,f);
	  strcpy(szImgFormat,".gif");
	  *uiWidth = LM_to_uint(buf[0],buf[1]);
	  *uiHeight = LM_to_uint(buf[2],buf[3]);
	  ret=1;
	}
      }
    }
    fclose (f);
  }

  if (!ret)
  {
    if( GetJPEGSize(strPathFileName, uiWidth, uiHeight ) == 0)
      ret=0;
    else
    {
      strcpy(szImgFormat,".jpg");
      ret=1;
    }
  }
  /*
  if (!ret)
    message_box(IDS_ERR_IMAGE_FORMAT, MB_OK | MB_ICONEXCLAMATION );
  */
  return ret;
}

char* GetScrollTextFile (int applet)
{
  switch (applet)
  {
  case APPLET_ANFYBUMP:
    return "bumpscr.txt";
  case APPLET_ANFLAG:
    return "flagscr.txt";
  case APPLET_ANFRACT:
    return "fracscr.txt";
  case APPLET_ANLAKE:
    return "lakescr.txt";
  case APPLET_BLOBS:
    return "blobscr.txt";
  case APPLET_CUBEMENU :
    return "cbmescr.txt";
  case APPLET_DEFORM:
    return "defscr.txt";
  case APPLET_FIRE:
    return "firescr.txt";
  case APPLET_FLOZOIDS:
    return "flozscr.txt";
  case APPLET_FLUID:
    return "fluidscr.txt";
  case APPLET_HUEROT:
    return "huerscr.txt";
  case APPLET_LIFE2D:
    return "lifescr.txt";
  case APPLET_ROTATE:
    return "rotscr.txt";
  case APPLET_PLASMA:
    return "plasmscr.txt";
  case APPLET_ANTUNNEL:
    return "tunnscr.txt";
  case APPLET_TMAPCUBE:
    return "tcubescr.txt";
  case APPLET_ANVOXEL:
    return "voxelscr.txt";
  case APPLET_TUNNEL3D:
    return "tu3dscr.txt";
  case APPLET_ANWATER:
    return "waterscr.txt";
  case APPLET_WARP:
    return "warpscr.txt";
  case APPLET_WORMHOLE:
    return "wormscr.txt";
  case APPLET_FLAGLOAD:
    return "flaglscr.txt";
  case APPLET_GALAXY:
    return "galascr.txt";
  case APPLET_ANFY3D:
    return "anfy3dscr.txt";
  case APPLET_WOBBLER:
    return "wobscr.txt";
  case APPLET_TSCROLLER:
    return "txtscr.txt";
  case APPLET_SOLIDSCROLLER:
    return "solidscr1.txt";
  case APPLET_ANFYSNOW:
    return "snowscr.txt";
  case APPLET_FIREWORKS:
    return "firewkscr.txt";
  default:
    return "";
  }
}

char* GetJarFile (int applet)
{
  if (applet == APPLET_ANBLUR)
    return "anblur.jar";
  else if (applet == APPLET_ANFYBUMP)
    return "anfybump.jar";
  else if (applet == APPLET_ANFADE)
    return "AnFade.jar";
  else if (applet == APPLET_ANFLAG)
    return "AnFlag.jar";
  else if (applet == APPLET_ANFRACT)
    return "AnFract.jar";
  else if (applet == APPLET_ANLAKE)
    return "AnLake.jar";
  else if (applet == APPLET_ANLENS)
    return "AnLens.jar";
  else if (applet == APPLET_BLOBS)
    return "blobs.jar";
  else if (applet == APPLET_CUBEMENU)
    return "CubeMenu.jar";
  else if (applet == APPLET_DEFORM)
    return "Deform.jar";
  else if (applet == APPLET_FIRE)
    return "fire.jar";
  else if (applet == APPLET_FLAGLOAD)
    return "FlagLoad.jar";
  else if (applet == APPLET_FLOZOIDS)
    return "flozoids.jar";
  else if (applet == APPLET_FLUID)
    return "fluid.jar";
  else if (applet == APPLET_HUEROT)
    return "huerot.jar";
  else if (applet == APPLET_MANDEL)
    return "mandel.jar";
  else if (applet == APPLET_LIFE2D)
    return "life2d.jar";
  else if (applet == APPLET_ROTATE)
    return "Rot2.jar";
  else if (applet == APPLET_PLASMA)
    return "Plasma3.jar";
  else if (applet == APPLET_ANTUNNEL)
    return "tunnel.jar";
  else if (applet == APPLET_TMAPCUBE)
    return "TmapCube.jar";
  else if (applet == APPLET_ANVOXEL)
    return "voxel.jar";
  else if (applet == APPLET_TUNNEL3D)
    return "tunnel3d.jar";
  else if (applet == APPLET_ANWATER)
    return "AnWater.jar";
  else if (applet == APPLET_WARP )
    return "Warp.jar";
  else if (applet == APPLET_WORMHOLE)
    return "wormhole.jar";
  else if (applet == APPLET_GALAXY)
    return "galaxy.jar";
  else if (applet == APPLET_BOOKFLIP)
    return "bookflip.jar";
  else if (applet == APPLET_MOSAIC)
    return "mosaic.jar";
  else if (applet == APPLET_ANFY3D)
    return "anfy3d.jar";
  else if (applet == APPLET_WHEEL)
    return "wheelmenu.jar";
  else if (applet == APPLET_MORPHMENU)
    return "morphmenu.jar";
  else if (applet == APPLET_WOBBLER)
    return "wobbler.jar";
  else if (applet == APPLET_TSCROLLER)
    return "txtscroll.jar";
  else if (applet == APPLET_TREEMENU)
    return "treemenu.jar";
  else if (applet == APPLET_ANFYCAM)
    return "anfycam.jar";
  else if (applet == APPLET_ANFYPAINT)
    return "anfypaint.jar";
  else if (applet == APPLET_SOLIDSCROLLER)
    return "solidscr.jar";
  else if (applet == APPLET_ANFYSNOW)
    return "ansnow.jar";
  else if (applet == APPLET_FIREWORKS)
    return "firewks.jar";
  else
    return NULL;
}

void GetAppletPath (int applet, char* dir)
{
  strcpy (dir, g_strAppPath);
  strcat (dir, "/");
  strcat (dir, g_strAppletDir);
  strcat (dir, "/");

  switch (applet)
  {
  case APPLET_ANBLUR:
    strcat (dir, "blur");
    break;
  case APPLET_ANFYBUMP:
    strcat (dir, "bump");
    break;
  case APPLET_ANFADE:
    strcat (dir, "cfade");
    break;
  case APPLET_ANFLAG:
    strcat (dir, "flag");
    break;
  case APPLET_ANFRACT:
    strcat (dir, "ifsfract");
    break;
  case APPLET_ANLAKE:
    strcat (dir, "lake");
    break;
  case APPLET_ANLENS:
    strcat (dir, "lens");
    break;
  case APPLET_MANDEL:
    strcat (dir, "mandel2");
    break;
  case APPLET_ROTATE:
    strcat (dir, "rot");
    break;
  case APPLET_ANTUNNEL:
    strcat (dir, "tunnel");
    break;
  case APPLET_ANVOXEL:
    strcat (dir, "voxel");
    break;
  case APPLET_ANWATER:
    strcat (dir, "water");
    break;
  case APPLET_BLOBS:
    strcat (dir, "blobs");
    break;
  case APPLET_CUBEMENU:
    strcat (dir, "cubemenu");
    break;
  case APPLET_DEFORM  :
    strcat (dir, "deform");
    break;
  case APPLET_FIRE    :
    strcat (dir, "fire");
    break;
  case APPLET_FLAGLOAD:
    strcat (dir, "flagload");
    break;
  case APPLET_FLOZOIDS:
    strcat (dir, "flozoids");
    break;
  case APPLET_FLUID   :
    strcat (dir, "fluid");
    break;
  case APPLET_HUEROT  :
    strcat (dir, "huerot");
    break;
  case APPLET_LIFE2D  :
    strcat (dir, "life2d");
    break;
  case APPLET_PLASMA  :
    strcat (dir, "plasma");
    break;
  case APPLET_TMAPCUBE:
    strcat (dir, "tmapcube");
    break;
  case APPLET_TUNNEL3D:
    strcat (dir, "tunnel3d");
    break;
  case APPLET_WARP    :
    strcat (dir, "warp");
    break;
  case APPLET_WORMHOLE:
    strcat (dir, "wormhole");
    break;
  case APPLET_GALAXY:
    strcat (dir, "galaxy");
    break;
  case APPLET_BOOKFLIP:
    strcat (dir, "bookflip");
    break;
  case APPLET_MOSAIC:
    strcat (dir, "mosaic");
    break;
  case APPLET_ANFY3D:
    strcat (dir, "anfy3d");
    break;
  case APPLET_WHEEL:
    strcat (dir, "wheelmenu");
    break;
  case APPLET_MORPHMENU:
    strcat (dir, "morphmenu");
    break;
  case APPLET_WOBBLER:
    strcat (dir, "wobbler");
    break;
  case APPLET_TSCROLLER:
    strcat (dir, "txtscroll");
    break;
  case APPLET_TREEMENU:
    strcat (dir, "treemenu");
    break;
  case APPLET_ANFYCAM:
    strcat (dir, "anfycam");
    break;
  case APPLET_ANFYPAINT:
    strcat (dir, "anfypaint");
    break;
  case APPLET_SOLIDSCROLLER:
    strcat (dir, "solidscr");
    break;
  case APPLET_ANFYSNOW:
    strcat (dir, "ansnow");
    break;
  case APPLET_FIREWORKS:
    strcat (dir, "firewks");
    break;

  default:
    strcat (dir, applet_info[applet].name);
  }
}

const char* GetDemoFile (int applet)
{
  switch (applet)
  {
  case APPLET_ANBLUR:
    return "anblur.html";
  case APPLET_ANFYBUMP:
    return "anfybump.html";
  case APPLET_ANFADE:
    return "anfade.html";
  case APPLET_ANFLAG:
    return "anflag.html";
  case APPLET_ANFRACT:
    return "anfract.html";
  case APPLET_ANLAKE:
    return "anlake.html";
  case APPLET_ANLENS:
    return "anlens.html";
  case APPLET_BLOBS:
    return "blobs.html";
  case APPLET_CUBEMENU:
    return "cubemenu.html";
  case APPLET_DEFORM:
    return "deform.html";
  case APPLET_FIRE:
    return "fire.html";
  case APPLET_FLAGLOAD:
    return "flagload.html";
  case APPLET_FLOZOIDS:
    return "flozoids.html";
  case APPLET_FLUID:
    return "fluid.html";
  case APPLET_HUEROT:
    return "huerot.html";
  case APPLET_MANDEL:
    return "mandel.html";
  case APPLET_LIFE2D:
    return "life2d.html";
  case APPLET_ROTATE:
    return "rota.html";
  case APPLET_PLASMA:
    return "plasma.html";
  case APPLET_ANTUNNEL:
    return "antunnel.html";
  case APPLET_TMAPCUBE:
    return "tmapcube.html";
  case APPLET_ANVOXEL:
    return "anvoxel.html";
  case APPLET_TUNNEL3D:
    return "tunnel3d.html";
  case APPLET_ANWATER:
    return "anwater.html";
  case APPLET_WARP:
    return "warp.html";
  case APPLET_WORMHOLE:
    return "wormhole.html";
  case APPLET_GALAXY:
    return "galaxy.html";
  case APPLET_BOOKFLIP:
    return "bookflip.html";
  case APPLET_MOSAIC:
    return "mosaic.html";
  case APPLET_ANFY3D:
    return "anfy3d.html";
  case APPLET_WHEEL:
    return "wheelmenu.html";
  case APPLET_MORPHMENU:
    return "morphmenu.html";
  case APPLET_WOBBLER:
    return "wobbler.html";
  case APPLET_TSCROLLER:
    return "txtscroll.html";
  case APPLET_TREEMENU:
    return "treemenu.html";
  case APPLET_ANFYCAM:
    return "anfycam.html";
  case APPLET_ANFYPAINT:
    return "anfypaint.html";
  case APPLET_SOLIDSCROLLER:
    return "solidscr.html";
  case APPLET_ANFYSNOW:
    return "ansnow.html";
  case APPLET_FIREWORKS:
    return "firewks.html";

  default:
    return applet_info[applet].name;
  }
}

int ReadWriteFileScrollText (int applet, char* pScrollText, int mode)
{
  int bRet = 1;
  char strAppletDir[1024];
  FILE* f;
  int len;

  GetAppletPath (applet, strAppletDir);
  strcat (strAppletDir, "/");
  strcat (strAppletDir, GetScrollTextFile(applet));
	
  if (mode == 0)
  {
    f = fopen (strAppletDir, "rb");

    if (f == NULL)
    {
      bRet = 0;
      message_box(_("Error reading textscroll file"), 0);
    }
    else
    {
      fseek (f, 0, SEEK_END);
      len = ftell (f);
      rewind (f);

      if (len > MAX_SCROLLTEXT_SIZE)
	len = MAX_SCROLLTEXT_SIZE;

      len = fread (pScrollText, 1, len, f);
      pScrollText[len]='\0';
      fclose (f);
    }
  }
  else
  {
    f = fopen (strAppletDir, "wb");

    if (f == NULL)
    {
      bRet = 0;
      message_box (_("Error writing textscroll file"), 0);
    }
    else
    {
      len = strlen (pScrollText);
      fwrite (pScrollText, 1, len, f);
      fclose (f);
    }
  }

  return bRet;
}

int ShowHelp (char* szTopic)
{
  char strFile[1024];

  strcpy (strFile, g_strAppPath);
  strcat (strFile, "/");

  if (strcmp (szTopic, TOPIC_REG_HELP) == 0)
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/registration.html");
    return StartBrowser (strFile);
  }
  else if (strcmp (szTopic, TOPIC_SCRTEXT_HELP) == 0)
  {
    strcat (strFile, g_strAppletDir);
    strcat (strFile, "/scrtext.txt");
  }
  else if (strcmp (szTopic, TOPIC_TECHFAQ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/generalfaq.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, TOPIC_COPYRIGHT_HELP) == 0 )
  {
    strcat (strFile, g_strAppletDir);
    strcat (strFile, "/copyright.txt");
  }
  else if (strcmp (szTopic, TOPIC_INSTRUCTION) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, HTML_HELP_ROOT);
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANBLUR].name) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/blur.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFYBUMP].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/bump.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFADE].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/cfade.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFLAG].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/flag.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFRACT].name  ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/ifsfract.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANLAKE].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/lake.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANLENS].name	 ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/lens.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_BLOBS].name    ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/blobs.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_CUBEMENU].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/cmenu.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_DEFORM].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/deform.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_FIRE].name     ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/fire.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_FLAGLOAD].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/flag2.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_FLOZOIDS].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/flozoids.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_FLUID].name    ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/fluid.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_HUEROT].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/huerot.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_MANDEL].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/mandel.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_LIFE2D].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/life2d.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ROTATE].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/rot.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_PLASMA].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/plasma.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANTUNNEL].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/tunnel.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_TMAPCUBE].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/tcube.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANVOXEL].name	 ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/voxel.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_TUNNEL3D].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/tunnel2.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANWATER].name  ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/water.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_WARP].name     ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/warp.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_WORMHOLE].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/worm.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_GALAXY].name   ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/galaxy.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_BOOKFLIP].name ) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/bookflip.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_MOSAIC].name	) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/mosaic.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFY3D].name	) == 0 )
  {
      strcat (strFile, g_strHelpDir);
      strcat (strFile, "/anfy3dl.html");
      return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_WHEEL].name	) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/wheel.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_MORPHMENU].name) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/morph.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_WOBBLER].name	) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/wobbler.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_TSCROLLER].name) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/text.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_TREEMENU].name	) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/tree.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFYCAM].name	) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/anfycam.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFYPAINT].name) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/anfypaint.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_SOLIDSCROLLER].name) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/solidscr.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_ANFYSNOW].name) == 0 )	
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/ansnow.html");
    return StartBrowser(strFile);
  }
  else if (strcmp (szTopic, applet_info[APPLET_FIREWORKS].name) == 0 )
  {
    strcat (strFile, g_strHelpDir);
    strcat (strFile, "/fireworks.html");
    return StartBrowser(strFile);
  }

  {
    GtkWidget *dlg, *vbox, *w, *hbox, *t;
    char* txt;
    FILE* f;
    int len;

    dlg = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (dlg), _("Text"));
    gtk_window_set_default_size (GTK_WINDOW (dlg), 500, 400);

    vbox = gtk_vbox_new (FALSE, 5);
    gtk_widget_show (vbox);
    gtk_container_add (GTK_CONTAINER (dlg), vbox);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);

    t = gtk_text_new (NULL, NULL);
    gtk_widget_show (t);
    gtk_box_pack_start (GTK_BOX (vbox), t, TRUE, TRUE, 0);

    hbox = gtk_hbox_new (FALSE, 10);
    gtk_widget_show (hbox);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    w = gtk_button_new_with_label (_("OK"));
    gtk_widget_show (w);
    gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
    gtk_widget_set_usize (w, 75, 22);
    gtk_signal_connect_object (GTK_OBJECT (w), "clicked",
			       (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (dlg));

    f = fopen (strFile, "rb");
    if (f == NULL)
      return 0;

    fseek (f, 0, SEEK_END);
    len = ftell (f);
    rewind (f);

    txt = g_malloc (len+1);
    len = fread (txt, 1, len, f);
    txt[len] = '\0';

    gtk_text_insert (GTK_TEXT (t), NULL, NULL, NULL, txt, len);
    g_free (txt);

    gtk_widget_show (dlg);
  }

  return 1;
}

typedef struct _AnfyFontProp
{
  char name[1024];
  guint32 width;
  guint32 height;
  guint32 vertSpacing;
  char charOrder[1024];
  char description[1024];
} AnfyFontProp;

static int GetAnfyFontProp (char* pathFileFont, AnfyFontProp* afp)
{
  int bRet = 1;
  char strTmp[1024];
  FILE* f;

  f = fopen (pathFileFont, "rt");
  if (f == NULL)
    return 0;

  if (fgets (afp->name, 1024, f) == NULL)
    bRet=FALSE;

  if (fgets (strTmp, 1024, f) == NULL)
    bRet=FALSE;
  afp->width=_ttoi(strTmp);
  if ( afp->width == 0 )
    bRet=FALSE;

  if (fgets (strTmp, 1024, f) == NULL)
    bRet=FALSE;
  afp->height=_ttoi(strTmp);
  if( afp->height == 0 )
    bRet=FALSE;

  if (fgets (strTmp, 1024, f) == NULL)
    bRet=FALSE;
  afp->vertSpacing=_ttoi(strTmp);

  if (fgets (afp->charOrder, 1024, f) == NULL)
    bRet=FALSE;

  fgets (afp->description, 1024, f);
  fgets (strTmp, 1024, f);
  strcat (afp->description, "\r\n");
  strcat (afp->description, strTmp);

  fclose (f);

  return bRet;
}

int UpdateFontProperties (char* cstrFontFile)
{
  int bRet = 1;
  char buf[16];

  AnfyFontProp afp;
  if (GetAnfyFontProp (cstrFontFile, &afp))
  {
    sprintf (buf, "%d", afp.width);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), buf);
    sprintf (buf, "%d", afp.height);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), buf);
    sprintf (buf, "%d", afp.vertSpacing);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "v_spac")), buf);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "chars")), afp.charOrder);
    gtk_text_insert (GTK_TEXT (lookup_widget (main_dlg, "comment")), NULL, NULL, NULL,
    		     afp.description, strlen(afp.description));
    gtk_adjustment_set_value (GTK_TEXT(lookup_widget (main_dlg, "comment"))->vadj, 0);
  }
  else
  {
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "width")), "");
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "height")), "");
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "v_spac")), "");
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (main_dlg, "chars")), "");
    gtk_text_insert (GTK_TEXT (lookup_widget (main_dlg, "comment")), NULL, NULL, NULL,
		     "", strlen(0));
    bRet = 0;
  }

  return bRet;
}

char GetMenuLinkNext(char* strAppletName, char cActualLink)
{
  char nextLink = 0;

  if (strcmp (strAppletName, applet_info[APPLET_WHEEL].name) == 0)
  {
    nextLink=cActualLink+1;
  }
  else if (strcmp (strAppletName, applet_info[APPLET_MORPHMENU].name) == 0)
  {
    if( cActualLink >='0' && cActualLink < '9' )
      nextLink=cActualLink+1;
    else if( cActualLink == '9' )
      nextLink='a';
    else if( cActualLink >='a' && cActualLink < 'j' )
      nextLink=cActualLink+1;
  }

  return nextLink;
}

extern int mozilla_remote (char *command);

void execute_program (char *program, char *arg)
{
  pid_t pid;
 
  pid = fork ();
  if (pid == 0)
    {
      pid = fork ();
      if (pid == 0)
        {
          execlp (program, program, arg, NULL);
          _exit (0);
        }
      else
        {
          _exit (0);
        }
    }
 
}

int StartBrowser (char* szPageHtml)
{
  char url[1024];

  sprintf (url, "openURL(%s,new-window)", szPageHtml); 

  if (!mozilla_remote (url))
  {
    execute_program ("netscape", szPageHtml);
    return 0;
  }
  else
    return 1;

  message_box (_("Create process failed, verify browser setting"), 0);

  return 0;
  /*
	TCHAR szEXE[_MAX_PATH];
	CString strTMP;
	CString strDDEServerName;

	GetSavedVar(_T(""), INI_BROWSER, strTMP);
	if( strTMP == INI_DEF_BROWSER )
	{
		CString strHtmlFile;
		strHtmlFile = g_strAppPath;
		strHtmlFile += _T("\\");
		strHtmlFile += g_strHelpDir;
		strHtmlFile += HTML_HELP_ROOT;
		//trova il percoso del programma associato ai file di tipo html
		if( FindExecutable( strHtmlFile, NULL, szEXE ) <=(HINSTANCE)32 )
			return FALSE;
	}
	else
		//utilizza il browser specificato dall'utente
		lstrcpy(szEXE, strTMP);

	BOOL bNavigator=FALSE;

	if( _tcsstr(_tcsupr(szEXE), _T("NETSCAPE")) != NULL )
	{
		strDDEServerName = DDE_SNAME_NAVIGATOR ;
		bNavigator=TRUE;
	}
	else
		if( _tcsstr(_tcsupr(szEXE), _T("IEXPLORE")) != NULL )
			strDDEServerName = DDE_SNAME_IE ;
		else
			goto create_process;

	if( !ConnectDDE(strDDEServerName) )
		goto create_process;
	
	if( !ExecuteDDE(strDDEServerName, szPageHtml) )
		goto create_process;

	//se c'e' gia una finestra aperta la attiva e la porta in primo
	//piano
	if( g_hwndMainBrowser != NULL )
	{
		::ShowWindow(g_hwndMainBrowser, SW_SHOWNORMAL );
		::SetWindowPos(g_hwndMainBrowser, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE );
	}

	return TRUE;

create_process:
	DisconnectDDE();

	//se non era ancora stato avviato il browser lo avvia ora
	lstrcat(szEXE, _T(" "));
	lstrcat(szEXE, szPageHtml);
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	memset(&si, 0, sizeof(si));
	si.cb = sizeof(si);
	if( CreateProcess(	NULL, szEXE, 
						NULL,NULL,FALSE,NORMAL_PRIORITY_CLASS,NULL,NULL,
						&si,
						&pi
						) )
	{
		TRACE(_T("CreateProcess\n"));

		WaitForInputIdle( pi.hProcess, INFINITE ); 

		EnumThreadWindows( pi.dwThreadId, (WNDENUMPROC)EnumThreadWndProc, (LPARAM)&g_hwndMainBrowser );
		TRACE(_T("HWND borwser : 0x%x\n"),g_hwndMainBrowser);

		return TRUE;
	}
	else
		return FALSE;
  */
}

