//
// Get the velues from the dialog
//

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include "support.h"
#include "main.h"
#include "custom.h"

// =========================================================

#define CHECK_NULL(a) { \
  if (GTK_WIDGET_SENSITIVE(lookup_widget (dlg, a))) \
    if (!null_entry (lookup_widget (dlg, a))) break; }
#define CHECK_INT(a,b,c) { \
  if (GTK_WIDGET_SENSITIVE(lookup_widget (dlg, a))) \
    if (!check_int (lookup_widget (dlg, a), b, c)) break; }
#define CHECK_FLOAT(a,b,c) { \
  if (GTK_WIDGET_SENSITIVE(lookup_widget (dlg, a))) \
    if (!check_float (lookup_widget (dlg, a), b, c)) break; }

#define IS_CHECKED(w) gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, w)))
#define GET_TEXT(w) gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, w)))
#define ROWS(w) GTK_CLIST (lookup_widget (main_dlg, w))->rows
#define FOCUS(w) { GtkWidget *widget = lookup_widget (main_dlg, w); \
  gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget)), widget); }
#define GET_INT(w, i) sscanf (gtk_entry_get_text (GTK_ENTRY (lookup_widget (main_dlg, w))), "%d", &i)
#define EXIST(w) (GTK_WIDGET_SENSITIVE(lookup_widget (dlg, w)))

// =========================================================

static int check_int (GtkWidget* widget, int min_value, int max_value)
{
  int value;
  char buf[256];
 
  strcpy (buf, gtk_entry_get_text (GTK_ENTRY (widget)));
  if (sscanf(buf, "%d", &value) == 1)
  {
    if (value >= min_value && value <= max_value)
      return 1;
  }

  sprintf (buf, "Please enter a value between %d and %d", min_value, max_value);
  message_box (buf, 0);
  gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget)), widget);
 
  return 0;
}

static int check_float (GtkWidget* widget, float min_value, float max_value)
{
  float value;
  char buf[256];
 
  strcpy (buf, gtk_entry_get_text (GTK_ENTRY (widget)));
  if (sscanf(buf, "%f", &value) == 1)
  {
    if (value >= min_value && value <= max_value)
      return 1;
  }

  sprintf (buf, "Please enter a value between %f and %f", min_value, max_value);
  message_box (buf, 0);
  gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget)), widget);
 
  return 0;
}

static int null_entry (GtkWidget* widget)
{
  if (strlen (gtk_entry_get_text (GTK_ENTRY (widget))) > 0)
    return 1;

  message_box ("Error ! Field can't be null", 0);
  gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget)), widget);
 
  return 0;
}

static int ValidateTreeMenu (GtkCTree* ptc, GtkCTreeNode* item)
{
  ITEM_DATA* pItemData;
  GtkCTreeNode* nextItem;

  if (item == NULL)
    return 1;

  pItemData=(ITEM_DATA*)gtk_ctree_node_get_row_data (ptc, item);
  if (pItemData != NULL)
  {
    if( pItemData->iIconIndex <0 )
    {
      gtk_ctree_select (ptc, item);
      return -1;
    }

    nextItem = GTK_CTREE_ROW (item)->children;
    if (nextItem != NULL)
    {
      int iRet = ValidateTreeMenu( ptc, nextItem );
      if (iRet <= 0)
	return iRet;
    }

    nextItem = GTK_CTREE_ROW (item)->sibling;
    if (nextItem != NULL)
    {
      int iRet = ValidateTreeMenu (ptc, nextItem);
      if (iRet <= 0)
	return iRet;
    }
  }
  else
    return 0;

  return 1;
}

static guint32 m_nNode;
static int ValidateTreeCtrl (GtkCTree* ptc, GtkCTreeNode* item, guint32 nItem)
{
  ITEM_DATA* pItemData;
  GtkCTreeNode* nextItem;

  if (item == NULL)
    return 1;

  nItem++;

  if (current_applet == APPLET_WHEEL)
  {
    if (m_nNode > 8) 
      return -1;
    if (nItem > 8)
      return -3;
  }

  if (current_applet == APPLET_MORPHMENU)
  {
    if (m_nNode > 20)
      return -1;
    if (nItem > 20)
      return -3;

    //controllo all'inizio, che in ogni nodo che ci sia almeno un back
    if (nItem == 1 && item != gtk_ctree_node_nth (ptc, 0))
    {
      guint32 nBackInNode = 0;
      ITEM_DATA* pItemDataTmp;
      GtkCTreeNode* next = item;

      while (next != NULL)
      {
	pItemDataTmp = (ITEM_DATA*)gtk_ctree_node_get_row_data (ptc, next);

	if (pItemDataTmp != NULL)
	{
	  if (pItemDataTmp->bBack)
	    nBackInNode++;
	}
	else
	  return 0;

	next = GTK_CTREE_ROW (next)->sibling;
      }
      if (nBackInNode <= 0)
	return -4;
    }
  }

  pItemData = (ITEM_DATA*)gtk_ctree_node_get_row_data (ptc, item);
  if (pItemData != NULL)
  {
    if (pItemData->bBack)
    {
      //verifico che allo stesso livello non ci siano altri 'item back'
      ITEM_DATA* pItemDataTmp;
      GtkCTreeNode* next = GTK_CTREE_ROW (item)->sibling;

      while (next != NULL)
      {
	pItemDataTmp = (ITEM_DATA*)gtk_ctree_node_get_row_data (ptc, next);
	if (pItemDataTmp != NULL)
	{
	  if (pItemDataTmp->bBack)
	    return -2;
	}
	else
	  return 0;

	next = GTK_CTREE_ROW (next)->sibling;
      }
    }

    nextItem = GTK_CTREE_ROW (item)->children;
    if (nextItem != NULL)
    {
      int iRet;
      m_nNode++;
      iRet=ValidateTreeCtrl( ptc, nextItem, 0 );
      if( iRet<=0 )
	return iRet;
    }

    nextItem = GTK_CTREE_ROW (item)->sibling;
    if (nextItem != NULL)
    {
      int iRet=ValidateTreeCtrl( ptc, nextItem, nItem );
      if( iRet<=0 )
	return iRet;
    }
  }
  else
    return 0;

  return 1;
}

// =========================================================

int validate_fields (GtkWidget *dlg)
{
  int ret = 0, val1, val2;

  switch (current_dialog)
  {
  case DLG_TEXTSTYLE:
    CHECK_NULL ("v_spac");
    CHECK_NULL ("speed");
    CHECK_NULL ("y_offset");
    CHECK_NULL ("sine_angle");
    CHECK_NULL ("sine_amplitude");
    CHECK_NULL ("sine_speed");
    CHECK_NULL ("jump_amplitude");
    CHECK_NULL ("jump_speed");

    if (EXIST ("min_entry"))
    {
      GET_INT ("min_entry", val1);
      GET_INT ("max_entry", val2);

      if (val1 >= val2)
      {
	message_box (_("Font's max size must be greater than min. size"), 0);
	FOCUS ("min_entry");
	break;
      }
    }
    ret = 1;
    break;

  case DLG_EXPERTMODE:
  {
    CHECK_NULL ("memory");
    CHECK_NULL ("task");
    CHECK_NULL ("minsync");

    if (IS_CHECKED ("enable_image"))
      CHECK_NULL ("image");

    CHECK_NULL ("xoffset");
    CHECK_NULL ("yoffset");

    ret = 1;
  } break;

  case DLG_REGISTRATION:
    CHECK_NULL ("mode_entry");
    CHECK_NULL ("target_entry");

    ret = 1;
    break;

  case DLG_ANBLUR:
    CHECK_NULL ("frames");
    CHECK_NULL ("play");
    CHECK_NULL ("num");
    CHECK_NULL ("pong_pause");
    CHECK_NULL ("ping_pause");

    ret = 1;
    break;

  case DLG_ANFADE:

    if (ROWS ("image_list") < 2 )
    {
      message_box(_("Error ! At least 2 pictures needed"), 0);
      FOCUS ("add");
      break;
    }

    CHECK_INT ("frame_speed", 1, 255);
    CHECK_NULL ("pause");

    ret = 1;
    break;

  case DLG_ANFLAG:
    CHECK_INT ("speed", 1, 8);

    ret = 1;
    break;

  case DLG_ANFYBUMP:
    CHECK_NULL ("image");
    CHECK_NULL ("anglex");
    CHECK_NULL ("angley");
    CHECK_INT ("power", 1, 100);
    CHECK_INT ("min", 1, 511);
    CHECK_INT ("max", 1, 511);
    CHECK_NULL ("bump_image");

    ret = 1;
    break;

  case DLG_BLOBS:
    CHECK_NULL ("i");
    CHECK_NULL ("j");
    CHECK_NULL ("k");

    ret = 1;
    break;

  case DLG_TMAPCUBE:
    CHECK_INT ("intensity", 1, 100);
    CHECK_INT ("light_min", 1, 511);
    CHECK_INT ("light_max", 1, 511);

    //attenzione non mettere il break tmapcube e' sovrainsieme di cubemenu
  case DLG_CUBEMENU:

    if (ROWS ("image_list") < 6)
    {
      message_box (_("Error ! Images must be 6"), 0);
      FOCUS ("add");
      break;
    }

    CHECK_NULL ("x");
    CHECK_NULL ("y");
    CHECK_NULL ("z");

    ret = 1;
    break;

  case DLG_DEFORM:
    CHECK_INT ("value", 0, 500);
    CHECK_INT ("factor", 1, 100);
    CHECK_INT ("speed", 1, 50);

    GET_INT ("width", val2);
    GET_INT ("offset", val1);

    if (val1 < 0 || val1 > val2)
    {
      message_box (_("Error ! The parameter range is 0 to image width"), 0);
      FOCUS ("offset");
      break;
    }

    CHECK_INT ("x1", 1, 19);
    CHECK_INT ("x2", 1, 19);
    CHECK_INT ("y1", 1, 19);
    CHECK_INT ("y2", 1, 19);

    ret = 1;
    break;

  case DLG_FIRE:
    CHECK_INT ("def", 0, 127);
    CHECK_INT ("start", 1, 127);
    CHECK_INT ("value", 1, 127);

    GET_INT ("start", val2);
    GET_INT ("value", val1);

    if (val1 > val2)
    {
      message_box (_("Error ! Fire cut value can't be greater than fire start value"), 0);
      FOCUS ("start");
      break;
    }

    ret = 1;
    break;

  case DLG_FLAGLOAD:
    CHECK_NULL ("image");
    CHECK_NULL ("speed");
    CHECK_NULL ("back_image");

    ret = 1;
    break;

  case DLG_FLOZOIDS:
    CHECK_NULL ("number");
    CHECK_NULL ("distance");
    CHECK_NULL ("center");
    CHECK_NULL ("acceleration");
    CHECK_NULL ("speed");
    CHECK_NULL ("bouncing");

    ret = 1;
    break;

  case DLG_FLUID:
    CHECK_NULL ("rsize");
    CHECK_NULL ("rfactor");
    CHECK_NULL ("ssize");
    CHECK_NULL ("sfactor");

    ret = 1;
    break;

  case DLG_HUEROT:
    CHECK_NULL ("hueshift");
    CHECK_INT ("hueshift", 0, 1000);

    ret = 1;
    break;

  case DLG_ANFRACT:
    CHECK_NULL ("dimension");
    CHECK_NULL ("branches");
    CHECK_NULL ("morph");

    CHECK_INT ("rotation", -99, 99);
    CHECK_INT ("rotation2", -99, 99);

    CHECK_NULL ("decline");
    /*
    strValue = GET_TEXT ("decline");

    if( _tcstod(strValue, NULL) == 0 && ( strValue.GetLength()>1 || strValue[0] != '0' ) )
      {
	GetDlgItem(IDC_DECLINE)->SetFocus();
	AfxMessageBox(IDS_ERR_INVALID_VALUE, MB_ICONEXCLAMATION);
	break;
      }
    */
    CHECK_NULL ("length");
    CHECK_NULL ("pen_progr");
    CHECK_NULL ("back_image");

    {
      char strImageFile[1024], strImgFormat[10], *strTmp;
      int iWidth, iWidthBack, iHeight, iHeightBack;

      strTmp = GET_TEXT ("back_image");
      GetAppletPath (current_applet, strImageFile);
      strcat (strImageFile, "/");
      strcat (strImageFile, strTmp);

      if (GetFormatAndSize( strImageFile, strImgFormat, &iWidthBack, &iHeightBack))
      {
	GET_INT ("width", iWidth);
	GET_INT ("height", iHeight);

	if (iWidth != iWidthBack || iHeight != iHeightBack)
	{
	  message_box(_("Error ! The background image width and height must equal the applet size"), 0);
	  break;
	}
      }
    }

    ret = 1;
    break;

  case DLG_ANLAKE:
    CHECK_NULL ("image");

    CHECK_INT ("wind_speed", 0, 200);
    CHECK_INT ("wave", 1, 200);
    CHECK_INT ("perspective", 1, 100);
    CHECK_INT ("farwaving", 1, 10000);

    ret = 1;
    break;

  case DLG_ANLENS:
    CHECK_NULL ("image");
    CHECK_NULL ("lens_width");

    {
      int iWidth, iHeight, iMax;

      GET_INT ("width", iWidth);
      GET_INT ("height", iHeight);
      GET_INT ("lens_width", val1);

      iMax = (iWidth < iHeight) ? iWidth : iHeight;
      if (val1  > ( iMax / 2 ) )
      {
	char msg[100];
	sprintf (msg, _("Error ! Value can't be grater than %d"), (iMax / 2));
	message_box (msg, 0);
	FOCUS ("lens_width");
	break;
      }
    }

    CHECK_INT ("zoom", 0, 30);
    CHECK_INT ("xy", -40, 40);
			
    ret = 1;
    break;

  case DLG_MANDEL:
    CHECK_NULL ("startx");
    CHECK_NULL ("starty");

    CHECK_INT ("xmin", -99999, 99999);
    CHECK_INT ("xmax", -99999, 99999);
    CHECK_INT ("ymin", -99999, 99999);
    CHECK_INT ("ymax", -99999, 99999);

    ret = 1;
    break;

  case DLG_PLASMA:
    CHECK_INT ("gen1", 8, 2000);
    CHECK_INT ("gen2", 3, 256);
    CHECK_INT ("gen3", 3, 256);
    CHECK_INT ("r0", 0, 255);
    CHECK_INT ("r1", 0, 255);
    CHECK_INT ("g0", 0, 255);
    CHECK_INT ("g1", 0, 255);
    CHECK_INT ("b0", 0, 255);
    CHECK_INT ("b1", 0, 255);

    ret = 1;
    break;

  case DLG_ROTATE:
    CHECK_NULL ("image");
    CHECK_INT ("zoom_min", 0, 80);
    CHECK_INT ("zoom_max", 0, 80);
    CHECK_INT ("zoom_speed", 1, 20);
    CHECK_INT ("first", 0, 10000);
    CHECK_INT ("second", 0, 10000);
    CHECK_INT ("distortion", 1, 255);

    ret = 1;
    break;

  case DLG_TUNNEL:
    CHECK_NULL ("image");
    CHECK_NULL ("x");
    CHECK_NULL ("y");
    CHECK_NULL ("z");
    CHECK_NULL ("startx");
    CHECK_NULL ("starty");

    ret = 1;
    break;

  case DLG_TUNNEL3D:
    CHECK_NULL ("image");
    CHECK_NULL ("radius");
    CHECK_NULL ("rotation");
    CHECK_NULL ("advancing");

    CHECK_INT ("light_min", 1, 127);
    CHECK_INT ("light_max", 1, 127);

    CHECK_NULL ("light_dec");
    CHECK_NULL ("bends_num");
    CHECK_NULL ("bends_min");
    CHECK_NULL ("bends_max");
    CHECK_NULL ("bends_offset");
    CHECK_NULL ("bends_offset_inc");
    CHECK_NULL ("bends_size_inc");
    CHECK_NULL ("x");
    CHECK_NULL ("y");
    CHECK_NULL ("z");

    ret = 1;
    break;

  case DLG_VOXEL:
    CHECK_NULL ("shadow");
    CHECK_INT ("roughness", 50, 200);
    CHECK_INT ("horizon", 0, 500);

    ret = 1;
    break;

  case DLG_WARP:
    CHECK_NULL ("image");
    CHECK_INT ("spiral_start", -10000, 10000);
    CHECK_INT ("spiral_add", 0, 250);
    CHECK_INT ("spiral_min", -10000, 10000);
    CHECK_INT ("spiral_max", -10000, 10000);
    CHECK_INT ("breath_start", -2000, 2000);
    CHECK_INT ("breath_add", 0, 25);
    CHECK_INT ("breath_min", -2000, 2000);
    CHECK_INT ("breath_max", -2000, 2000);

    ret = 1;
    break;

  case DLG_ANWATER:
    CHECK_NULL ("rsize");
    CHECK_NULL ("rfactor");
    CHECK_NULL ("ssize");
    CHECK_NULL ("sfactor");
    CHECK_NULL ("factor");

    if (IS_CHECKED ("fix_drops"))
    {
      CHECK_NULL ("fix_x");
      CHECK_NULL ("fix_y");
      CHECK_NULL ("fix_size");
      CHECK_NULL ("fix_factor");
    }

    if (IS_CHECKED ("light"))
    {
      CHECK_INT ("min", -255, 255);
      CHECK_INT ("max", -255, 255);
    }

    CHECK_INT ("pressure", 1, 2000);

    ret = 1;
    break;

  case DLG_WORMHOLE:
    CHECK_INT ("dot", 1, 23);

    ret = 1;
    break;

  case DLG_GALAXY:
    {
      char* str = GET_TEXT ("dynamic");

      if (str && strlen (str))
      {
	/*
				if( _tcstod(strValue, NULL) == 0 
					&& ( strValue.GetLength()>1 || strValue[0] != '0' ) 
					&& strValue != _T("0.0"))
				{
					GetDlgItem(IDC_DINPERSIST)->SetFocus();
					AfxMessageBox(IDS_ERR_INVALID_VALUE, MB_ICONEXCLAMATION);
					break;
				}
	*/
      }

      ret = 1;
    } break;

  case DLG_BOOKFLIP:

    if (ROWS ("image_list") < 2 )
    {
      message_box (_("Error ! At least 2 pictures needed"), 0);
      FOCUS ("add");
      break;
    }

    CHECK_INT ("frame_speed", 1, 255);
    CHECK_NULL ("pause");
    CHECK_NULL ("extra_height");
    ret = 1;
  break;

  case DLG_MOSAIC:

    if (ROWS ("image_list") < 2 )
    {
      message_box (_("Error ! At least 2 pictures needed"), 0);
      FOCUS ("add");
      break;
    }

    CHECK_NULL ("pause");

    GET_INT ("width", val2);
    GET_INT ("tile_width", val1);

    if (val1 < 1 || val1 > (val2/3))
    {
      message_box (_("Error ! The minimum value is 1 and maximum value must be image width/3"), 0);
      FOCUS ("tile_width");
      break;
    }

    GET_INT ("height", val2);
    GET_INT ("tile_height", val1);

    if (val1 <1 || val1 > (val2/3))
    {
      message_box (_("Error ! The minimum value is 1 and maximum value must be image height/3"), 0);
      FOCUS ("tile_height");
      break;
    }

    CHECK_INT ("tile_steps", 8, 32);
    CHECK_NULL ("back_image");

    ret = 1;
    break;

  case DLG_ANFY3D:
    CHECK_NULL ("real_w");
    CHECK_NULL ("real_h");
    CHECK_NULL ("t0_entry");
    CHECK_NULL ("t1_entry");
    CHECK_NULL ("t2_entry");
    CHECK_NULL ("p_entry");
    CHECK_NULL ("g_entry");

    CHECK_INT ("start", 0, 1000);
    CHECK_INT ("stop", 0, 1000);

    if (EXIST ("render_entry"))
    {
      /*
	0  Plain texture
	1  Gouraud shading
	2  Phong shading          ** NOTE: "phongmap" must be loaded
	3  Gaussian phong shading ** NOTE: "gaussmap" must be loaded
	4  Transparent (light)
	5  Transparent (medium)
	6  Liquid texture
	7  Liquid gouraud shading
	8  Liquid phong shading    ** NOTE: "phongmap" must be loaded
	9  Liquid transparent
	10 Reflects mirror view    ** NOTE: texture is ignored, a mirror must exists
	11 Particles               ** NOTE: texture is ignored
	12 Darker plain texture
	13 Metal shading           ** NOTE: "phongmap" must be loaded
	14 Environment mapping
      */

      char* str = GET_TEXT ("render_entry");

      if ((strcmp (str, "Phong shading") == 0) || (strcmp (str, "Liquid phong shading") == 0) ||
	  (strcmp (str, "Metal shading") == 0))
	if (strlen (GET_TEXT ("p_entry")) == 0)
	{
	  message_box (_("\"Phong map\" must be set"), 0);
	  FOCUS ("p_entry");
	  break;
	}

      if (strcmp (str, "Gaussian phong shading") == 0)
	if (strlen (GET_TEXT ("g_entry")) == 0)
	{
	  message_box (_("\"Gauss map\" must be set"), 0);
	  FOCUS ("g_entry");
	  break;
	}

      if ((strcmp (GET_TEXT ("script_entry"), "Objects") != 0) && 
	  (strcmp (str, "Reflects mirror view") == 0))
      {
	message_box (_("Reflects mirror view require script 'Objects'"), 0);
	FOCUS ("render_entry");
	break;
      }

      if (EXIST ("bt_entry"))
      {
	str = GET_TEXT ("bt_entry");

	if (strcmp (str, "1") == 0)
	  if (strlen (GET_TEXT ("t1_entry")) == 0)
	  {
	    message_box (_("\"Texture 1\" must be set"), 0);
	    break;
	  }

	if (strcmp (str, "2") == 0)
	  if (strlen (GET_TEXT ("t2_entry")) == 0)
	  {
	    message_box (_("\"Texture 2\" must be set"), 0);
	    break;
	  }
      }
    }

    ret = 1;
    break;

  case DLG_ANFY3D_0:
    {
      char strTmp[256], *str;

      str = GET_TEXT ("main_render_entry");
      GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_PHONGMAP, strTmp);
      if (strlen (strTmp) == 0)
	if ((strcmp (str, "Phong shading") == 0) || (strcmp (str, "Liquid phong shading") == 0) ||
	    (strcmp (str, "Metal shading") == 0))
	  {
	    message_box (_("\"Phong map\" must be set, to set go back to previous step"), 0);
	    break;
	  }

      GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_GPHONGMAP, strTmp);
      if (strlen (strTmp) == 0)
	if (strcmp (str, "Gaussian phong shading") == 0)
	  {
	    message_box (_("\"Gauss map\" must be set, to set go back to previous step"), 0);
	    break;
	  }


      str = GET_TEXT ("child_render_entry");
      GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_PHONGMAP, strTmp);
      if (strlen (strTmp) == 0)
	if ((strcmp (str, "Phong shading") == 0) || (strcmp (str, "Liquid phong shading") == 0) ||
	    (strcmp (str, "Metal shading") == 0))
	  {
	    message_box (_("\"Phong map\" must be set, to set go back to previous step"), 0);
	    break;
	  }

      GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_GPHONGMAP, strTmp);
      if (strlen (strTmp) == 0)
	if (strcmp (str, "Gaussian phong shading") == 0)
	  {
	    message_box (_("\"Gauss map\" must be set, to set go back to previous step"), 0);
	    break;
	  }

      str = GET_TEXT ("child_tex_entry");
      if (strcmp (str, "1") == 0)
      {
	GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_TEXTURE1, strTmp);
	if (strlen (strTmp) == 0)
	{
	  message_box (_("\"Texture 1\" must be set, to set go back to previous step"), 0);
	  break;
	}
      }
      if (strcmp (str, "2") == 0)
      {
	GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_TEXTURE1, strTmp);
	if (strlen (strTmp) == 0)
	{
	  message_box (_("\"Texture 2\" must be set, to set go back to previous step"), 0);
	  break;
	}
      }

      str = GET_TEXT ("main_tex_entry");
      if (strcmp (str, "1") == 0)
      {
	GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_TEXTURE1, strTmp);
	if (strlen (strTmp) == 0)
	{
	  message_box (_("\"Texture 1\" must be set, to set go back to previous step"), 0);
	  break;
	}
      }
      if (strcmp (str, "2") == 0)
      {
	GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_TEXTURE1, strTmp);
	if (strlen (strTmp) == 0)
	{
	  message_box (_("\"Texture 2\" must be set, to set go back to previous step"), 0);
	  break;
	}
      }

      str = GET_TEXT ("light_entry");
      GetSavedVar(applet_info[APPLET_ANFY3D].name, INI_PHONGMAP, strTmp);
      if (strlen (strTmp) == 0)
	if (strcmp (str, "0") != 0)
	{
	  message_box(_("\"Phong map\" must be set, to set go back to previous step"), 0);
	  break;
	}
    }

    CHECK_FLOAT ("x", 0, 2.0);
    CHECK_FLOAT ("y", 0, 2.0);
    CHECK_FLOAT ("z", 0, 2.0);
    CHECK_FLOAT ("add", 0, 3.14);
    CHECK_NULL ("height");
    CHECK_NULL ("width");
    CHECK_NULL ("height_faces");
    CHECK_NULL ("width_faces");
    CHECK_NULL ("ampl");
    CHECK_NULL ("p1");
    CHECK_NULL ("p2");

    ret = 1;
    break;

  case DLG_TSCROLLER:
    CHECK_NULL ("back_image");

    ret = 1;
    break;

  case DLG_WOBBLER:
    CHECK_NULL ("image");

    if (strlen (GET_TEXT ("dist_initial")) > 0)
      CHECK_FLOAT ("dist_initial", 0.01, 500)
    else
      CHECK_NULL ("dist_initial");

    if (strlen (GET_TEXT ("zoom")) > 0)
      CHECK_FLOAT ("zoom", 0.0, 50)
    else
      CHECK_NULL ("zoom");

    if (strlen (GET_TEXT ("dist_min")) > 0)
      CHECK_FLOAT ("dist_min", 0.01, 500)
    else
      CHECK_NULL ("dist_min");

    if (strlen (GET_TEXT ("dist_max")) > 0)
      CHECK_FLOAT ("dist_max", 0.01, 500)
    else
      CHECK_NULL ("dist_max");

    if (strlen (GET_TEXT ("radius")) > 0)
      CHECK_FLOAT ("radius", 0.0, 999)
    else
      CHECK_NULL ("radius");

    if (strlen (GET_TEXT ("wave")) > 0)
      CHECK_FLOAT ("wave", 0.0, 999)
    else
      CHECK_NULL ("wave");

    if (strlen (GET_TEXT ("speed1")) > 0)
      CHECK_FLOAT ("speed1", 0.0, 999)
    else
      CHECK_NULL ("speed1");

    if (strlen (GET_TEXT ("speed2")) > 0)
      CHECK_FLOAT ("speed2", 0.0, 999)
    else
      CHECK_NULL ("speed2");

    if (strlen (GET_TEXT ("speed3")) > 0)
      CHECK_FLOAT ("speed3", 0.0, 999)
    else
      CHECK_NULL ("speed3");

    if (strlen (GET_TEXT ("speed4")) > 0)
      CHECK_FLOAT ("speed4", 0.0, 999)
    else
      CHECK_NULL ("speed4");

    CHECK_INT ("movex", 0, 1000);
    CHECK_INT ("movey", 0, 1000);
    CHECK_NULL ("start");
    CHECK_INT ("move", 0, 1000);

    ret = 1;
    break;

  case DLG_CONF_MENU:
    {
      GtkCTree* ptc = GTK_CTREE (lookup_widget (main_dlg, "tree"));

      if (ROWS ("tree") <= 0)
      {
	message_box (_("Error ! Insert at least one element"), 0);
	FOCUS ("add");
	break;
      }

      if (current_applet == APPLET_TREEMENU)
      {
	int iErr = ValidateTreeMenu (ptc, gtk_ctree_node_nth (ptc, 0));
	switch (iErr)
	{
	case 0:
	  message_box(_("Internal error: applet setting invalid"), 0);
	  break;
	case -1:
	  message_box(_("Error ! This item is not associated with an icon image file"), 0);
	  break;
	}

	if (iErr <= 0)
	  break;
      }
      else
      {
	int iErr;
	m_nNode = 1;
	iErr=ValidateTreeCtrl( ptc, gtk_ctree_node_nth (ptc, 0), 0 );

	switch (iErr)
	{
	case 0:
	  message_box(_("Internal error: applet setting invalid"), 0);
	  break;
	case -1:
	  if (current_applet == APPLET_WHEEL)
	    message_box(_("Error ! You have only 8 wheels available"), 0);
	  else if (current_applet == APPLET_MORPHMENU)
	    message_box(_("Error ! You have only 20 sub-menu available"), 0);
	  break;
	case -2:
	  if (current_applet == APPLET_WHEEL)
	    message_box(_("Error ! It's impossible to have more than one 'back item' for wheel"), 0);
	  else if (current_applet == APPLET_MORPHMENU)
	    message_box(_("Error ! It's impossible to have more than one 'back item' for sub-menu"), 0);
	  break;
	case -3:
	  if (current_applet == APPLET_WHEEL)
	    message_box(_("Error ! You have only 8 item for wheels available"), 0);
	  else if (current_applet == APPLET_MORPHMENU)
	    message_box(_("Error ! You have only 20 item for menus available"), 0);
	  break;
	case -4:
	  message_box(_("Error ! Place at least a back item in each submenu"), 0);
	  break;
	}

	if( iErr<=0 )
	  break;
      }
    }

    ret = 1;
    break;

  case DLG_CONF_MENU_2:
    CHECK_NULL ("x");
    CHECK_NULL ("y");

    ret = 1;
    break;

   case DLG_WHEEL:
     if (strcmp (GET_TEXT ("mode_entry"), "image") == 0)
       CHECK_NULL ("back_image");

     CHECK_NULL ("up");
     CHECK_FLOAT ("up", -1000, 1000);

     CHECK_INT ("left", 0, 20);
     CHECK_INT ("submenu", 1, 100);
     CHECK_NULL ("statusbar");
     CHECK_NULL ("extrude");
     CHECK_NULL ("text");

     ret = 1;
     break;

  case DLG_MORPHMENU:
     if (strcmp (GET_TEXT ("fx_entry"), "image") == 0)
       CHECK_NULL ("back_image");

     CHECK_NULL ("statusbar");
     CHECK_NULL ("menu_y");

     ret = 1;
     break;

  case DLG_TREEMENU:
    CHECK_NULL ("scroll_close");
    CHECK_NULL ("scroll_open");
    CHECK_NULL ("close");
    CHECK_NULL ("open");
    CHECK_NULL ("gravity");
    CHECK_NULL ("amplitude");

    ret = 1;
    break;

  case DLG_ANFYCAM:
    CHECK_NULL ("image");
    CHECK_NULL ("interval");
    CHECK_INT ("fade", 1, 255);

    ret = 1;
    break;

  case DLG_ANFYPAINT:
    CHECK_INT ("width", 580, 9999);
    CHECK_INT ("height", 150, 9999);
    CHECK_INT ("jpeg", 1, 100);

    {
      char* str = GET_TEXT ("mail");

      if (strlen (str))
	if (strchr (str, '@') == NULL || strchr (str, '.') == NULL)
	  {
	    message_box (_("Error ! Wrong e-mail address"), 0);
	    FOCUS ("mail");
	    break;
	  }
    }

    ret = 1;
    break;

  case DLG_SOLIDSCROLLER:
    CHECK_NULL ("wave_speed");
    CHECK_NULL ("v_spac");

    val1 = 0;
    GET_INT ("speed", val1);

    if (val1 < 1)
    {
      message_box (_("Error ! Value can't be smaller than 1"), 0);
      FOCUS ("speed");
      break;
    }

    CHECK_NULL ("y_offset");
    CHECK_NULL ("jump_amp");
    CHECK_NULL ("jump_speed");
    CHECK_NULL ("mirror_height");

    ret = 1;
    break;

  case DLG_ANFYFONT:
    CHECK_NULL ("font");

    ret = 1;
    break;

  case DLG_ANFYSNOW:
    CHECK_NULL ("back_image");
    CHECK_NULL ("far");
    CHECK_NULL ("middle");
    CHECK_NULL ("near");
    CHECK_NULL ("fore");
    CHECK_INT ("drop_speed", 1, 50);

    ret = 1;
    break;

  case DLG_FIREWORKS:
    CHECK_NULL ("max");
    CHECK_NULL ("power");
    CHECK_NULL ("density");
    CHECK_NULL ("duration");
    CHECK_NULL ("gravity");

    ret = 1;
    break;

    default:
      ret = 1;
  }

  // common controls
  if (ret != 0)
  {
    GtkWidget* w;

    w = valid_widget (main_dlg, "width");
    if (w)
      if (GTK_WIDGET_SENSITIVE (w))
	if (!null_entry (w))
	  return 0;

    w = valid_widget (main_dlg, "height");
    if (w)
      if (GTK_WIDGET_SENSITIVE (w))
	if (!null_entry (w))
	  return 0;

    w = valid_widget (main_dlg, "charset");
    if (w)
      if (GTK_WIDGET_SENSITIVE (w))
	if (!null_entry (w))
	  return 0;

    w = valid_widget (main_dlg, "text");
    if ((w != NULL) && ((current_dialog == DLG_ANFYFONT) || (current_dialog == DLG_TEXTSTYLE)))
      if (GTK_WIDGET_SENSITIVE (w))
	if (strlen (gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1)) == 0)
	{
	  message_box ("Error ! Field can't be null", 0);
	  gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(w)), w);
 
	  return 0;
	}
  }

  return ret;
}
