#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "custom.h"
#include "main.h"
#include "support.h"
#include "misc.h"

#define HTML_PREVIW_PAGE		"preview.html"
#define GIF_ANFY_LINK			"ajbut1.gif"
#define CHARSET_DEFAULT			"iso-8859-1"

// =========================================================

#define _ttoi(s) strtol(s, NULL, 10)
#define _tcstoul strtoul
#define GetRValue(rgb)  ((guint8)(rgb))
#define GetGValue(rgb)  ((guint8)(((guint16)(rgb)) >> 8))
#define GetBValue(rgb)  ((guint8)((rgb)>>16))  

void restore_dialog_data (int usedefault);
int validate_fields (GtkWidget *dlg);
void save_dialog_data ();
void file_browse (char* caption, char** filename);

// =========================================================

static char* mycat(char* a, char* b)
{
  char* str;
  int l;

  if (a)
    l = strlen (a);
  else
    l = 0;

  l += strlen (b) + 1;

  str = g_malloc (l);
  strcpy (str, a);
  strcat (str, b);
  g_free (a);

  return str;
}


// =========================================================

gboolean SetAppletProject(const int applet)
{
  GtkWidget *entry;
  entry = lookup_widget (main_dlg, "applet_entry");

  switch (applet)
  {
  case APPLET_BOOKFLIP:
  case APPLET_MOSAIC:
  case APPLET_ANFADE:
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "cat_banner")), TRUE);
    gtk_entry_set_text (GTK_ENTRY (entry), applet_info[applet].name);
    return TRUE;

  case APPLET_FLOZOIDS:
  case APPLET_ANFRACT:
  case APPLET_MANDEL:
  case APPLET_LIFE2D:
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "cat_fractals")), TRUE);
    gtk_entry_set_text (GTK_ENTRY (entry), applet_info[applet].name);
    return TRUE;

  case APPLET_ANFYBUMP:
  case APPLET_DEFORM:   
  case APPLET_HUEROT:   
  case APPLET_ANLAKE:   
  case APPLET_ANLENS:	 
  case APPLET_WARP:     
  case APPLET_ROTATE:   
  case APPLET_ANWATER:
  case APPLET_WOBBLER:
  case APPLET_ANFYSNOW:
  case APPLET_FIREWORKS:
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "cat_image")), TRUE);
    gtk_entry_set_text (GTK_ENTRY (entry), applet_info[applet].name);
    return TRUE;

  case APPLET_WORMHOLE: 
  case APPLET_ANVOXEL:	 
  case APPLET_TUNNEL3D: 
  case APPLET_ANTUNNEL: 
  case APPLET_TMAPCUBE: 
  case APPLET_FLUID:    
  case APPLET_GALAXY:   
  case APPLET_ANFY3D:   
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "cat_3d")), TRUE);
    gtk_entry_set_text (GTK_ENTRY (entry), applet_info[applet].name);
    return TRUE;

  case APPLET_CUBEMENU:
  case APPLET_WHEEL:	
  case APPLET_MORPHMENU:	
  case APPLET_TREEMENU:	
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "cat_navigation")), TRUE);
    gtk_entry_set_text (GTK_ENTRY (entry), applet_info[applet].name);
    return TRUE;

  case APPLET_ANBLUR:   
  case APPLET_ANFLAG:   
  case APPLET_BLOBS:    
  case APPLET_FIRE:     
  case APPLET_FLAGLOAD: 
  case APPLET_PLASMA:   
  case APPLET_TSCROLLER:
  case APPLET_ANFYCAM:
  case APPLET_ANFYPAINT:
  case APPLET_SOLIDSCROLLER:
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (main_dlg, "cat_other")), TRUE);
    gtk_entry_set_text (GTK_ENTRY (entry), applet_info[applet].name);
    return TRUE;

  default:
    return FALSE;
  }
}

typedef struct _RegistryEntry
{
  char     name[128];
  guint32  type;
  char     value[512];
  guint32  valueSize;
  char     filler[64];
} RegistryEntry;

#define PRJ_FILE_HEADER_14   "A J P 1 4"

int OpenProjectOld(char* a_pstrPathName)
{
  return -1;

  /*
	CString strPathName;
	strPathName = a_pstrPathName;

	CString strApplet;
	TCHAR szBuffer[MAX_LENGTH_INI_STR_VAR];
	if( GetPrivateProfileSectionNames( szBuffer, MAX_LENGTH_INI_STR_VAR, strPathName) == (MAX_LENGTH_INI_STR_VAR - 2) )
		return -1;
	else
		strApplet = szBuffer;

	if( strApplet.GetLength() == 0 )
		return -1;

	if (current_dialog != DLG_STEP0)
	{
		//sto cercando i caricare un progetto di un applet diversa 
		//da quella corrente
		if( m_strCurrentApplet != strApplet)
			return 0;
	}
	else
	{
		m_strCurrentApplet = szBuffer;
	}

	const nSize=32767;
	TCHAR szStr[nSize];
	memset(szStr, (TCHAR)NULL, nSize);
	unsigned int ret = GetPrivateProfileSection( strApplet, szStr, nSize, strPathName );
	if(  ret == ( nSize - 2) )
		return -1;

	CString strKeyPath(INI_REG_PATH_SETTING);
	strKeyPath+=_T("\\");
	strKeyPath+=m_strCurrentApplet;

	HKEY hKey;
	RegistryEntry RegEntry;

	int iRet=1;
	if( RegOpenKeyEx(HKEY_CURRENT_USER, strKeyPath, 0, KEY_WRITE, &hKey) == ERROR_SUCCESS )
	{
		int i=0;
		int i1;
		while( !(szStr[i] == '\0' && szStr[i+1] == '\0') )
		{
			if( szStr[i] == '\0' )
				i++;

			memset((LPVOID)&RegEntry, 0, sizeof RegistryEntry);
			for( i1=0; szStr[i]!='='; i++)
			{
				RegEntry.name[i1]=szStr[i];
				i1++;
			}
			i++;
			for( i1=0; szStr[i]!='\0'; i++)
			{
				RegEntry.value[i1]=szStr[i];
				i1++;
			}

			RegEntry.type=REG_SZ;
			RegEntry.valueSize=lstrlen((LPCTSTR)RegEntry.value);

            //////////////////////////////////////////////////
            //questo e' una fix, per sistemare il fatto che nei progetti salvati
            //nella 1.3 nel treemenu c'e' un paramentro TexSize che e' a 84
            //e che nella 1.3 non ha senso ma siccome nelle versioni successive viene
            //utilizzato da effetti non molto sgradevoli, per cui viene forzato a 11
            //////////////////////////////////////////////////
            if(    m_strCurrentApplet==APPLET_TREEMENU
                && lstrcmp(RegEntry.name, INI_TEXT_SIZE) == 0 )
            {
                _itot(11, (TCHAR*)RegEntry.value, 10);
            }
            //////////////////////////////////////////////////

			if( RegSetValueEx(hKey
							 ,RegEntry.name
							 ,0
							 ,RegEntry.type
							 ,LPBYTE(RegEntry.value)
							 ,RegEntry.valueSize) 

							 != ERROR_SUCCESS )
			{
				iRet=-1;
				break;
			}

		}

		RegCloseKey(hKey);
	}
	else
		iRet=-1;

	return iRet;
  */
}

// =========================================================

char* TranslateMenuSettingInHTML(char* applet_name, char* applet_tag, char x, char y, char cParent)
{
  char strCaption[256], strLine[256], strItem[256];

  sprintf (strCaption, "%s%c%c", INI_MENU_TEXT_, x, y);
  GetSavedVar(applet_name, strCaption, strItem );
  if (strlen (strItem) == 0)
    return applet_tag;

  sprintf (strLine, "<param name=\"MenuText%c%c\" value=\"%s\">\r\n",x, y, strItem);
  applet_tag = mycat (applet_tag, strLine);

  sprintf (strCaption, "%s%c%c",INI_MENU_URL_, x, y);
  GetSavedVar( applet_name, strCaption, strItem );
  if (strlen (strItem) != 0)
  {
    sprintf (strLine, "<param name=\"MenuUrl%c%c\" value=\"%s\">\r\n", x, y, strItem);
    applet_tag = mycat (applet_tag, strLine);
  }

  sprintf (strCaption, "%s%c%c",INI_MENU_TARGET_, x, y);
  GetSavedVar( applet_name, strCaption, strItem );
  if (strlen (strItem) != 0)
  {
    sprintf (strLine, "<param name=\"MenuTarget%c%c\" value=\"%s\">\r\n", x, y, strItem);
    applet_tag = mycat (applet_tag, strLine);
  }

  sprintf (strCaption, "%s%c%c",INI_MENU_LINK_, x, y);
  GetSavedVar( applet_name, strCaption, strItem );
  if (strlen (strItem) != 0)
  {
    sprintf (strLine, "<param name=\"MenuLink%c%c\" value=\"%s\">\r\n", x, y, strItem);
    applet_tag = mycat (applet_tag, strLine);

    //se non e' un link all'indietro
    if( strItem[0] != cParent )
    {
      applet_tag = TranslateMenuSettingInHTML(applet_name, applet_tag, strItem[0], '0', x);
    }
  }

  applet_tag = TranslateMenuSettingInHTML(applet_name, applet_tag, x, GetMenuLinkNext(applet_name, y), cParent);

  return applet_tag;
}

char* TranslateTreeMenuSettingInHTML(char* applet_name, char* applet_tag, guint32* pidMenu, guint32 idLevel, guint32 idParent)
{
  char strCaption[256], strLine[256], strLevel[256], strText[256];
  char strIconIndex[256], strLink[256], strUrl[256];

  sprintf (strCaption, "%s%d",INI_MENU_LEVEL_, *pidMenu);
  GetSavedVar(applet_name, strCaption, strLevel );
  if (idLevel != (guint32)_ttoi(strLevel))
  {
    --(*pidMenu);
    return applet_tag;
  }

  sprintf (strCaption, "%s%d",INI_MENU_TEXT_, *pidMenu);
  GetSavedVar(applet_name, strCaption, strText );
  if (strlen (strText) == 0)
    {
      --(*pidMenu);
      return applet_tag;
    }

  if( *pidMenu == 0 )
  {
    char strIconImage[256];
    guint32 i;

    for (i = 0;  ; i++)
    {
      sprintf (strCaption, "%s%d",INI_ICON_, i);
      GetSavedVar(applet_name, strCaption, strIconImage );
      if (strlen (strIconImage) == 0)
	break;
      sprintf (strLine, "<param name=\"Icon%d\" value=\"%s\">\r\n",i,strIconImage);
      applet_tag = mycat (applet_tag, strLine);
    }
  }

  sprintf (strCaption, "%s%d",INI_MENU_ICON_, *pidMenu);
  GetSavedVar(applet_name, strCaption, strIconIndex );
  if (strlen (strIconIndex) == 0)
    strcpy(strIconIndex,"0");

  sprintf (strCaption, "%s%d",INI_MENU_URL_, *pidMenu);
  GetSavedVar(applet_name, strCaption, strUrl );
  if (strlen (strUrl) == 0)
    strcpy (strUrl, "u");

  sprintf (strCaption, "%s%d",INI_MENU_LINK_, *pidMenu);
  GetSavedVar(applet_name, strCaption, strLink );

  sprintf (strLine, "<param name=\"config%d\" value=\"%d,%s,%s,%s\">\r\n"
    ,*pidMenu,idParent,strText,strIconIndex,strUrl);
  applet_tag = mycat (applet_tag, strLine);

  if (strlen (strLink) != 0)
  {
    guint32 idParent=*pidMenu;
    ++(*pidMenu);
    applet_tag = TranslateTreeMenuSettingInHTML(applet_name, applet_tag, pidMenu, idLevel+1, idParent);
  }

  ++(*pidMenu);
  applet_tag = TranslateTreeMenuSettingInHTML(applet_name, applet_tag, pidMenu, idLevel, idParent);
  return applet_tag;
}

char* GenerateHtmlAppletTag (int applet)
{
  char line[256], strtmp[256], section[64], *applet_tag = NULL;
  char *applet_name, *jarfile, float_mode[64];
  int index;

  const char* aszPattern[] = {
    "", "Small dots", "Double dots", "Stars",
    "Stars with central hole", "Triangles", "Triple dots",
    "Triple dots dithered 1", "Triple dots dithered 2", NULL
  };

  const char* aszNation[] = {
    "","Italy","Spain","France","Ireland","Austria","Germany",
    "Netherlands (Holland)","Belgium","Luxembourg","Sweden",
    "Norway","Iceland","Groenland","Denmark","Finland","Poland",
    "Hungary","Switzerland","South Africa","Russia","Japan",
    "Israel","Greece","Usa","Canada","Australia","New Zealand",
    "United Kingdom","Argentina","Peru'","Venezuela",
    "Mexico","Brazil","Colombia",NULL 
  };

  const char* render[] = {
    "Plain texture", "Gouraud shading", "Phong shading", "Gaussian phong shading",
    "Transparent (light)", "Transparent (medium)", "Liquid texture", "Liquid gouraud shading",
    "Liquid phong shading", "Liquid transparent", "Reflects mirror view", "Particles",
    "Darker plain texture", "Metal shading", "Environment mapping", NULL
  };

  applet_name = applet_info[applet].name;
  jarfile = GetJarFile(applet);
  strcpy (float_mode, "");

  if (applet == APPLET_BLOBS)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"blobs.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
                                                                    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NBLOBS, strtmp);
    sprintf (line,"<param name=\"nblobs\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MINSPEED, strtmp);
    sprintf (line,"<param name=\"minspeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BVAL1, strtmp);
    sprintf (line,"<param name=\"bval1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BVAL2, strtmp);
    sprintf (line,"<param name=\"bval2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BVAL3, strtmp);
    sprintf (line,"<param name=\"bval3\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_COLORPAL, strtmp);
    sprintf (line,"<param name=\"numpal\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SINEMODE, strtmp);
    sprintf (line,"<param name=\"sinemode\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_STARSHAPE, strtmp);
    sprintf (line,"<param name=\"starshape\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NEGCOLOR, strtmp);
    sprintf (line,"<param name=\"negative\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANBLUR)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"anblur.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_IMGS_VALUE, strtmp);
    sprintf (line,"<param name=\"imgs\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FORMAT_VALUE, strtmp);
    sprintf (line,"<param name=\"format\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NIMGS, strtmp);
    sprintf (line,"<param name=\"nimgs\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PINGPONG, strtmp);
    sprintf (line,"<param name=\"pingpong\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DOUBLESIZE, strtmp);
    sprintf (line,"<param name=\"doublesize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SPEED, strtmp);
    sprintf (line,"<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PINGPAUSE, strtmp);
    sprintf (line,"<param name=\"pingpause\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PONGPAUSE, strtmp);
    sprintf (line,"<param name=\"pongpause\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_AUDIOPING, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line,"<param name=\"audioping\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_AUDIOPONG, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line,"<param name=\"audiopong\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFYBUMP)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"anfybump.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"picture\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ALT_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"bump\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BUMP_HEIGHT, strtmp);
    sprintf (line,"<param name=\"bump_height\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ANGLE_X, strtmp);
    sprintf (line,"<param name=\"angle_x\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ANGLE_Y, strtmp);
    sprintf (line,"<param name=\"angle_y\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHTSIZE, strtmp);
    sprintf (line,"<param name=\"lightsize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LPOW, strtmp);
    sprintf (line,"<param name=\"lightpower\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LMIN, strtmp);
    sprintf (line,"<param name=\"lightmin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LMAX, strtmp);
    sprintf (line,"<param name=\"lightmax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INTERACTIVE_MODE, strtmp);
    sprintf (line,"<param name=\"interactive\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFADE)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"AnFade.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    for (index = 0; ; index++)
    {
      sprintf (section, "%s%d", INI_IMAGE_, index+1);
      GetSavedVar (applet_name, section, strtmp);
      if (strlen (strtmp) == 0 )
	break;
      sprintf (line,"<param name=\"image%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_LINK_, index+1);
      GetSavedVar(applet_name, section, strtmp);
      if (strlen (strtmp) == 0 )
	strcpy (strtmp, INI_NO);
      sprintf (line,"<param name=\"link%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_STATUS_MSG_, index+1);
      GetSavedVar (applet_name, section, strtmp);
      sprintf (line,"<param name=\"statusmsg%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    GetSavedVar(applet_name, INI_FRAME_SPEED, strtmp);
    sprintf (line,"<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PAUSE, strtmp);
    sprintf (line,"<param name=\"pause\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PROGFADE, strtmp);
    sprintf (line,"<param name=\"progressivefade\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFLAG)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"AnFlag.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_FLAG_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_FLAG_PATTERN, strtmp);
    FindInArray(aszPattern, strtmp);
    sprintf (line,"<param name=\"pattern\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_WIND_SPEED, strtmp);
    sprintf (line,"<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CURVE, strtmp);
    sprintf (line,"<param name=\"curve\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND, strtmp);
    sprintf (line,"<param name=\"wind\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_FLAG_NATION, strtmp);
    FindInArray( aszNation, strtmp);
    sprintf (line,"<param name=\"nation\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_PIXDENSITY, strtmp);
    sprintf (line,"<param name=\"pixdensity\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_CUBEMENU)
  {
    GetSavedVar(applet_name, INI_FLOATMODE, float_mode);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"CubeMenu.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    for (index = 0; index < 6; index++)
    {
      sprintf (section, "%s%d", INI_IMAGE_, index+1);
      GetSavedVar(applet_name, section, strtmp);
      if (strlen (strtmp) == 0 )
	break;
      sprintf (line,"<param name=\"image%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_LINK_, index+1);
      GetSavedVar(applet_name, section, strtmp);
      if (strlen (strtmp) == 0 )
	strcpy (strtmp, INI_NO);
      sprintf (line,"<param name=\"link%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_STATUS_MSG_, index+1);
      GetSavedVar(applet_name, section, strtmp);
      sprintf (line,"<param name=\"statusmsg%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    GetSavedVar(applet_name, INI_ROTX, strtmp);
    sprintf (line,"<param name=\"rotx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTY, strtmp);
    sprintf (line,"<param name=\"roty\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTZ, strtmp);
    sprintf (line,"<param name=\"rotz\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line,"<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backr\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backb\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CLEAR_BACKGROUND, strtmp);
    sprintf (line,"<param name=\"clearback\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_DEFORM)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"Deform.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DISTVAL, strtmp);
    sprintf (line,"<param name=\"distval\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DFACT, strtmp);
    sprintf (line,"<param name=\"dfact\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DSPEED, strtmp);
    sprintf (line,"<param name=\"dspeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DOFFSET, strtmp);
    sprintf (line,"<param name=\"doffset\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XV1, strtmp);
    sprintf (line,"<param name=\"xv1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YV1, strtmp);
    sprintf (line,"<param name=	\"yv1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XV2, strtmp);
    sprintf (line,"<param name=\"xv2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YV2, strtmp);
    sprintf (line,"<param name=\"yv2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_FIRE)
  {
    char strXRes[64], strYRes[64];
    int iRealWidth,iRealHeight;

    GetSavedVar(applet_name, INI_FIRE_WIDTH, strtmp);
    GetSavedVar(applet_name, INI_FIRE_X_RES, strXRes);
    iRealWidth=_ttoi(strtmp)*_ttoi(strXRes);
    GetSavedVar(applet_name, INI_FIRE_HEIGHT, strtmp);
    GetSavedVar(applet_name, INI_FIRE_Y_RES, strYRes);
    iRealHeight=_ttoi(strtmp)*_ttoi(strYRes);

    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"fire.class\" width=\"%d\"", jarfile, iRealWidth);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%d\">\r\n", iRealHeight );
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_FIRE_WIDTH, strtmp);
    sprintf (line,"<param name=\"fwidth\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIRE_HEIGHT, strtmp);
    sprintf (line,"<param name=\"fheigth\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIRE_X_RES, strtmp);
    sprintf (line,"<param name=\"resx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIRE_Y_RES, strtmp);
    sprintf (line,"<param name=\"resy\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FC_START, strtmp);
    sprintf (line,"<param name=\"fcut\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FC_VALUE, strtmp);
    sprintf (line,"<param name=\"fsub\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DLIGHT, strtmp);
    sprintf (line,"<param name=\"deflight\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIRE_FORM, strtmp);
    sprintf (line,"<param name=\"fmode\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RND_REP, strtmp);
    sprintf (line,"<param name=\"rndrep\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PALR, strtmp);
    sprintf (line,"<param name=\"paletteR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PALG, strtmp);
    sprintf (line,"<param name=\"paletteG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PALB, strtmp);
    sprintf (line,"<param name=\"paletteB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_FLAGLOAD)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"FlagLoad.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_FLAG_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_FLAG_PATTERN, strtmp);
    FindInArray(aszPattern, strtmp);
    sprintf (line,"<param name=\"pattern\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_WIND_SPEED, strtmp);
    sprintf (line,"<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CURVE, strtmp);
    sprintf (line,"<param name=\"curve\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND, strtmp);
    sprintf (line,"<param name=\"wind\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FLAGLOAD, strtmp);
    sprintf (line,"<param name=\"flagload\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    sprintf (line,"<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PIXDENSITY, strtmp);
    sprintf (line,"<param name=\"pixdensity\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_FLOZOIDS)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"flozoids.class\" width=\"%s\"", jarfile, strtmp);
                                             applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_NFLOZ, strtmp);
    sprintf (line,"<param name=\"numfloz\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DISTANCE, strtmp);
    sprintf (line,"<param name=\"flozdistance\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TOCENTER, strtmp);
    sprintf (line,"<param name=\"tocenter\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ACCELL, strtmp);
    sprintf (line,"<param name=\"acceleration\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FLOZ_SPEED, strtmp);
    sprintf (line,"<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SIDE_BOUNCING, strtmp);
    sprintf (line,"<param name=\"sidebouncing\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_HEADS, strtmp);
    sprintf (line,"<param name=\"fheads\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INTERACTIVE_MODE, strtmp);
    sprintf (line,"<param name=\"interactive\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GO_OVER_TEXT, strtmp);
    sprintf (line,"<param name=\"overtext\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    sprintf (line,"<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIRST_COLOR_R, strtmp);
    sprintf (line,"<param name=\"flozR1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIRST_COLOR_G, strtmp);
    sprintf (line,"<param name=\"flozG1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIRST_COLOR_B, strtmp);
    sprintf (line,"<param name=\"flozB1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LAST_COLOR_R, strtmp);
    sprintf (line,"<param name=\"flozR2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LAST_COLOR_G, strtmp);
    sprintf (line,"<param name=\"flozG2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LAST_COLOR_B, strtmp);
    sprintf (line,"<param name=\"flozB2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_FLUID)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"fluid.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CHK_AUTODESIGN, strtmp);
    sprintf (line,"<param name=\"autodesign\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FLUID_DENSITY, strtmp);
    sprintf (line,"<param name=\"density\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NFISH, strtmp);
    sprintf (line,"<param name=\"fishnum\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RSIZE, strtmp);
    sprintf (line,"<param name=\"rainsize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RFACTOR, strtmp);
    sprintf (line,"<param name=\"rainfactor\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SRSIZE, strtmp);
    sprintf (line,"<param name=\"srainsize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SRFACTOR, strtmp);
    sprintf (line,"<param name=\"srainfactor\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CHK_VOLCANO, strtmp);
    sprintf (line,"<param name=\"volcano\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_HUEROT)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"huerot.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_HUESHIFT, strtmp);
    sprintf (line,"<param name=\"hueshift\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFRACT)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"AnFract.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_DIM, strtmp);
    sprintf (line,"<param name=\"dimensions\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BRANCH, strtmp);
    sprintf (line,"<param name=\"branches\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MORPH_SPEED, strtmp);
    sprintf (line,"<param name=\"morph_speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROT_SPEED, strtmp);
    sprintf (line,"<param name=\"rot_speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROT_SPEED2, strtmp);
    sprintf (line,"<param name=\"rot_speed2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DECLINE, strtmp);
    sprintf (line,"<param name=\"decline\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LENGTH, strtmp);
    sprintf (line,"<param name=\"length\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PENSIZE, strtmp);
    sprintf (line,"<param name=\"pensize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PROGRESS, strtmp);
    sprintf (line,"<param name=\"progress\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PROGINVERSE, strtmp);
    sprintf (line,"<param name=\"progrinverse\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PEN_PROGR, strtmp);
    sprintf (line,"<param name=\"progrstep\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ITERSKIP, strtmp);
    sprintf (line,"<param name=\"iterskip\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IFS1_R, strtmp);
    sprintf (line,"<param name=\"ifsR1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IFS1_G, strtmp);
    sprintf (line,"<param name=\"ifsG1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IFS1_B, strtmp);
    sprintf (line,"<param name=\"ifsB1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IFS2_R, strtmp);
    sprintf (line,"<param name=\"ifsR2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IFS2_G, strtmp);
    sprintf (line,"<param name=\"ifsG2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IFS2_B, strtmp);
    sprintf (line,"<param name=\"ifsB2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_OVERTEXT, strtmp);
    sprintf (line,"<param name=\"overtext\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line,"<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANLAKE)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"AnLake.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_WSPEED, strtmp);
    sprintf (line,"<param name=\"wavspeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PERSPECTIVE, strtmp);
    sprintf (line,"<param name=\"perspective\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FARWAVING, strtmp);
    sprintf (line,"<param name=\"farwaving\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND, strtmp);
    sprintf (line,"<param name=\"wind\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND_VSPEED, strtmp);
    sprintf (line,"<param name=\"windvarval\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND_VMIN, strtmp);
    sprintf (line,"<param name=\"windvarmin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND_VMAX, strtmp);
    sprintf (line,"<param name=\"windvarmax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_HALFREFLECT, strtmp);
    sprintf (line,"<param name=\"halfreflect\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANLENS)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"AnLens.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LENS_WIDTH, strtmp);
    sprintf (line,"<param name=\"lenswidth\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DIAG_DISTORSION, strtmp);
    sprintf (line,"<param name=\"distdval\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XY_DISTORSION, strtmp);
    sprintf (line,"<param name=\"distoxy\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ZOOM_FACTOR, strtmp);
    sprintf (line,"<param name=\"zoomfactor\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INTERACTIVE, strtmp);
    sprintf (line,"<param name=\"interactive\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CHK_DISTORSION, strtmp);
    sprintf (line,"<param name=\"distort\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XSPEED, strtmp);
    sprintf (line,"<param name=\"spdx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YSPEED, strtmp);
    sprintf (line,"<param name=\"spdy\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_LIFE2D)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"life2d.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES_LIFE, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FOR_COLOR_R, strtmp);
    sprintf (line,"<param name=\"foreR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FOR_COLOR_G, strtmp);
    sprintf (line,"<param name=\"foreG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FOR_COLOR_B, strtmp);
    sprintf (line,"<param name=\"foreB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIFEDENSITY, strtmp);
    sprintf (line,"<param name=\"ldensity\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_MANDEL)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"mandel.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_MANDEL_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XSTART, strtmp);
    sprintf (line,"<param name=\"xstart\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YSTART, strtmp);
    sprintf (line,"<param name=\"ystart\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XMIN, strtmp);
    sprintf (line,"<param name=\"xmin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XMAX, strtmp);
    sprintf (line,"<param name=\"xmax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YMIN, strtmp);
    sprintf (line,"<param name=\"ymin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YMAX, strtmp);
    sprintf (line,"<param name=\"ymax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NUMPAL, strtmp);
    sprintf (line,"<param name=\"numpal\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_PLASMA)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"Plasma3.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_PLASMA_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GEN1, strtmp);
    sprintf (line,"<param name=\"gen1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GEN2, strtmp);
    sprintf (line,"<param name=\"gen2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GEN3, strtmp);
    sprintf (line,"<param name=\"gen3\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RED1, strtmp);
    sprintf (line,"<param name=\"red1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RED2, strtmp);
    sprintf (line,"<param name=\"red2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RED3, strtmp);
    sprintf (line,"<param name=\"red3\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GREEN1, strtmp);
    sprintf (line,"<param name=\"green1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GREEN2, strtmp);
    sprintf (line,"<param name=\"green2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GREEN3, strtmp);
    sprintf (line,"<param name=\"green3\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BLU1, strtmp);
    sprintf (line,"<param name=\"blue1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BLU2, strtmp);
    sprintf (line,"<param name=\"blue2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BLU3, strtmp);
    sprintf (line,"<param name=\"blue3\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PLASMA_SPEED, strtmp);
    sprintf (line,"<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ROTATE)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"Rot2.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_ROT_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    {
      const char* aszRotMode[]={
	"" //questo elemento vuoto server perche' il valore deve partire da 1
	,"Zoom and rotate"
	,"Rotate only"
	,"Zoom only"
	,"Distort only"
	,"Zoom and distort"
	,"Automatic"
	,NULL
      };

      GetSavedVar(applet_name, INI_ROTMODE, strtmp);
      FindInArray(aszRotMode, strtmp);
      sprintf (line,"<param name=\"rotmode\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    GetSavedVar(applet_name, INI_RSPEED, strtmp);
    sprintf (line,"<param name=\"rspeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ZMIN, strtmp);
    sprintf (line,"<param name=\"zoommin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ZMAX, strtmp);
    sprintf (line,"<param name=\"zoommax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ZSPEED, strtmp);
    sprintf (line,"<param name=\"zoomspd\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MODIX, strtmp);
    sprintf (line,"<param name=\"modix\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MODIY, strtmp);
    sprintf (line,"<param name=\"modiy\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_AUTO1, strtmp);
    sprintf (line,"<param name=\"auto1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_AUTO2, strtmp);
    sprintf (line,"<param name=\"auto2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DISTVALUE, strtmp);
    sprintf (line,"<param name=\"distval\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TILE_IMAGE, strtmp);
    sprintf (line,"<param name=\"tile\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line,"<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backr\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backb\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_TMAPCUBE)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"TmapCube.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    for (index=0; index<6 ; index++)
    {
      sprintf (section, "%s%d", INI_IMAGE_, index+1);
      GetSavedVar ( applet_name, section, strtmp);
      if (strlen (strtmp) == 0 )
	break;
      sprintf (line,"<param name=\"image%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    GetSavedVar(applet_name, INI_INTERACTIVE_MODE, strtmp);
    sprintf (line,"<param name=\"interactive\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHTSOURCE, strtmp);
    sprintf (line,"<param name=\"lightsourced\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHT_INT, strtmp);
    sprintf (line,"<param name=\"lightpower\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHT_MIN, strtmp);
    sprintf (line,"<param name=\"lightmin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHT_MAX, strtmp);
    sprintf (line,"<param name=\"lightmax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_ROTX, strtmp);
    sprintf (line,"<param name=\"rotx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTY, strtmp);
    sprintf (line,"<param name=\"roty\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTZ, strtmp);
    sprintf (line,"<param name=\"rotz\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line,"<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backr\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backb\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CLEAR_BACKGROUND, strtmp);
    sprintf (line,"<param name=\"clearback\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANTUNNEL)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"tunnel.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);


    GetSavedVar(applet_name, INI_TUNNEL_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XSTART, strtmp);
    sprintf (line,"<param name=\"startx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YSTART, strtmp);
    sprintf (line,"<param name=\"starty\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CONEX, strtmp);
    sprintf (line,"<param name=\"conex\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CONEY, strtmp);
    sprintf (line,"<param name=\"coney\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CONEZ, strtmp);
    sprintf (line,"<param name=\"conez\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    {
      const char* aszMovement[]={
	""
	,"Advance"
	,"Back"
	,"Anticlockwise rotation"
	,"Clockwise rotation"
	,"Advance + clockwise rotation"
	,"Advance + anticlockwise rotation"
	,"Back + clockwise rotation"
	,"Back + anticlockwise rotation"
	,NULL
      };

      GetSavedVar(applet_name, INI_MOVEMENT, strtmp);
      FindInArray(aszMovement, strtmp);
      sprintf (line,"<param name=\"movement\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
    GetSavedVar(applet_name, INI_EFFECT, strtmp);
    sprintf (line,"<param name=\"effect\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_TUNNEL3D)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"tunnel3d.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TUNNEL_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_LMIN, strtmp);
    sprintf (line,"<param name=\"lightmin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LMAX, strtmp);
    sprintf (line,"<param name=\"lightmax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LDEC, strtmp);
    sprintf (line,"<param name=\"lightdec\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RADIUS, strtmp);
    sprintf (line,"<param name=\"tunnelradius\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SHADING, strtmp);
    sprintf (line,"<param name=\"shade\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_REVERSE, strtmp);
    sprintf (line,"<param name=\"lightinv\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SQUARE, strtmp);
    sprintf (line,"<param name=\"squared\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTATION, strtmp);
    sprintf (line,"<param name=\"tunrot\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ADVANCING, strtmp);
    sprintf (line,"<param name=\"tunadvance\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTX, strtmp);
    sprintf (line,"<param name=\"xrot\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTY, strtmp);
    sprintf (line,"<param name=\"yrot\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTZ, strtmp);
    sprintf (line,"<param name=\"zrot\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INTERACTIVE, strtmp);
    sprintf (line,"<param name=\"interactive\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BENDS, strtmp);
    sprintf (line,"<param name=\"bends\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NBENDS, strtmp);
    sprintf (line,"<param name=\"numbends\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SBMIN, strtmp);
    sprintf (line,"<param name=\"bendsizemin\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SBMAX, strtmp);
    sprintf (line,"<param name=\"bendsizemax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BOFFSET, strtmp);
    sprintf (line,"<param name=\"bendoffset\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BOFFSETINC, strtmp);
    sprintf (line,"<param name=\"bendoffsetinc\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BSIZEINC, strtmp);
    sprintf (line,"<param name=\"bendsizeinc\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANVOXEL)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"voxel.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SHADOW, strtmp);
    sprintf (line,"<param name=\"shadow\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROUGHNESS, strtmp);
    sprintf (line,"<param name=\"roughness\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    {
      const char* aszPal[]={
	""
	,"red -> blue"
	,"orange -> green"
	,"green -> blue"
	,"blue -> green"
	,"green -> red"
	,"blue -> red"
	,"yellow"
	,"yellow -> blue"
	,"violet"
	,"red"
	,"violet -> green"
	,"pink -> green"
	,"blue"
	,"blue2"
	,"azure"
	,"green"
	,"green2"
	,"gray"
	,"blue -> yellow"
	,"yellow"
	,"azure -> red"
	,"green -> violet"
	,"red"
	,"violet"
	,"gray"
	,"gray2"
	,"gray3"
	,NULL
      };

      GetSavedVar(applet_name, INI_NUMPAL, strtmp);
      FindInArray(aszPal, strtmp);
      sprintf (line,"<param name=\"numpal\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backr\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backb\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_HORIZON, strtmp);
    sprintf (line,"<param name=\"horizon\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line,"<param name=\"mapimg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_WARP)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"Warp.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WSTART1, strtmp);
    sprintf (line,"<param name=\"w1start\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WSTART2, strtmp);
    sprintf (line,"<param name=\"w2start\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WADD1, strtmp);
    sprintf (line,"<param name=\"wadd1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WADD2, strtmp);
    sprintf (line,"<param name=\"wadd2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WMIN1, strtmp);
    sprintf (line,"<param name=\"w1min\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WMAX1, strtmp);
    sprintf (line,"<param name=\"w1max\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WMIN2, strtmp);
    sprintf (line,"<param name=\"w2min\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WMAX2, strtmp);
    sprintf (line,"<param name=\"w2max\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WARP_MODE, strtmp);
    sprintf (line,"<param name=\"warpmode\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANWATER)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"AnWater.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line,"<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line,"<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ANWATER_LIGHT, strtmp);
    sprintf (line,"<param name=\"light\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHT_MIN, strtmp);
    sprintf (line,"<param name=\"minlight\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHT_MAX, strtmp);
    sprintf (line,"<param name=\"maxlight\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CHK_AUTODESIGN, strtmp);
    sprintf (line,"<param name=\"autodesign\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FLUID_DENSITY, strtmp);
    sprintf (line,"<param name=\"density\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NFISH, strtmp);
    sprintf (line,"<param name=\"fishnum\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CROSS, strtmp);
    sprintf (line,"<param name=\"cross\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CROSS_FACTOR, strtmp);
    sprintf (line,"<param name=\"crossfactor\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RSIZE, strtmp);
    sprintf (line,"<param name=\"rainsize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RFACTOR, strtmp);
    sprintf (line,"<param name=\"rainfactor\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SRSIZE, strtmp);
    sprintf (line,"<param name=\"srainsize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SRFACTOR, strtmp);
    sprintf (line,"<param name=\"srainfactor\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIXDROPVALUE, strtmp);
    sprintf (line,"<param name=\"fixdrop\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIXDROPX, strtmp);
    sprintf (line,"<param name=\"fixdropX\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIXDROPY, strtmp);
    sprintf (line,"<param name=\"fixdropY\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIXDROPS, strtmp);
    sprintf (line,"<param name=\"fixdropS\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FIXDROPF, strtmp);
    sprintf (line,"<param name=\"fixdropF\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PRESSURE, strtmp);
    sprintf (line,"<param name=\"pressure\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FLUIDMODE, strtmp);
    sprintf (line,"<param name=\"fluidmode\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INTERACTIVE_MODE, strtmp);
    sprintf (line,"<param name=\"interactive\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_WORMHOLE)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"wormhole.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_DOT, strtmp);
    sprintf (line,"<param name=\"dotden\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line,"<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line,"<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line,"<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XHOLESPEED, strtmp);
    sprintf (line,"<param name=\"xspeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YHOLESPEED, strtmp);
    sprintf (line,"<param name=\"yspeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_NEGATIVE, strtmp);
    sprintf (line,"<param name=\"negative\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_GALAXY)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line,"<applet archive=\"%s\" code=\"galaxy.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line," height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS2 );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_PARTICLES, strtmp);
    sprintf (line,"<param name=\"particles\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_RANDOM, strtmp);
    sprintf (line,"<param name=\"random\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    {
      const char* aszScript[]={
	"Circular Galaxy"
	,"Sphere -> Ring"
	,"Tubular Galaxy"
	,"Spiral Galaxy"
	,"Rotating System"
	,"Into Sphere"
	,"Supernova"
	,"Sphere -> Tubular"
	,"Wave -> Atom"
	,"Cone -> Wave"
	,"All effects one color"
	,"All effects all colors"
	,NULL
      };

      GetSavedVar(applet_name, INI_SCRIPT, strtmp);
      FindInArray(aszScript, strtmp);
      sprintf (line,"<param name=\"script\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    {
      const char* aszGPal[]={
	"white"
	,"blue"
	,"red->black"
	,"red->violet"
	,"green"
	,"red->white"
	,NULL
      };

      GetSavedVar(applet_name, INI_GPALETTE, strtmp);
      FindInArray(aszGPal, strtmp);
      sprintf (line,"<param name=\"gpalette\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
		
    GetSavedVar(applet_name, INI_PERSIST, strtmp);
    sprintf (line,"<param name=\"persist\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DINPERSIST, strtmp);
    sprintf (line,"<param name=\"dinamic_persist\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DINPERSISTMIN, strtmp);
    sprintf (line,"<param name=\"dinamic_min\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_DINPERSISTMAX, strtmp);
    sprintf (line, "<param name=\"dinamic_max\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_BOOKFLIP)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"bookflip.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line, "<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    for (index = 0; ; index++)
    {
      sprintf (section, "%s%d", INI_IMAGE_, index+1);
      GetSavedVar (applet_name, section, strtmp);
      if (strlen (strtmp) == 0)
	break;
      sprintf (line, "<param name=\"image%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_LINK_, index+1);
      GetSavedVar( applet_name, section, strtmp);
      if (strlen (strtmp) == 0)
	strcpy (strtmp, INI_NO);
      sprintf (line, "<param name=\"link%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_STATUS_MSG_, index+1);
      GetSavedVar( applet_name, section, strtmp);
      sprintf (line, "<param name=\"statusmsg%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_FLIP_, index+1);
      {
	const char* aszFlip[]={
	  "IN -> up-right"
	  ,"IN -> down-right"
	  ,"IN -> down-left"
	  ,"IN -> up-left"
	  ,"OUT -> down-left"
	  ,"OUT -> up-left"
	  ,"OUT -> up-right"
	  ,"OUT -> down-righ"
	  ,NULL
	};
	GetSavedVar( applet_name, section, strtmp);
	FindInArray(aszFlip, strtmp);
      }
      sprintf (line, "<param name=\"flip%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    GetSavedVar(applet_name, INI_FRAME_SPEED, strtmp);
    sprintf (line, "<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PAUSE, strtmp);
    sprintf (line, "<param name=\"pause\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_EXTRAH, strtmp);
    sprintf (line, "<param name=\"extrah\" value=\"%s\">\r\n", strtmp); 
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FLIPCURVE, strtmp);
    sprintf (line, "<param name=\"flipcurve\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SHADING_BF, strtmp);
    sprintf (line, "<param name=\"shading\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line, "<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line, "<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line, "<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_MOSAIC)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"mosaic.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    strcpy (line, STR_PARAMCREDITS);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line, "<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    for (index=0; ; index++)
    {
      sprintf (section, "%s%d", INI_IMAGE_, index+1);
      GetSavedVar( applet_name, section, strtmp);
      if (strlen (strtmp) == 0 )
	break;
      sprintf (line, "<param name=\"image%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_LINK_, index+1);
      GetSavedVar( applet_name, section, strtmp);
      if (strlen (strtmp) == 0 )
	strcpy (strtmp, INI_NO);
      sprintf (line, "<param name=\"link%d\" value=\"%s\">\r\n", index+1, strtmp);
      applet_tag = mycat (applet_tag, line);

      sprintf (section, "%s%d", INI_STATUS_MSG_, index+1);
      GetSavedVar( applet_name, section, strtmp);
      sprintf (line, "<param name=\"statusmsg%d\" value=\"%s\">\r\n", index   +1, strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    GetSavedVar(applet_name, INI_PAUSE, strtmp);
    sprintf (line, "<param name=\"pause\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TILESWIDTH, strtmp);
    sprintf (line, "<param name=\"tileswidth\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TILESHEIGHT, strtmp);
    sprintf (line, "<param name=\"tilesheight\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TILESSTEPS, strtmp);
    sprintf (line, "<param name=\"tilesteps\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line, "<param name=\"backr\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line, "<param name=\"backg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line, "<param name=\"backb\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFY3D)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"anfy3d.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS2 );   applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_REALWIDTH, strtmp);
    sprintf (line, "<param name=\"real_width\" value=\"%s\">\r\n", strtmp);
                                             applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_REALHEIGTH, strtmp);
    sprintf (line, "<param name=\"real_height\" value=\"%s\">\r\n", strtmp);
                                             applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTURE0, strtmp);
    if (strlen (strtmp) != 0)
      {
	sprintf (line, "<param name=\"texture0\" value=\"%s\">\r\n", strtmp);
	applet_tag = mycat (applet_tag, line);
      }
    GetSavedVar(applet_name, INI_TEXTURE1, strtmp);
    if (strlen (strtmp) != 0)
      {
	sprintf (line, "<param name=\"texture1\" value=\"%s\">\r\n", strtmp);
	applet_tag = mycat (applet_tag, line);
      }
    GetSavedVar(applet_name, INI_TEXTURE2, strtmp);
    if (strlen (strtmp) != 0)
      {
	sprintf (line, "<param name=\"texture2\" value=\"%s\">\r\n", strtmp);
	applet_tag = mycat (applet_tag, line);
      }
    GetSavedVar(applet_name, INI_PHONGMAP, strtmp);
    if (strlen (strtmp) != 0)
      {
	sprintf (line, "<param name=\"phongmap\" value=\"%s\">\r\n", strtmp);
	applet_tag = mycat (applet_tag, line);
      }
    GetSavedVar(applet_name, INI_GPHONGMAP, strtmp);
    if (strlen (strtmp) != 0)
      {
	sprintf (line, "<param name=\"gaussmap\" value=\"%s\">\r\n", strtmp);
	applet_tag = mycat (applet_tag, line);
      }
    GetSavedVar(applet_name, INI_INTERACTIVE_MODE, strtmp);
    sprintf (line, "<param name=\"interactive\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SCRIPT, strtmp);
    {
      const char* script[]={
	"Objects"
	,"Labyrinth"
	,NULL
      };
      FindInArray(script, strtmp);
    }
    sprintf (line, "<param name=\"script\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SHADOW, strtmp);
    {
      const char* shadow[]={
	"No"
	,"Hires"
	,"Lores"
	,NULL
      };
      FindInArray(shadow, strtmp);
    }
    sprintf (line, "<param name=\"shadow\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MIRROR, strtmp);
    sprintf (line, "<param name=\"mirror\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_LIGHTBALLS, strtmp);
    sprintf (line, "<param name=\"lightballs\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MAINTYPE, strtmp);
    {
      const char* maintype[]={
	"sphere"
	,"sheet"
	,"cone"
	,NULL
      };
      FindInArray(maintype, strtmp);
    }
    sprintf (line, "<param name=\"main_type\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MAINRENDER, strtmp);
    FindInArray(render, strtmp);
    sprintf (line, "<param name=\"main_render\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MAINTEXTURE, strtmp);
    sprintf (line, "<param name=\"main_texture\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTX, strtmp);
    sprintf (line, "<param name=\"rot_x\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTY, strtmp);
    sprintf (line, "<param name=\"rot_y\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ROTZ, strtmp);
    sprintf (line, "<param name=\"rot_z\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MAINTYPE, strtmp);
    if (strcmp (strtmp, "sheet") == 0)
    {
      GetSavedVar(applet_name, INI_SHEET_HEIGHT, strtmp);
      sprintf (line, "<param name=\"sheet_x\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_WIDTH, strtmp);
      sprintf (line, "<param name=\"sheet_y\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_NUMX, strtmp);
      sprintf (line, "<param name=\"sheet_numx\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_NUMY, strtmp);
      sprintf (line, "<param name=\"sheet_numy\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_ADD, strtmp);
      sprintf (line, "<param name=\"sheet_add\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_H, strtmp);
      sprintf (line, "<param name=\"sheet_h\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_PERIOD1, strtmp);
      sprintf (line, "<param name=\"sheet_period1\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_PERIOD2, strtmp);
      sprintf (line, "<param name=\"sheet_period2\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SHEET_FUNCTION, strtmp);
      sprintf (line, "<param name=\"sheet_function\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
    GetSavedVar(applet_name, INI_AIMLIGHT, strtmp);
    sprintf (line, "<param name=\"animlight\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CHILDTYPE, strtmp);
    {
      const char* childtype[]={
	"Off"
	,"Toroids"
	,"Stretching spheres"
	,"Cones"
	,NULL
      };
      FindInArray(childtype, strtmp);
    }
    sprintf (line, "<param name=\"child_type\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CHILDRENDER, strtmp);
    FindInArray(render, strtmp);
    sprintf (line, "<param name=\"child_render\" value=\"%s\"	>\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_CHILDTEXTURE, strtmp);
    sprintf (line, "<param name=\"child_texture\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACKMODE, strtmp);
    {
      const char* backmode[]={
	"black"
	,"sphere"
	,"colour"
	,NULL
      };
      FindInArray(backmode, strtmp);
    }
    sprintf (line, "<param name=\"back_type\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_RENDER, strtmp);
    FindInArray(render, strtmp);
    sprintf (line, "<param name=\"back_render\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACKTEXTURE, strtmp);
    sprintf (line, "<param name=\"back_texture\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line, "<param name=\"back_r\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line, "<param name=\"back_g\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line, "<param name=\"back_b\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FOG, strtmp);
    sprintf (line, "<param name=\"fog\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FOGNEAR, strtmp);
    sprintf (line, "<param name=\"fog_near\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FOGFAR, strtmp);
    sprintf (line, "<param name=\"fog_far\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BLUR, strtmp);
    sprintf (line, "<param name=\"blur_motion\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_TSCROLLER)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"txtscroll.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );   applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_R, strtmp);
    sprintf (line, "<param name=\"backR\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_G, strtmp);
    sprintf (line, "<param name=\"backG\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_COLOR_B, strtmp);
    sprintf (line, "<param name=\"backB\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_WOBBLER)
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"wobbler.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );   applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line, "<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE, strtmp);
    sprintf (line, "<param name=\"image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INITDISTANCE, strtmp);
    sprintf (line, "<param name=\"initialdistance\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ZOOMSPEED   , strtmp);
    sprintf (line, "<param name=\"zoomspeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MINDISTANCE , strtmp);
    sprintf (line, "<param name=\"mindistance\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MAXDISTANCE , strtmp);
    sprintf (line, "<param name=\"maxdistance\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WRADIUS     , strtmp);
    sprintf (line, "<param name=\"radius\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WSTRENGTH, strtmp);
    sprintf (line, "<param name=\"wavestrength\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SPD1        , strtmp);
    sprintf (line, "<param name=\"spd1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SPD2        , strtmp);
    sprintf (line, "<param name=\"spd2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SPD3        , strtmp);
    sprintf (line, "<param name=\"spd3\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SPD4        , strtmp);
    sprintf (line, "<param name=\"spd4\" value=\"%s\">\r\n", strtmp); 
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_XMOVE       , strtmp);
    sprintf (line, "<param name=\"xmove\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YMOVE       , strtmp);
    sprintf (line, "<param name=\"ymove\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_VAR_START, strtmp);
    sprintf (line, "<param name=\"variationstart\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_VAR_MOVE, strtmp);
    sprintf (line, "<param name=\"variationmove\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PRECISION, strtmp);
    sprintf (line, "<param name=\"precision\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_WHEEL) 
  {
    GetSavedVar(applet_name, INI_FLOATMODE, float_mode);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"wheelmenu.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS1 );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_FILTER, strtmp);
    {
      const char* filter[]={ "none", "smooth", "colorize", NULL };
      FindInArray(filter, strtmp);
    }
    sprintf (line, "<param name=\"filter\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FONT, strtmp);
    {
      const char* font[]={ "font0.class", "font1.class", NULL };
      FindInArray(font, strtmp);
    }
    sprintf (line, "<param name=\"font\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_TYPE    , strtmp);
    sprintf (line, "<param name=\"background\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BK_COLOR	  , strtmp);
    sprintf (line, "<param name=\"bgcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WHEEL_COLOR, strtmp);
    sprintf (line, "<param name=\"wheelcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_COLOR	, strtmp);
    sprintf (line, "<param name=\"textcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SEL_COLOR	, strtmp);
    sprintf (line, "<param name=\"selectcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FLAT         , strtmp);
    sprintf (line, "<param name=\"flat\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MOUSESENS    , strtmp);
    sprintf (line, "<param name=\"mouseSens\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_YROTATION    , strtmp);
    sprintf (line, "<param name=\"YRotation\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_OFFSETSTEP   , strtmp);
    sprintf (line, "<param name=\"offsetStep\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_STATUSBARY   , strtmp);
    sprintf (line, "<param name=\"statusbar_y\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MODE         , strtmp);
    sprintf (line, "<param name=\"mode\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTURE_FILE , strtmp);
    sprintf (line, "<param name=\"texture\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_EXTRUDE_DIFF , strtmp);
    sprintf (line, "<param name=\"extrudediff\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTDIFF     , strtmp);
    sprintf (line, "<param name=\"textdiff\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_MORPHMENU)
  {
    GetSavedVar(applet_name, INI_FLOATMODE, float_mode);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"morphmenu.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS1 );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_MENUFX, strtmp);
    sprintf (line, "<param name=\"menu_fx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MENUY, strtmp);
    sprintf (line, "<param name=\"menu_y\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_FONT, strtmp);
    sprintf (line, "<param name=\"fontName\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_SIZE, strtmp);
    sprintf (line, "<param name=\"font_h\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_ALIGN, strtmp);
    sprintf (line, "<param name=\"textalign\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_COLOR	, strtmp);
    sprintf (line, "<param name=\"textcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SEL_COLOR	, strtmp);
    sprintf (line, "<param name=\"selectedcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BK_COLOR	  , strtmp);
    sprintf (line, "<param name=\"backcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_STATUSBARY   , strtmp);
    sprintf (line, "<param name=\"statusbar_y\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACKFX, strtmp);
    sprintf (line, "<param name=\"back_fx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    if (strcmp (strtmp, "mmgalaxy") == 0)
    {
      GetSavedVar(applet_name, INI_SHAPE			, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_fx\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SEED_COLOR		, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_scol\" value=\"%02X%02X%02X\">\r\n", 
	       GetRValue(_tcstoul(strtmp,NULL,16)), 
	       GetGValue(_tcstoul(strtmp,NULL,16)), 
	       GetBValue(_tcstoul(strtmp,NULL,16)));
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_BACK_CLR		, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_clr\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_FIRE_COLOR_R	, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_r\" value=\"%u\">\r\n", (guint32)(_ttoi(strtmp)/16) );
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_FIRE_COLOR_G	, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_g\" value=\"%u\">\r\n", (guint32)(_ttoi(strtmp)/16) );
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_FIRE_COLOR_B	, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_b\" value=\"%u\">\r\n", (guint32)(_ttoi(strtmp)/16) );
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SMOOTH_FILTER	, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_filter\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_CHK_BACK_COLOR	, strtmp);
      sprintf (line, "<param name=\"mmgalaxy_bcol\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
    else if (strcmp (strtmp, "mmblobs") == 0)
    {
      GetSavedVar(applet_name, INI_NBLOBS, strtmp);
      sprintf (line, "<param name=\"mmblobs_nblobs\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_MINSPEED, strtmp);
      sprintf (line, "<param name=\"mmblobs_minspeed\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_BVAL1, strtmp);
      sprintf (line, "<param name=\"mmblobs_bval1\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_BVAL2, strtmp);
      sprintf (line, "<param name=\"mmblobs_bval2\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_BVAL3, strtmp);
      sprintf (line, "<param name=\"mmblobs_bval3\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_COLORPAL, strtmp);
      sprintf (line, "<param name=\"mmblobs_numpal\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_SINEMODE, strtmp);
      sprintf (line, "<param name=\"mmblobs_sinemode\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_STARSHAPE, strtmp);
      sprintf (line, "<param name=\"mmblobs_starshape\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_NEGCOLOR, strtmp);
      sprintf (line, "<param name=\"mmblobs_negative\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
    
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"back_image\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_TREEMENU )
  {
    GetSavedVar(applet_name, INI_FLOATMODE, float_mode);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"treemenu.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    if (strcmp (float_mode, INI_YES) == 0)
      strcpy (strtmp, "1");
    else
      GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS1 );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_INGOING, strtmp);
    sprintf (line, "<param name=\"scrollSpeedInGoing\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_OUTGOING, strtmp);
    sprintf (line, "<param name=\"scrollSpeedOutGoing\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INBOUNC, strtmp);
    sprintf (line, "<param name=\"numberOfBouncesInGoing\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_OUTBOUNC, strtmp);
    sprintf (line, "<param name=\"numberOfBouncesOutGoing\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BGRAVITY, strtmp);
    sprintf (line, "<param name=\"bounceGravity\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BAMPLITUDE, strtmp);
    sprintf (line, "<param name=\"bounceAmplitude\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_FONT, strtmp);
    sprintf (line, "<param name=\"treeFont\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_SIZE, strtmp);
    sprintf (line, "<param name=\"treeFontSize\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_ITALIC, strtmp);
    sprintf (line, "<param name=\"treeItalic\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_BOLD, strtmp);
    sprintf (line, "<param name=\"treeBold\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_SCROLL_COLOR, strtmp);
    sprintf (line, "<param name=\"systemColor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FILTER_COLOR, strtmp);
    sprintf (line, "<param name=\"filterColor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_COLOR, strtmp);
    sprintf (line, "<param name=\"foregroundColor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BK_COLOR, strtmp);
    sprintf (line, "<param name=\"backgroundColor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SEL_COLOR, strtmp);
    sprintf (line, "<param name=\"foregroundColorSelection\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SEL_BAKC_COLOR, strtmp);
    sprintf (line, "<param name=\"backgroundColorSelection\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_AUTOHIGHLIGHT, strtmp);
    sprintf (line, "<param name=\"autoHighLight\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BORDER_STYLE, strtmp);
    {
      const char* borders[]={
	"no borders"
	,"1 line"
	,"3 lines"
	,NULL
      };
      FindInArray(borders, strtmp);
    }
    sprintf (line, "<param name=\"borderStyle\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SCROLLBAR_SIZE, strtmp);
    {
      const char* scroll[]={
	"11 pixel"
	,"16 pixels"
	,NULL
      };
      FindInArray(scroll, strtmp);
    }
    sprintf (line, "<param name=\"scrollBarStyle\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"backImage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFYCAM) 
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"anfycam.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_RES, strtmp);
    sprintf (line, "<param name=\"res\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_INTERVAL, strtmp);
    sprintf (line, "<param name=\"interval\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_IMAGE_FILE, strtmp);
    sprintf (line, "<param name=\"url\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FADESPEED, strtmp);
    sprintf (line, "<param name=\"fadespeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PROGRFADE, strtmp);
    sprintf (line, "<param name=\"progressivefade\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFYPAINT) 
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"anfypaint.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );   applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_JPGQUALITY, strtmp);
    sprintf (line, "<param name=\"jpgquality\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SAVEBUTTON, strtmp);
    sprintf (line, "<param name=\"savebutton\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SENDTOMAIL, strtmp);
    sprintf (line, "<param name=\"sendtomail\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BK_COLOR	  , strtmp);
    sprintf (line, "<param name=\"backcolor\" value=\"%02X%02X%02X\">\r\n", 
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_PEN_COLOR, strtmp);
    sprintf (line, "<param name=\"pencolor\" value=\"%02X%02X%02X\">\r\n",
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_STARTMESSAGE, strtmp);
    sprintf (line, "<param name=\"startmsg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_SOLIDSCROLLER) 
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"solidscr.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_TEXTSCROLL, strtmp);
    sprintf (line, "<param name=\"textscroll\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXT_TYPE, strtmp);
    sprintf (line, "<param name=\"texttype\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTSPEED, strtmp);
    sprintf (line, "<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WAVEMODE, strtmp);
    sprintf (line, "<param name=\"wavemode\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WAVETYPE, strtmp);
    sprintf (line, "<param name=\"wavetype\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WAVESPEED, strtmp);
    sprintf (line, "<param name=\"wavespeed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_REVERSEFX, strtmp);
    sprintf (line, "<param name=\"reversefx\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTJUMPAMP, strtmp);
    sprintf (line, "<param name=\"TextJumpAmp\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTJUMPSPEED, strtmp);
    sprintf (line, "<param name=\"TextJumpSpd\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTOFFSET, strtmp);
    sprintf (line, "<param name=\"textoffset\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_TEXTVSPACE, strtmp);
    sprintf (line, "<param name=\"textvspace\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MIRRORMODE, strtmp);
    sprintf (line, "<param name=\"mirrormode\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_MIRRORHEIGTH, strtmp);
    sprintf (line, "<param name=\"mirrorheight\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FONTINDEX, strtmp);
    sprintf (line, "<param name=\"fontindex\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BK_COLOR	  , strtmp);
    sprintf (line, "<param name=\"bgcolor\" value=\"%02X%02X%02X\">\r\n",
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_ANFYSNOW) 
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"ansnow.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_F1_NUMFAR, strtmp);
    sprintf (line, "<param name=\"flakes1\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_F2_NUMMIDDLE, strtmp);
    sprintf (line, "<param name=\"flakes2\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_F3_NUMNEAR, strtmp);
    sprintf (line, "<param name=\"flakes3\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_F4_NUMFORROUND, strtmp);
    sprintf (line, "<param name=\"flakes4\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND_MAX_SPEED, strtmp);
    sprintf (line, "<param name=\"windmax\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_WIND_VAR, strtmp);
    sprintf (line, "<param name=\"windvariation\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SNOW_SPEED, strtmp);
    sprintf (line, "<param name=\"speed\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BK_COLOR	  , strtmp);
    sprintf (line, "<param name=\"bgcolor\" value=\"%02X%02X%02X\">\r\n",
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_SNOWOVER, strtmp);
    sprintf (line, "<param name=\"overtext\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }
  else if (applet == APPLET_FIREWORKS) 
  {
    GetSavedVar(applet_name, INI_WIDTH, strtmp);
    sprintf (line, "<applet archive=\"%s\" code=\"firewks.class\" width=\"%s\"", jarfile, strtmp);
    applet_tag = g_strdup (line);
    GetSavedVar(applet_name, INI_HEIGHT, strtmp);
    sprintf (line, " height=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    sprintf (line, STR_PARAMCREDITS );
    applet_tag = mycat (applet_tag, line);
    
    GetSavedVar(applet_name, INI_AUDIOFILE, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"boomsound\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FW_MAX, strtmp);
    sprintf (line, "<param name=\"maxfireworks\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FW_POWER, strtmp);
    sprintf (line, "<param name=\"fireworkspower\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FW_DENSITY, strtmp);
    sprintf (line, "<param name=\"fireworksdensity\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_FW_TIMEDURATION, strtmp);
    sprintf (line, "<param name=\"fireworksduration\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_GRAVITY, strtmp);
    sprintf (line, "<param name=\"gravity\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BACK_IMAGE_PATH, strtmp);
    if (strlen (strtmp) == 0 )
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"backimage\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_BK_COLOR	  , strtmp);
    sprintf (line, "<param name=\"bgcolor\" value=\"%02X%02X%02X\">\r\n",
	     GetRValue(_tcstoul(strtmp,NULL,16)), 
	     GetGValue(_tcstoul(strtmp,NULL,16)), 
	     GetBValue(_tcstoul(strtmp,NULL,16)));
    applet_tag = mycat (applet_tag, line);

    GetSavedVar(applet_name, INI_SHOWOVERTEXT, strtmp);
    sprintf (line, "<param name=\"overtext\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }

  if ((applet == APPLET_WHEEL)
      || (applet == APPLET_MORPHMENU)
      || (applet == APPLET_TREEMENU)
      || (applet == APPLET_CUBEMENU))
    {
      GetSavedVar(applet_name, INI_FLOATMODE, strtmp);
      sprintf (line, "<param name=\"floatmode\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_WIDTH, strtmp);
      sprintf (line, "<param name=\"floatw\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_HEIGHT, strtmp);
      sprintf (line, "<param name=\"floath\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_FLOAT_X, strtmp);
      sprintf (line, "<param name=\"floatx\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_FLOAT_Y, strtmp);
      sprintf (line, "<param name=\"floaty\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_FLOAT_TITLE, strtmp);
      sprintf (line, "<param name=\"floatitle\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_FLOATINFRONT, strtmp);
      sprintf (line, "<param name=\"floatinfront\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }

  if ((applet == APPLET_WHEEL) ||
      (applet == APPLET_MORPHMENU))
  {
    applet_tag = TranslateMenuSettingInHTML(applet_name, applet_tag, '0', '0', '0');
  }

  if (applet == APPLET_TREEMENU )
  {
    guint32 idMenu=0;
    applet_tag = TranslateTreeMenuSettingInHTML(applet_name, applet_tag, &idMenu, 0, 0);
  }

  if (applet != APPLET_ANFYPAINT)
  {
    GetSavedVar(applet_name, INI_OVERIMG, strtmp);
    if (strlen (strtmp) == 0)
      strcpy (strtmp, INI_NO);
    sprintf (line, "<param name=\"overimg\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_OVERIMGX, strtmp);
    sprintf (line, "<param name=\"overimgX\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_OVERIMGY, strtmp);
    sprintf (line, "<param name=\"overimgY\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
  }

  //parametri text scroll comuni
  if ((applet == APPLET_BLOBS) ||
      (applet == APPLET_ANFYBUMP) ||
      (applet == APPLET_ANFLAG) ||
      (applet == APPLET_CUBEMENU ) ||
      (applet == APPLET_DEFORM) ||
      (applet == APPLET_FIRE) ||
      (applet == APPLET_FLAGLOAD) ||
      (applet == APPLET_FLOZOIDS) ||
      (applet == APPLET_FLUID) ||
      (applet == APPLET_HUEROT) ||
      (applet == APPLET_ANFRACT) ||
      (applet == APPLET_ANLAKE) ||
      (applet == APPLET_LIFE2D) ||
      (applet == APPLET_PLASMA) ||
      (applet == APPLET_ROTATE) ||
      (applet == APPLET_TMAPCUBE) ||
      (applet == APPLET_ANTUNNEL) ||
      (applet == APPLET_TUNNEL3D) ||
      (applet == APPLET_ANVOXEL) ||
      (applet == APPLET_WARP) ||
      (applet == APPLET_ANWATER) ||
      (applet == APPLET_WORMHOLE) ||
      (applet == APPLET_GALAXY) ||
      (applet == APPLET_ANFY3D) ||
      (applet == APPLET_WOBBLER) ||
      (applet == APPLET_TSCROLLER) ||
      (applet == APPLET_ANFYSNOW) ||
      (applet == APPLET_FIREWORKS))
  {
    GetSavedVar(applet_name, INI_TEXTSCROLL, strtmp);
    sprintf (line, "<param name=\"textscroll\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    if (strcmp (strtmp, INI_NO) != 0)
    {
      GetSavedVar(applet_name, INI_TEXTMODE, strtmp);
      sprintf (line, "<param name=\"texttype\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTVSPACE, strtmp);
      sprintf (line, "<param name=\"textvspace\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_SIZE_MIN, strtmp);
      sprintf (line, "<param name=\"textminfont\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_SIZE_MAX, strtmp);
      sprintf (line, "<param name=\"textmaxfont\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTOFFSET, strtmp);
      sprintf (line, "<param name=\"textoffset\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTSPEED, strtmp);
      sprintf (line, "<param name=\"textspeed\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_FONT, strtmp);
      sprintf (line, "<param name=\"textfont\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_BOLD, strtmp);
      sprintf (line, "<param name=\"textbold\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_ITALIC, strtmp);
      sprintf (line, "<param name=\"textitalic\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_SIZE, strtmp);
      sprintf (line, "<param name=\"textsize\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTSHADOW, strtmp);
      sprintf (line, "<param name=\"textshadow\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTS_COLORRED, strtmp);
      sprintf (line, "<param name=\"TextSColR\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTS_COLORGREEN, strtmp);
      sprintf (line, "<param name=\"TextSColG\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTS_COLORBLU, strtmp);
      sprintf (line, "<param name=\"TextSColB\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_COLORRED, strtmp);
      sprintf (line, "<param name=\"TextColR\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_COLORGREEN, strtmp);
      sprintf (line, "<param name=\"TextColG\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXT_COLORBLU, strtmp);
      sprintf (line, "<param name=\"TextColB\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTJUMPAMP, strtmp);
      sprintf (line, "<param name=\"TextJumpAmp\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTJUMPSPEED, strtmp);
      sprintf (line, "<param name=\"TextJumpSpd\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTSINEAMPL, strtmp);
      sprintf (line, "<param name=\"TextSineAmp\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTSINSPEED, strtmp);
      sprintf (line, "<param name=\"TextSineSpd\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
      GetSavedVar(applet_name, INI_TEXTSINANGLE, strtmp);
      sprintf (line, "<param name=\"TextSineAngle\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
  }

  //i parametri per i codici di restrazione
  {
    char strRegCode[256];
    GetSavedVar("", INI_REGCODE, strRegCode );
    if(strlen (strRegCode)<=0)
      strcpy (strRegCode, INI_NO);

    sprintf (line, "<param name=\"regcode\" value=\"%s\">\r\n", strRegCode);
    applet_tag = mycat (applet_tag, line);
  }

  if (applet != APPLET_ANFYPAINT)
  {
    if ((applet != APPLET_ANFADE) &&
	(applet != APPLET_CUBEMENU) &&
	(applet != APPLET_BOOKFLIP) &&
	(applet != APPLET_MOSAIC) &&
	(applet != APPLET_WHEEL) &&
	(applet != APPLET_MORPHMENU) &&
	(applet != APPLET_TREEMENU))
    {
      GetSavedVar(applet_name, INI_REGLINK, strtmp);
      sprintf (line, "<param name=\"reglink\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
    GetSavedVar(applet_name, INI_REGNEWFRAME, strtmp);
    sprintf (line, "<param name=\"regnewframe\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    GetSavedVar(applet_name, INI_REGFRAMENAME, strtmp);
    sprintf (line, "<param name=\"regframename\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);

    if ((applet != APPLET_ANFADE) &&
	(applet != APPLET_CUBEMENU) &&
	(applet != APPLET_BOOKFLIP) &&
	(applet != APPLET_MOSAIC) &&
	(applet != APPLET_WHEEL) &&
	(applet != APPLET_MORPHMENU) &&
	(applet != APPLET_TREEMENU))
    {
      GetSavedVar(applet_name, INI_STATUSMSG, strtmp);
      sprintf (line, "<param name=\"statusmsg\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    if (applet !=  APPLET_ANBLUR)
    {
      GetSavedVar(applet_name, INI_EX_MEMDELAY, strtmp);
      sprintf (line, "<param name=\"memdelay\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    GetSavedVar(applet_name, INI_EX_TASK_PRIORITY, strtmp);
    sprintf (line, "<param name=\"priority\" value=\"%s\">\r\n", strtmp);
    applet_tag = mycat (applet_tag, line);
    
    if ((applet == APPLET_CUBEMENU) ||
	(applet == APPLET_TMAPCUBE))
    {
      sprintf (line, "<param name=\"priorityL\" value=\"3\">\r\n");
      applet_tag = mycat (applet_tag, line);
    }

    if (applet !=  APPLET_ANBLUR)
    {
      GetSavedVar(applet_name, INI_EX_MINSYNC, strtmp);
      sprintf (line, "<param name=\"MinSYNC\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }

    if ((applet ==  APPLET_BLOBS)
	|| (applet ==  APPLET_FIRE)
	|| (applet ==  APPLET_PLASMA)
	|| (applet ==  APPLET_ANVOXEL)
	|| (applet ==  APPLET_GALAXY))
    {
      GetSavedVar(applet_name, INI_FIXNETSCAPE, strtmp);
      sprintf (line, "<param name=\"fixnetscape\" value=\"%s\">\r\n", strtmp);
      applet_tag = mycat (applet_tag, line);
    }
  }

  GetSavedVar(applet_name, INI_NOJAVAMSG, strtmp);
  sprintf (line, "%s\r\n", strtmp);
  applet_tag = mycat (applet_tag, line);
  sprintf (line, "</applet>\r\n");
  applet_tag = mycat (applet_tag, line);

  return applet_tag;
}

int GeneratePreviewFile (const int applet, int bForPublish)
{
  char* strAppletTag, strAppletDir[1024], strPathFilePreview[1024], strGifPath[1024];
  char strCharSet[256];
  int bRet = 1;
  FILE* f;

  strAppletTag = GenerateHtmlAppletTag (applet);

  GetAppletPath(applet, strAppletDir);
  strcpy (strPathFilePreview, strAppletDir);
  strcat (strPathFilePreview, "/");
  strcat (strPathFilePreview, HTML_PREVIW_PAGE);

  if (bForPublish)
  {
    strcpy (strGifPath, GIF_ANFY_LINK);
  }
  else
  {
    strcpy (strGifPath, "../");
    strcat (strGifPath, GIF_ANFY_LINK);
  }

  strcpy (strCharSet, CHARSET_DEFAULT);
  
  if (applet == APPLET_MORPHMENU
      || applet == APPLET_TREEMENU
      || applet == APPLET_TSCROLLER)
    {
      GetSavedVar(applet_info[applet].name, INI_CHARSET, strCharSet);
    }

  f = fopen (strPathFilePreview, "wt");
  if (f != NULL)
  {
    fprintf (f, "<html>\r\n");
    fprintf (f, "\r\n");
    fprintf (f, "<head>\r\n");
    fprintf (f, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\r\n", strCharSet);
    fprintf (f, "<meta name=\"Anfy\" content=\"\">\r\n");
    fprintf (f, "<title>Anfy preview</title>\r\n");
    fprintf (f, "</head>\r\n");
    fprintf (f, "<body bgcolor=\"#000000\" text=\"#FFFF22\" link=\"#5FFF5F\" vlink=\"#553AEB\" alink=\"#FF8000\">\r\n");
    fprintf (f, "<table width=100%% height=90%%>\r\n");
    fprintf (f, "<tr>\r\n");
    fprintf (f, "<td align=center valign=center>\r\n");
    fprintf (f, "%s", strAppletTag);
    fprintf (f, "</td>\r\n");
    fprintf (f, "</tr>\r\n");
    fprintf (f, "</table>\r\n");
    fprintf (f, "<p><a href=\"http://www.anfyteam.com\"><img src=\"%s\" alt=\"This site is Anfy Enhanced\" border=\"0\" width=\"85\" height=\"31\"></a></p>\r\n", strGifPath);
    fprintf (f, "</body>\r\n");
    fprintf (f, "</html>\r\n");
    fclose (f);
  }
  else
  {
    bRet = 0;
    message_box (_("Error writing preview html file"), 0);
  }
  
  free (strAppletTag);
  return bRet;
}

int Publish (int applet, char* szNewHtmlPage)
{
  char strOldPath[1024], szNewPath[1024], strSection[64], strTmp[1024];
  int iIndex;

  PathFromFile (szNewHtmlPage, szNewPath);
  GetAppletPath (applet, strOldPath);

  {
    char strPreviewFile[1024];

    if (!GeneratePreviewFile(applet, 1))
      goto exit_copy_error;

    strcpy (strPreviewFile, strOldPath);
    strcat (strPreviewFile, "/");
    strcat (strPreviewFile, HTML_PREVIW_PAGE);

    if (!CopyFile(strPreviewFile,szNewHtmlPage))
      goto exit_copy_error;

    strcpy (strTmp, g_strAppPath);
    strcat (strTmp, "/");
    strcat (strTmp, g_strAppletDir);

    if( !MyCopyFile(strTmp, szNewPath, GIF_ANFY_LINK) )
      goto exit_copy_error;
  }

  {
    char strImgFile[1024];
    gboolean bNoAnfyClass=FALSE;
    gboolean bNoLWareClass=FALSE;
    gboolean bNoOverImg=FALSE;

    switch (applet)
    {
      case APPLET_ANFYBUMP:
	{
	  if( !MyCopyFile(strOldPath,szNewPath, "anfybump.class"))
	    goto exit_copy_error;
			
	  GetSavedVar(applet_info[applet].name, INI_IMAGE_FILE, strImgFile );
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;

	  GetSavedVar(applet_info[applet].name, INI_ALT_IMAGE_FILE, strImgFile );
	  if (strcmp(strImgFile, INI_NO) != 0 )
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_ANFLAG:
	{
	  if (!MyCopyFile(strOldPath,szNewPath,"AnFlag.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANFRACT:
	{
	  bNoAnfyClass=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath,"AnFract.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANLAKE:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"AnLake.class"))
	    goto exit_copy_error;

	  GetSavedVar(applet_info[applet].name, INI_IMAGE_FILE, strImgFile );
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_BLOBS:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"blob.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"blobs.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_CUBEMENU:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"CubeMenu.class"))
	    goto exit_copy_error;


	  for(iIndex=0; iIndex<6 ; iIndex++)
	    {
	      sprintf (strSection, "%s%d", INI_IMAGE_, iIndex+1);
	      GetSavedVar(applet_info[applet].name, strSection, strImgFile);
	      if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
		goto exit_copy_error;
	    }
	}
	break;
      case APPLET_DEFORM:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"Deform.class"))
	    goto exit_copy_error;
	  
	  GetSavedVar( applet_info[applet].name, INI_IMAGE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_FIRE:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"fire.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANLENS:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"AnLens.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE_FILE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_FLAGLOAD:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"FlagLoad.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_FLAGLOAD, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANFADE:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"AnFade.class"))
	    goto exit_copy_error;

	  for(iIndex=0; ; iIndex++)
	    {
	      sprintf (strSection, "%s%d", INI_IMAGE_, iIndex+1);
	      GetSavedVar( applet_info[applet].name, strSection, strImgFile);
	      if ((strlen (strImgFile) == 0) || (strcmp (strImgFile, INI_NO) == 0))
		break;
	      if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
		goto exit_copy_error;
	    }
	}
	break;
      case APPLET_FLOZOIDS:
	{
	  bNoAnfyClass=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath,"flozoids.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANBLUR:
	{
	  char strTMP[256], strFormat[256];
	  int iNImgs;

	  bNoAnfyClass=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath,"anblur.class"))
	    goto exit_copy_error;

	  GetSavedVar(applet_info[applet].name, INI_NIMGS, strTMP );
	  iNImgs=_ttoi(strTMP);
	  GetSavedVar(applet_info[applet].name, INI_IMGS_VALUE, strTMP);
	  GetSavedVar(applet_info[applet].name, INI_FORMAT_VALUE, strFormat);
	  for(iIndex=0; iIndex<iNImgs; iIndex++)
	    {
	      sprintf (strImgFile, "%s%d%s", strTMP, iIndex+1, strFormat);
	      if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
		goto exit_copy_error;
	    }

	  GetSavedVar( applet_info[applet].name, INI_AUDIOPING, strImgFile);
	  if ((strlen (strImgFile) != 0) && (strcmp (strImgFile, INI_NO) != 0))
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_AUDIOPONG, strImgFile);
	  if ((strlen (strImgFile) != 0) && (strcmp (strImgFile, INI_NO) != 0))
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_FLUID:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"fluid.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_HUEROT:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"huerot.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE_FILE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_LIFE2D:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"life2d.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_MANDEL:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"mandel.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_ROTATE:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"Rot2.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE_FILE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_PLASMA:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"Plasma3.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANTUNNEL:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"tunnel.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE_FILE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_TMAPCUBE:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"TmapCube.class"))
	    goto exit_copy_error;

	  for(iIndex=0; iIndex<6 ; iIndex++)
	    {
	      sprintf (strSection, "%s%d", INI_IMAGE_, iIndex+1);
	      GetSavedVar( applet_info[applet].name, strSection, strImgFile);
	      if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
		goto exit_copy_error;
	    }
	}
	break;
      case APPLET_ANVOXEL:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"voxel.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_TUNNEL3D:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"tunnel3d.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE_FILE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANWATER:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"AnWater.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE_FILE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_WARP:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"Warp.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE_FILE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_WORMHOLE:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"wormhole.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_GALAXY:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"galaxy.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_BOOKFLIP:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"bookflip.class"))
	    goto exit_copy_error;

	  for(iIndex=0; ; iIndex++)
	    {
	      sprintf (strSection, "%s%d", INI_IMAGE_, iIndex+1);
	      GetSavedVar( applet_info[applet].name, strSection, strImgFile);
	      if ((strlen (strImgFile) == 0)
		  || (strcmp (strImgFile, INI_NO) == 0))
		break;
	      if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
		goto exit_copy_error;
	    }
	}
	break;
      case APPLET_MOSAIC:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"mosaic.class"))
	    goto exit_copy_error;

	  for(iIndex=0; ; iIndex++)
	    {
	      sprintf (strSection, "%s%d", INI_IMAGE_, iIndex+1);
	      GetSavedVar( applet_info[applet].name, strSection, strImgFile);
	      if ((strlen (strImgFile) == 0)
		  || (strcmp (strImgFile, INI_NO) == 0))
		break;
	      if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
		goto exit_copy_error;
	    }

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if(strcmp(strImgFile, INI_NO) != 0 )
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_ANFY3D:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"anfy3d.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"anfy3ds0.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"anfy3ds1.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"Box01.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"Cone01.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"Sphere01.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"Sphere02.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_TEXTURE0, strImgFile);
	  if (strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	  
	  GetSavedVar( applet_info[applet].name, INI_TEXTURE1, strImgFile);
	  if (strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_TEXTURE2, strImgFile);
	  if (strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_PHONGMAP, strImgFile);
	  if (strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_GPHONGMAP, strImgFile);
	  if (strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_TSCROLLER:
	{
	  bNoAnfyClass=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath,"txtscroll.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if(strcmp(strImgFile, INI_NO) != 0 )
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_WOBBLER:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"wobbler.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_IMAGE, strImgFile);
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_WHEEL:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"wheelmenu.class"))
	    goto exit_copy_error;

	  GetSavedVar(applet_info[applet].name, INI_FONT, strTmp);
	  if( !MyCopyFile(strOldPath,szNewPath,strTmp) )
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if ((strcmp  (strImgFile, INI_NO) != 0) && (strlen (strImgFile) != 0))
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_TEXTURE_FILE, strImgFile);
	  if ((strcmp  (strImgFile, INI_NO) != 0) && (strlen (strImgFile) != 0))
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_MORPHMENU:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"morphmenu.class"))
	    goto exit_copy_error;

	  if( !MyCopyFile(strOldPath,szNewPath,"morphmenub.class"))
	    goto exit_copy_error;

	  if( !MyCopyFile(strOldPath,szNewPath,"morphmenuc.class"))
	    goto exit_copy_error;

	  GetSavedVar(applet_info[applet].name, INI_BACKFX, strTmp);
	  if (strcmp (strTmp, "mmgalaxy") == 0)
	    {
	      if( !MyCopyFile(strOldPath,szNewPath,"mmgalaxy.class"))
		goto exit_copy_error;
	    }
	  else
	    if (strcmp (strTmp, "mmblobs") == 0)
	      {
		if( !MyCopyFile(strOldPath,szNewPath,"mmblob.class"))
		  goto exit_copy_error;

		if( !MyCopyFile(strOldPath,szNewPath,"mmblobs.class"))
		  goto exit_copy_error;
	      }

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if ((strcmp (strImgFile, INI_NO) != 0) && (strlen (strImgFile) != 0))
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_TREEMENU:
	{
	  char strCaption[64];
	  guint32 ui;

	  if( !MyCopyFile(strOldPath,szNewPath,"treemenu.class"))
	    goto exit_copy_error;

	  for(ui=0; ;ui++)
	    {
	      sprintf (strCaption, "%s%d", INI_ICON_, ui);
	      GetSavedVar( applet_info[applet].name, strCaption, strImgFile);
	      if(strlen (strImgFile) == 0)
		break;
	      if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
		goto exit_copy_error;
	    }

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if (strcmp (strImgFile, INI_NO) != 0 && strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_ANFYCAM:
	{
	  if( !MyCopyFile(strOldPath,szNewPath,"anfycam.class"))
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANFYPAINT:
	{
	  bNoAnfyClass=TRUE;
	  bNoOverImg=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath,"anfypaint.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"anfypaintb.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"anfypaintc.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"anfypaintd.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"anfypainte.class"))
	    goto exit_copy_error;
	  if( !MyCopyFile(strOldPath,szNewPath,"anfypaintf.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if (strcmp (strImgFile, INI_NO) != 0 && strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_SOLIDSCROLLER:
	{
	  char strFontFile[1024], strFontGifFile[1024], *ptr;
	  bNoAnfyClass=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath,"solidscr.class"))
	    goto exit_copy_error;
			
	  GetSavedVar( applet_info[applet].name, INI_FONTINDEX, strFontFile);
	  if (strlen (strFontFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strFontFile) )
	      goto exit_copy_error;


	  strcpy (strFontGifFile, strFontFile);
	  ptr = strchr (strFontGifFile, '.');

	  if (ptr != NULL)
	    *ptr = 0;

	  strcat (strFontGifFile, ".gif");
	  if( !MyCopyFile(strOldPath,szNewPath,strFontGifFile) )
	    goto exit_copy_error;
	}
	break;
      case APPLET_ANFYSNOW:
	{
	  bNoAnfyClass=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath, "ansnow.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if (strcmp (strImgFile, INI_NO) != 0 && strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;
	}
	break;
      case APPLET_FIREWORKS:
	{
	  bNoAnfyClass=TRUE;

	  if( !MyCopyFile(strOldPath,szNewPath, "firewk.class"))
	    goto exit_copy_error;

	  if( !MyCopyFile(strOldPath,szNewPath, "firewks.class"))
	    goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_BACK_IMAGE_PATH, strImgFile);
	  if (strcmp (strImgFile, INI_NO) != 0 && strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	      goto exit_copy_error;

	  GetSavedVar( applet_info[applet].name, INI_AUDIOFILE, strTmp);
	  if (strcmp (strImgFile, INI_NO) != 0 && strlen (strImgFile) != 0)
	    if( !MyCopyFile(strOldPath,szNewPath,strTmp) )
	      goto exit_copy_error;
	}
	break;
    }

    if( !bNoAnfyClass )
      if( !MyCopyFile(strOldPath,szNewPath, "anfy.class"))
	goto exit_copy_error;
    if( !bNoLWareClass )
      if( !MyCopyFile(strOldPath,szNewPath, "Lware.class"))
	goto exit_copy_error;
    if( !bNoOverImg )
      {
	GetSavedVar(applet_info[applet].name,INI_OVERIMG, strImgFile);
	if (strcmp (strImgFile, INI_NO) != 0 && strlen (strImgFile) != 0)
	  if( !MyCopyFile(strOldPath,szNewPath,strImgFile) )
	    goto exit_copy_error;
      }
  }

  {
    char* jarFile=GetJarFile(applet);
    if( jarFile != NULL)
      if( !MyCopyFile(strOldPath,szNewPath,jarFile ) )
	goto exit_copy_error;
  }

  {
    char* szScrollTextFile=GetScrollTextFile(applet);
    if( szScrollTextFile != NULL)
      if( szScrollTextFile[0] != '\0' )
	if( !MyCopyFile(strOldPath,szNewPath,szScrollTextFile ) )
	  goto exit_copy_error;
  }

  return 1;

exit_copy_error:
  message_box (_("Error copying file"), 0);

  return 0;
}

